// FILE: ./assets/js/utilities/gopUtil.js
function gopUtil(ScreenService, General, templateManager, ValidationEngine, eventAggregator, ZHPromise, restEngine, $rootScope, ZhFilterPreferences, ZhPreferences, Query, metaDataFactory, maskEngine, MessageService) {

    this.openPopupParametro = function(widget) {
        var filterSelect = templateManager.container.getWidget('filterSelect');
        ScreenService.openPopup(filterSelect);
        if(widget){
            if(widget.currentRow.CDFILIAL){
                filterSelect.currentRow.CDFILIAL = widget.currentRow.CDFILIAL;
                filterSelect.currentRow.NMFILIAL = widget.currentRow.NMFILIAL;
            }
        }
    };

    this.adiciona = function(widget) {
        // ScreenService.openSwipe(widget);
        widget.newRow();
    };

    this.cancela = function(widget) {
        if (widget.currentRow.__is_new) {
			widget.list(true);
			widget.dataSource.data.pop();
			widget.currentRow = {};
		    ScreenService.goBack();
		} else if (widget.mode === 'edit') {
			widget.view();
			for (var i in widget.backupRow) {
				widget.currentRow[i] = widget.backupRow[i];
			}
		}
        widget.reload();
    };

    this.exclui = function(widget, msg, confirmMsg, modes) {
     
        if (widget.getCheckedRows().length == 0) {
            this.deleteCrud(widget, msg, confirmMsg, modes).then(function() {
                widget.reload();
            }.bind(this), function(erro) {
                ScreenService.showMessage(erro);
            });
        }else {
            this.deleteCrudCheckedRows(widget, msg, confirmMsg, modes).then(function() {
                widget.reload();
            }.bind(this), function(erro) {
                ScreenService.showMessage(erro);
            });
        }
    };

    this.edita = function(widget) {
        widget.backupRow = angular.copy(widget.currentRow);
        widget.edit();
    };

    this.setStripeLabel = function(value) {
        var stripeWidget = templateManager.container.getWidget("stripeWidget");
        stripeWidget.getField("UNIDADE").label = "Unidade: " + value;
    };

    this.habilitaCampoFilial = function(widget) {

        if (widget.currentRow.TPPARAMETRIZACAO == 'P') {
            widget.getField("NMFILIAL").isVisible = false;
            widget.currentRow.CDFILIAL = "";
            widget.currentRow.NMFILIAL = "";
        }
        else {
            widget.getField("NMFILIAL").isVisible = true;
        }
        widget.currentRow.CDFILIAL = widget.currentRow.CDFILIAL ? widget.currentRow.CDFILIAL : "";
        widget.currentRow.NMFILIAL = widget.currentRow.NMFILIAL ? widget.currentRow.NMFILIAL : "";
    };

    this.buscaParametrizacao = function(widget, mainWidget) {
        
        if (widget.isValid()) {

            var geralWidget = templateManager.container.getWidget(mainWidget);

            var textoStripe = "";
            var filtro = "";

            if (!widget.currentRow.CDFILIAL) {
                textoStripe += "Parametrização Padrão";
            }
            else {
                textoStripe += widget.currentRow.CDFILIAL + " - " + widget.currentRow.NMFILIAL;
                filtro = widget.currentRow.CDFILIAL;
            }

            geralWidget.dataSourceFilter = [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": filtro
            }];

            this.setStripeLabel(textoStripe);
            if(ScreenService.openSwipe){
                ScreenService.closeSwipe();
            }
            ScreenService.closePopup();
            geralWidget.reload();
            geralWidget.activate();
        }

    };

    this.setFilterSelectValues = function() {
        var filterSelectWidget = templateManager.container.getWidget('filterSelect');
        filterSelectWidget.applyDefaultValue();

        var filterSelectWidgetRow = filterSelectWidget.currentRow;
        filterSelectWidgetRow.TPPARAMETRIZACAO = "F";
        filterSelectWidgetRow.NMFILIAL = getLocalVar("NMFILIAL");
        filterSelectWidgetRow.CDFILIAL = getLocalVar("CDFILIAL");


        this.habilitaCampoFilial(filterSelectWidget);

    };

    this.getWidgetContainer = function(widgetName) {
        return templateManager.container.getWidget(widgetName);
    };

    this.aplicaParametrizacaoPadrao = function(mainWidget, geralWidget) {
        var CDFILIAL = geralWidget.currentRow.CDFILIAL;
        var flag = geralWidget.currentRow.IDUTPADRROPFIL === "S";

        if (CDFILIAL != 'P') {
            this.disableChildWidget(flag, mainWidget);
        }
        else {
            this.disableChildWidget(false, mainWidget);
        }
    };

    this.disableChildWidget = function(boolean, mainWidget) {
        if(!Util.isEmptyOrBlank(mainWidget.widgets)){
            mainWidget.widgets.forEach(function(childWidget) {
                this.disableField(boolean, childWidget.fields);
                this.disableAction(!boolean, childWidget.actions);
                if(!Util.isEmptyOrBlank(childWidget.widgets)){
                    childWidget.widgets.forEach(function(childWid) {
                        this.disableField(boolean, childWid.fields);
                        this.disableAction(!boolean, childWid.actions);
                    }.bind(this));  
                }
            }.bind(this));    
        }
    };
    
    this.disableField = function(boolean, fields){
        if(fields){
            fields.forEach(function(field) {
                field.readOnly = boolean;
                if (field.label.indexOf("Código") != -1 || field.name == "SGIMPOSTO") {
                    field.readOnly = true;
                }
            }); 
        }
    };
    
    this.disableAction = function(boolean, actions){
        if(actions){
            actions.forEach(function(action){
                if(action.name.indexOf("ACTIVE") != -1 || action.name.indexOf("EDIT") != -1 || action.name.indexOf("DEL") != -1 ){
                    action.isVisible = boolean;
                }
            });
        }
    };
   
    this.validations = function() {

        var percentValidation99 = function(value, validationAttrs) {
            var retorno;
            
            value = this.parseMaskStringToFloat(value);
            if (value < 0 || value > 100) {
                retorno = {
                    "valid": false,
                    "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("O valor deve estar entre 0 e 100")
                };
               
            }
            else {
                //var numberSizeReceived = value.replace(/[^0-9]/g,'').length;
                 retorno = {
                    "valid": true
                };
            }
            return retorno;
        }.bind(this);
        
        var percentValidationFrom1To99 = function(value, validationAttrs) {
            var retorno;
            
            value = this.parseMaskStringToFloat(value);
            if ((value >= 1) && (value <= 100)) {
                retorno = {
                    "valid": true
                };
            }
            else {
                retorno = {
                    "valid": (value >= 1 && value <= 100),
                    "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Porcentagem inválida")
                };
            }
            return retorno;
        }.bind(this);
        
        var dayValidation = function(value, validationAttrs) {

            var retorno;

            if (Util.isEmptyOrBlank(value)) {
                retorno = {
                    "valid": true
                };
            }else{
                if (value >= 1 && value <= 31) {
                    retorno = {
                        "valid": true
                    };
                }
                else {
                    retorno = {
                        "valid": (value >= 1 && value <= 31),
                        "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Dia inválido")
                    };
                }
            }
            return retorno;
        };
        
        var yearValidation = function(value, validationAttrs) {

            var retorno;
            if ((value >= 1900 && value <= 2125) || Util.isEmptyOrBlank(value)) {
                retorno = {
                    "valid": true
                };
            }
            else {
                retorno = {
                    "valid": false,
                    "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Ano inválido")
                };
            }
            return retorno;
        };
        
        var min5CaracterValidation = function(value, validationAttrs) {

            var retorno;
            if (value.length > 4 && !Util.isEmptyOrBlank(value)) {
                retorno = {
                    "valid": true
                };
            }
            else {
                retorno = {
                    "valid": (value.length > 4 && !Util.isEmptyOrBlank(value)),
                    "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Mínimo de 5 caracteres")
                };
            }
            return retorno;
        };
        
        var negativeValidation = function(value, validationAttrs) {

            var retorno;
            
            if ((value > 0 || Util.isEmptyOrBlank(value) || isNaN(value) && value.indexOf("-") == -1)){
                retorno = {
                    "valid": true
                };
            }
            else {
                retorno = {
                    "valid": ((value > 0 || Util.isEmptyOrBlank(value) || isNaN(value) && value.indexOf("-") == -1)),
                    "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Campo negativo")
                };
            }
            return retorno;
        };
        
        var negativeValidationBigger0 = function(value, validationAttrs) {

            var retorno;
            
            if ((value >= 0 || Util.isEmptyOrBlank(value) || isNaN(value) && value.indexOf("-") == -1)){
                retorno = {
                    "valid": true
                };
            }
            else {
                retorno = {
                    "valid": ((value >= 0 || Util.isEmptyOrBlank(value) || isNaN(value) && value.indexOf("-") == -1)),
                    "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Campo negativo")
                };
            }
            return retorno;
        };
        
        var number = function(value, validationAttrs){
            var regExp = new RegExp("[0-9]", "gi");
            var retorno;

            if (Util.isEmptyOrBlank(value)) {
                retorno = {
                    "valid": true
                };
            }else{
                if (regExp.test(value)) {
                    retorno = {
                        "valid": true
                    };
                }
                else {
                    retorno = {
                        "valid": (regExp.test(value)),
                        "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Campo numérico")
                    };
                }
            }
            return retorno;
        };

        ValidationEngine.registerValidation('isValidPercent99', percentValidation99);
        ValidationEngine.registerValidation('percentValidationFrom1To99', percentValidationFrom1To99);
        ValidationEngine.registerValidation('isValidDay', dayValidation);
        ValidationEngine.registerValidation('isValidYear', yearValidation);
        ValidationEngine.registerValidation('isNegative', negativeValidation);
        ValidationEngine.registerValidation('min5Caracteres', min5CaracterValidation);
        ValidationEngine.registerValidation('isNegativeBigger0', negativeValidationBigger0);
    };

    eventAggregator.subscribe('authSuccess', function(session) {
        this.validations();
    }.bind(this));
    
    
    this.buscaColunas = function(nrItem){
      var obj = [];
      obj[1] =  {"Columns":["CDIMPOSTO", "NMIMPOSTO"],"DataSource": "gop_parametro#/crud/gop21000_item_detalhe_imposto"};
      obj[2] =  {"Columns":["CDPRODUTO", "CDARVPROD", "NMPRODUTO"],"DataSource": "gop_parametro#/crud/gop21000_item_detalhe_produto"};
      obj[3] =  {"Columns":["CDORCAMENTOMO", "DSORCAMENTOMO"],"DataSource": "gop_parametro#/crud/gop21000_item_detalhe_orcamentomo"};
      //obj[4] =  {"Columns":["", "", "", ""],"DataSource": ""}; Taxa Administrativa
      obj[5] =  {"Columns":["CDDESPPREV", "NMDESPPREV"],"DataSource": "gop_parametro#/crud/gop21000_item_detalhe_despprev"};
      obj[6] =  {"Columns":["CDDESPPREV", "NMDESPPREV"],"DataSource": "gop_parametro#/crud/gop21000_item_detalhe_despprev"};
      obj[7] =  {"Columns":["CDDESPPREV", "NMDESPPREV"],"DataSource": "gop_parametro#/crud/gop21000_item_detalhe_despprev"};
      obj[71] = {"Columns":["CDTIPOOPER", "IDENTRSAIDOP", "NMTIPOOPER"],"DataSource": "gop_parametro#/crud/gop21000_item_detalhe_tipooper"};
      obj[8] =  {"Columns":["CDIMPOSTO", "NMIMPOSTO"],"DataSource": "gop_parametro#/crud/gop21000_item_detalhe_imposto"};
      obj[9] =  {"Columns":["CDIMPOSTO", "NMIMPOSTO"],"DataSource": "gop_parametro#/crud/gop21000_item_detalhe_imposto"};
      obj[10] = {"Columns":["CDINVEST", "NMINVEST"],"DataSource": "gop_parametro#/crud/gop21000_item_detalhe_investimento"};
      //obj[11] = {"Columns":["", "", "", ""],"DataSource": ""};Depreciação
      //  5	Despesas da Matriz
      //  6	Despesas Diversas
      //  7	Despesas Lanc. Entrada
      return obj[nrItem];
    };
    
    this.buscaColunasUnidade = function(nrItem){
      var obj = [];
      obj[1] =  {"Columns":["CDIMPOSTO", "NMIMPOSTO"],"DataSource": "gop_parametro#/crud/gop21000_unidade_item_detalhe_imposto"};
      obj[2] =  {"Columns":["CDPRODUTO", "CDARVPROD", "NMPRODUTO"],"DataSource": "gop_parametro#/crud/gop21000_unidade_item_detalhe_produto"};
      obj[3] =  {"Columns":["CDORCAMENTOMO", "DSORCAMENTOMO"],"DataSource": "gop_parametro#/crud/gop21000_unidade_item_detalhe_orcamentomo"};
      //obj[4] =  {"Columns":["", "", "", ""],"DataSource": ""}; Taxa Administrativa
      obj[5] =  {"Columns":["CDDESPPREV", "NMDESPPREV"],"DataSource": "gop_parametro#/crud/gop21000_unidade_item_detalhe_despprev"};
      obj[6] =  {"Columns":["CDDESPPREV", "NMDESPPREV"],"DataSource": "gop_parametro#/crud/gop21000_unidade_item_detalhe_despprev"};
      obj[7] =  {"Columns":["CDDESPPREV", "NMDESPPREV"],"DataSource": "gop_parametro#/crud/gop21000_unidade_item_detalhe_despprev"};
      obj[71] = {"Columns":["CDTIPOOPER", "NMTIPOOPER"],"DataSource": "gop_parametro#/crud/gop21000_unidade_item_detalhe_tipooper"};
      obj[8] =  {"Columns":["CDIMPOSTO", "NMIMPOSTO"],"DataSource": "gop_parametro#/crud/gop21000_unidade_item_detalhe_imposto"};
      obj[9] =  {"Columns":["CDIMPOSTO", "NMIMPOSTO"],"DataSource": "gop_parametro#/crud/gop21000_unidade_item_detalhe_imposto"};
      obj[10] = {"Columns":["CDINVEST", "NMINVEST"],"DataSource": "gop_parametro#/crud/gop21000_unidade_item_detalhe_investimento"};
      //obj[11] = {"Columns":["", "", "", ""],"DataSource": ""};Depreciação
      //  5	Despesas da Matriz
      //  6	Despesas Diversas
      //  7	Despesas Lanc. Entrada
        if(obj[nrItem]){
            return obj[nrItem];
        }else{
            return undefined;
        }    
    };
    
    this.validaPercentual = function(valor){
        if(valor < 0 || valor > 100){
            ScreenService.showMessage("O percentual deve estar entre 0 e 100");
            return false;
        }
        return true;
    };
    
     this.getPeriodo = function(idPeriodo){
        var periodo = [];
        
        periodo.MENSAL =  this.getDataMensal();
        periodo.BIMESTRAL = this.getDataBimestral();
        periodo.TRIMESTRAL = this.getDataTrimestral();
        periodo.QUADRIMESTRAL = this.getDataQuadrimestral();
        periodo.SEMESTRAL = this.getDataSemestral();
        periodo.ANUAL = this.getDataMensal();
       
        if(periodo[idPeriodo]){
            return periodo[idPeriodo];
        }else{
            return [];
        }    
    };

    this.getDataMensal = function(){
        return [
			{
				"name": "Janeiro",
				"value": "01",
				"mesInicial": "01"
			},
			{
				"name": "Fevereiro",
				"value": "02",
				"mesInicial": "02"
			},
			{
				"name": "Março",
				"value": "03",
				"mesInicial": "03"
			},
			{
				"name": "Abril",
				"value": "04",
				"mesInicial": "04"
			},
			{
				"name": "Maio",
				"value": "05",
				"mesInicial": "05"
			},
			{
				"name": "Junho",
				"value": "06",
				"mesInicial": "06"
			},
			{
				"name": "Julho",
				"value": "07",
				"mesInicial": "07"
			},
			{
				"name": "Agosto",
				"value": "08",
				"mesInicial": "08"
			},
			{
				"name": "Setembro",
				"value": "09",
				"mesInicial": "09"
			},
			{
				"name": "Outubro",
				"value": "10",
				"mesInicial": "10"
			},
			{
				"name": "Novembro",
				"value": "11",
				"mesInicial": "11"
			},
			{
				"name": "Dezembro",
				"value": "12",
				"mesInicial": "12"
			}
		];

    };
    
    this.getDataBimestral = function(){
        return [
            {
                "name": "1º Bimestre",
                "value": "1",
                "mesInicial": "01"
            },
            {
                "name": "2º Bimestre",
                "value": "2",
                "mesInicial": "03"
            },
            {
                "name": "3º Bimestre",
                "value": "3",
                "mesInicial": "05"
            },
            {
                "name": "4º Bimestre",
                "value": "4",
                "mesInicial": "07"
            },
            {
                "name": "5º Bimestre",
                "value": "5",
                "mesInicial": "09"
            },
            {
                "name": "6º Bimestre",
                "value": "6",
                "mesInicial": "11"
            }
        ];

        
    };
    
    this.getDataTrimestral = function(){
        return [
            {
                "name": "1º Trimestre",
                "value": "1",
                "mesInicial": "01"
            },
            {
                "name": "2º Trimestre",
                "value": "2",
                "mesInicial": "04"
            },
            {
                "name": "3º Trimestre",
                "value": "3",
                "mesInicial": "07"
            },
            {
                "name": "4º Trimestre",
                "value": "4",
                "mesInicial": "10"
            }
        ];

    };
    
    this.getDataQuadrimestral = function(){
        return [
            {
                "name": "1º Quadrimestre",
                "value": "1",
                "mesInicial": "01"
            },
            {
                "name": "2º Quadrimestre",
                "value": "2",
                "mesInicial": "05"
            },
            {
                "name": "3º Quadrimestre",
                "value": "3",
                "mesInicial": "09"
            }

        ];

    };
    
    this.getDataSemestral = function(){
        return [
            {
                "name": "1º Semestre",
                "value": "1",
                "mesInicial": "01"
            },
            {
                "name": "2º Semestre",
                "value": "2",
                "mesInicial": "07"
            }

        ];

    };
    
    this.delimitCaracter = function(args, limit){
        if(!Util.isEmptyOrBlank(args.owner.field.value())){
            var ano = args.owner.field.value().toString();
            if(ano.length > limit){
                args.owner.field.setValue(parseInt(ano.substr(0, limit),10));
            } 
        }
    };
    
    this.getDiaFinalMes = function(mes, ano){
        if(mes == 4 || mes == 6 || mes == 9 || mes == 11){
            return 30;
        }else if (mes == 2){
            if(ano%4 == 0){
                return 29;
            }else{
                return 28;
            }
        }else{
            return 31;
        }
    };
        
    this.underBlockWidget = function(args){
      $('.swipe-tip').before('<div style="position: absolute; background: #000;left: 0;top: 0;bottom: 0;right: 0;opacity: 0.4;transform: translate(-100%, 0);-webkit-transform: translate(-100%, 0);"></div>');  
    };
     
    
    /*
        *  
        * Função semelhante ao saveCrud porém não fecha o swipe, permanecendo no view.
        *
    */
    this.saveCrudBackToView = function (widget) {
		var deferred = ZHPromise.defer();
		if (widget.isValid()) {

			var paramsNewCode = widget.paramsNewCode;
			if (paramsNewCode && paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = paramsNewCode.__table_pk;
				widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
				widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
				widget.currentRow.__columns_to_return = paramsNewCode.__columns_to_return;
			}

			var fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
			if (fieldsToValidateDuplication && fieldsToValidateDuplication.length){
				General.setDuplicationValidateParamsToCurrentRow(widget);
			}

			widget.saveRow().then(function (success) {
				widget.activate();
				if (widget.currentRow.__is_new) {
				    // widget.currentRow = Object.assign(success.dataset.columnsToReturn[0], widget.currentRow);
				    success.dataset.columnsToReturn.forEach(function(column){
                    	Object.assign(widget.currentRow, column);
                    });
					widget.reload().then(function (callback) {
					    widget.view();
						deferred.resolve(callback);
					});
				} else {
					widget.computeDescriptionOrder();
					widget.view();
					deferred.resolve(success);
				}
				widget.currentRow.__is_new = false;
			}.bind(this), function (error) {
				var errorMessage = 'TIMEOUT';
				if (error.data) {
					errorMessage = error.data.error;
				}
				deferred.reject(errorMessage);
			});
		} else {
			// @todo create messages class
			deferred.reject("Preencha todos os campos corretamente!");
		}

		return deferred.promise;
	};
	
	this.deleteCrudCheckedRows = function (widget, msg, confirmMsg, modes) {
        var deferred = ZHPromise.defer();
        if(Util.isEmptyOrBlank(confirmMsg)){
            confirmMsg = ScreenService.i18n('Apagar os registros selecionados?');
        }
        ScreenService.confirmMessage(confirmMsg, 'question', function (success) {
            widget.removeCheckedRows().then(function () {
                widget.reload();
                widget.unSelectAll();
                deferred.resolve();
                if(!Util.isEmptyOrBlank(modes)){
				        if(modes[0] == "list")
				            widget.list();
				        else if(modes[0] == "view")
				            widget.view();
				    }
            }, function (error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    if(Util.isEmptyOrBlank(msg)){
                        msg = 'EXCLUSAO_NAO_PERMITIDA';
                    }
                    MessageService.showMessageByCode('EXCLUSAO_NAO_PERMITIDA');
                } else {
                    ScreenService.showMessage(error.data.error);
                }
                deferred.reject();
                if(!Util.isEmptyOrBlank(modes)){
				        if(modes[1] == "list")
				            widget.list();
				        else if(modes[1] == "view")
				            widget.view();
				    }
            });
        });
        return deferred.promise;
    };
    
    this.deleteCrud = function (widget, msg, confirmMsg, modes) {
		var deferred = ZHPromise.defer();
		if(Util.isEmptyOrBlank(confirmMsg)){
            confirmMsg = ScreenService.i18n('Apagar registro?');
        }
		ScreenService.confirmMessage(confirmMsg, 'question', function (success) {
			widget.removeRow().then(function () {
				ScreenService.closeSwipe().then(function () {
					widget.activate();
					deferred.resolve();
				    if(!Util.isEmptyOrBlank(modes)){
				        if(modes[0] == "list")
				            widget.list();
				        else if(modes[0] == "view")
				            widget.view();
				    }
				});
			}, function (error) {
				if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
				    if(Util.isEmptyOrBlank(msg)){
                        
                        msg = 'EXCLUSAO_NAO_PERMITIDA';
                        
                    }
                    MessageService.showMessageByCode('EXCLUSAO_NAO_PERMITIDA');
				} else {
					ScreenService.showMessage(error.data.error);
				}
				deferred.reject();
				if(!Util.isEmptyOrBlank(modes)){
				        if(modes[1] == "list")
				            widget.list();
				        else if(modes[1] == "view")
				            widget.view();
				    }
			});
		});
		return deferred.promise;
	};
    
    this.maskThousand = function(valor){
        var mensagem;
        if(valor.toString().indexOf(".") != -1){
	        mensagem = valor.toString().split(".");
	        mensagem[0] = General.valueToFloat(mensagem[0]);
    	    mensagem[0] = mensagem[0].toLocaleString();
    	    for(var i = 0; i<8; i++){
    	        if(mensagem[0].indexOf(",") != -1){
    	            mensagem[0] = mensagem[0].replace(",", ".");
    	        }
    	    }
    	    mensagem = mensagem.join(",");
	    }else{
	        mensagem = valor.toLocaleString();
	        for(var y = 0; y<8; y++){
    	        if(mensagem.indexOf(",") != -1){
    	            mensagem = mensagem.replace(",", ".");
    	        }
    	    }
    	    mensagem+=",00";
	    }
	    return mensagem;
    };
    
    this.includeCodeZero = function(value, size) {
        var dif = size - value.length;
        var zeros = "";
        for (var x = 0; x < dif; x++) {
            zeros += "0";
        }
        return zeros + value;
    };
    
    this.parseMaskStringToFloat = function(string){
        if(typeof string == "string"){
            for(var i = 0; i < (parseInt(string.length / 3) - 1); i++){
                string = string.replace(".", "");
            }
            string = string.replace(",", ".");    
        }
        return parseFloat(string);
    };
    
    this.initialUpperCase = function(string){
        var texto = string.toLowerCase();
        var array = texto.split("");
        
        array[0] = array[0].toUpperCase();
        
        texto = array.join("");
        
        if(texto == "Marco")
            texto = "Março";
        
        return texto;
    };
    
    this.backFilterCloneBackupRow = function(widget){
        if(widget.backUpRow)
            widget.currentRow = Util.clone(widget.backUpRow);
        ScreenService.closePopup();
    };
    
    this.checkPeriodo = function(filterSelect){
        if(filterSelect.currentRow.IDPERIODOORCA == "ANUAL"){
            filterSelect.getField("NRPERIODOORCA").validations = {};
            filterSelect.getField("NRPERIODOORCA").readOnly = true;
        }else{
            filterSelect.getField("NRPERIODOORCA").validations = { "required" : true};
            filterSelect.getField("NRPERIODOORCA").readOnly = false;
        }
    };
    
    this.openFilterSelect = function(){
        var filterSelect = this.getWidgetContainer('filterSelect');
        ScreenService.openPopup(filterSelect);
    };
    
    this.openPopupFiltroPeriodo = function(){
        var filterSelect = this.getWidgetContainer('filterSelect');
         
        restEngine.post('gop#/crud/paramorcaoperac', { "requestType": "FilterData",
                                                        "filter": ""
        }).then(function(retorno){
                                                            
            filterSelect.currentRow.IDPERIODOORCA = retorno.dataset.paramorcaoperac[0].IDPERIODOORCA;
            filterSelect.currentRow.IDORCDESPENTROP = retorno.dataset.paramorcaoperac[0].IDORCDESPENTROP;
            filterSelect.getField("NRPERIODOORCA").dataSource.data = this.getPeriodo(retorno.dataset.paramorcaoperac[0].IDPERIODOORCA);
            filterSelect.currentRow.IDPERIODOORCA = retorno.dataset.paramorcaoperac[0].IDPERIODOORCA;
            
            this.checkPeriodo(filterSelect);
            ScreenService.openPopup(filterSelect);
        }.bind(this));
    };

    //Filter
    ZhFilterPreferences.oldApplyFilter = ZhFilterPreferences.applyFilter;
    ZhFilterPreferences.applyFilter = function (widget, filter, reloadWidget) {
        return ZhFilterPreferences.oldApplyFilter(widget, filter, reloadWidget).then(function (widgetPromiseReturn) {
           if (reloadWidget !== false) {
               updateStripe(widget);
           }
        }.bind(this));
    };


    var updateStripe = function (gridWidget) {
        return handleStripe(gridWidget).then(function () {
            var filterInfo = gridWidget.__filterInfoStripe;
            deleteUsedFilterParameterOnStripeFields(filterInfo);
            setFieldsByFilterOnStripe(gridWidget._filterInfo, filterInfo);
            hideFieldsNotUsedOnStripe(filterInfo);
            activateCollapseButtonByVisibleFieldsOnStripe(filterInfo);
        });
    };
    this.updateStripe = updateStripe;

    /**
     * Deleta o parâmetro que indica quando uma field do stripe
     * já foi usada para exibir indormações do filtro definido
     * (em todos os fields do stripe).
     * 
     * @param {object} stripe widgtet stripe.
     */
    function deleteUsedFilterParameterOnStripeFields(stripe) {

        stripe.fields.forEach(function (field) {

            if (field.hasOwnProperty("usedFiedlOnStripe")) {
                delete field.usedFiedlOnStripe;
            }

        });

    }

    /**
     * Define a rotina para exibir os valores filtrados
     * no stripe, onde "OBJECTS_BY_LABEL_ON_STRIPE" e a variável
     * que define o máximo de informações do filtro atual a serem
     * setadas por field.
     * 
     * @param {object} filterInfo informações do filtro
     * definido na widget.
     * 
     * @param {object} stripe widgtet stripe.
     */
    function setFieldsByFilterOnStripe(filterInfo, stripe) {

        if (!Util.isEmptyOrBlank(filterInfo)) {

            var validfilterInfo = getValidFilterInfo(filterInfo);

            var OBJECTS_BY_LABEL_ON_STRIPE = 3;
            var filterInfoLength = validfilterInfo.length;

            if (filterInfoLength > OBJECTS_BY_LABEL_ON_STRIPE) {

                buildStripeField(validfilterInfo.slice(0, 3), stripe);
                setFieldsByFilterOnStripe(validfilterInfo.slice(3), stripe);

            } else if (filterInfoLength > 0) {
                buildStripeField(validfilterInfo, stripe);
            }

        }

    }

    /**
     * Retonar um array com os valores válidos
     * a serem definidos no stripe, ou seja, retorna
     * apenas as informações que possuem label.
     * 
     * @param {object} filterInfo informações do filtro
     * definido na widget.
     */
    function getValidFilterInfo(filterInfo) {

        let labelsToExclude = ["TODAS", "TODOS"];
        let validFilterInfo = [];
        if (isSavedFilter(filterInfo)) {
            validFilterInfo.push({
                label: "Filtro Aplicado",
                value: filterInfo[0].filterName
            });
        } else {
            validFilterInfo = filterInfo.filter(function (filterInfo) {
                let valueIsFilled = (!Util.isEmptyOrBlank(filterInfo.value));
                let labelIsFilled = (!Util.isEmptyOrBlank(filterInfo.label));
    
                let upperCaseLabel = filterInfo.label.toUpperCase();
                let labelIsNotInExcludeList = (labelsToExclude.indexOf(upperCaseLabel) == -1);
    
                return (valueIsFilled && labelIsFilled && labelIsNotInExcludeList);
            });
        }
        return validFilterInfo;
    }

    function isSavedFilter(filterInfo) {
        return filterInfo.length && filterInfo[0] && filterInfo[0].filterName;
    }

    /**
     * Cria um field, ou utiliza um já existente,
     * para ser definido no stripe.
     * 
     * @param {object} filterInfo informações do filtro
     * definido na widget.
     * 
     * @param {object} stripe widgtet stripe.
     */
    function buildStripeField(filterInfo, stripe) {

        var stripeFields = stripe.fields;
        var availableFieldIndex = getAvailableFieldIndex(stripeFields);

        if (availableFieldIndex !== undefined) {

            var stripeField = stripeFields[availableFieldIndex];
            var fieldLabel = buildStripeFielLabelByFilterInfo(filterInfo, stripe);
            stripeField.label = fieldLabel;

            if (stripeField.isVisible === false) {
                stripeField.isVisible = true;
            }

        } else {
            stripeFields.push(buildFieldForStripe(filterInfo, stripe));
        }

    }

    /**
     * Retonar o index de um field que ainda não foi utilizado,
     * no stripe, para exibir valores do filtro.
     * 
     * @param {object} stripeFields fields existentes no stripe.
     */
    function getAvailableFieldIndex(stripeFields) {

        if (stripeFields !== undefined && stripeFields.length > 0) {
            for (var i = 0; i < stripeFields.length; i++) {
                if (!Boolean(stripeFields[i].usedFiedlOnStripe)) {
                    stripeFields[i].usedFiedlOnStripe = true;
                    return i;
                }
            }
        }
        return undefined;
    }

    /**
     * Cria um field para exibir informações do filtro
     * no stripe.
     * 
     * @param {object} filterInfo informações do filtro
     * definido na widget.
     * 
     * @param {object} stripe widgtet stripe.
     */
    function buildFieldForStripe(filterInfo, stripe) {

        var fieldLabel = buildStripeFielLabelByFilterInfo(filterInfo);

        var field = {
            name: "STRIPE_FIELD",
            label: fieldLabel,
            widget: stripe,
            usedFiedlOnStripe: true
        };

        return metaDataFactory.fieldFactory(field);

    }

    /**
     * Cria o texto que será definido na label de um field
     * no stripe.
     * 
     * @param {object} filterInfo informações do filtro
     * que serão utilizadas para montar o texto.
     */
    function buildStripeFielLabelByFilterInfo(filterInfo) {

        var fieldLabel;

        if (filterInfo.length === 1) {
            fieldLabel = ScreenService.i18n(filterInfo[0].label) + ": " + filterInfo[0].value;
        } else {

            fieldLabel = filterInfo.reduce(function (acc, cur) {
                return acc + ScreenService.i18n(cur.label) + ": " + cur.value + "   •   ";
            }, "");
            fieldLabel = fieldLabel.replace(/•([^•]*)$/, '$1');

        }

        return fieldLabel;

    }

    /**
     * Oculta fields que não são usandos no stripe.
     * 
     * @param {object} stripe widget stripe.
     */
    function hideFieldsNotUsedOnStripe(stripe) {

        stripe.fields.forEach(function (field) {

            if (!field.hasOwnProperty("usedFiedlOnStripe")) {
                field.isVisible = false;
            }

        });

    }

    function activateCollapseButtonByVisibleFieldsOnStripe(stripe) {

        var countStripeVisibleFields = 0;
        var NUMBER_OF_FIELDS_TO_SHOW_COLLAPSE_BUTTON = 3;
        var stripeFields = stripe.fields;

        stripeFields.forEach(function (field) {

            if (field.isVisible) {
                countStripeVisibleFields++;
            }

        });

        if (countStripeVisibleFields >= NUMBER_OF_FIELDS_TO_SHOW_COLLAPSE_BUTTON) {
            stripe.collapsible = true;
        } else {
            stripe.collapsible = false;
        }

    }

    var handleStripe = function (gridWidget) {
        var deferred = ZHPromise.defer();
        if (!gridWidget.__filterInfoStripe) {
            var stripeName = gridWidget.stripeToShowGridFilterInfo;
            if (stripeName) {
                var stripeWidget = templateManager.container.getWidget(stripeName);
                if (stripeWidget) {
                    if (~stripeWidget.template.indexOf("stripe.html")) {
                        var stripe = gridWidget.__filterInfoStripe = stripeWidget;
                        stripe.__stripeOriginalFields = stripe.fields;
                        factoryStripeFields([], gridWidget).then(function (fields) {
                            stripe.fields = fields;
                            deferred.resolve();
                        });
                    } else {
                        console.error("ZhGridFilter: Widget " + stripeName + " is not a Stripe");
                        deferred.reject();
                    }
                } else {
                    console.error("ZhGridFilter: There is no widget named " + stripeName);
                    deferred.reject();
                }
            }
        } else {
            deferred.resolve();
        }
        return deferred.promise;
    };

    var factoryStripeFields = function (fields, gridWidget) {
        var deferred = ZHPromise.defer();
        var stripe = gridWidget.__filterInfoStripe;
        fields = stripe.__stripeOriginalFields ? fields.concat(stripe.__stripeOriginalFields) : fields;
        var fieldsMap = fields.map(function (field) {
            var factoredField = metaDataFactory.fieldFactory(field);
            factoredField.widget = stripe;
            return factoredField;
        });
        deferred.resolve(fields);
        return deferred.promise;
    };

    this.startFilter = function (widget, currentRow) {
        return ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function (result) {
            if (result && result.DEFAULT_FILTER) {
                ZhPreferences.getPreferences('filter', Query.build().where('WIDGET_ID').equals(widget.id)).then(function (filters) {
                    var defaultFilter = filters.filter(function (filter) {
                        return filter.ID == result.DEFAULT_FILTER;
                    })[0];
                    var filter = {
                        ID: defaultFilter.LABEL_TO_PREFERENCE || 'EDITED',
                        FILTER_ROW: JSON.parse(defaultFilter.FILTER_ROW)
                    };

                    currentRow = filter.FILTER_ROW;
                    
                    ZhFilterPreferences.applyFilter(widget, filter).then(function () {
                        widget.appliedFilter = widget.appliedFilter || filter;
                        widget.appliedFilter.ID = result.DEFAULT_FILTER;
                        if (defaultFilter.LABEL_TO_PREFERENCE) {
                            widget._filterInfo[0].value = defaultFilter.LABEL_TO_PREFERENCE;
                        }
                    });
                });
            } else {
                if (Util.isUndefinedOrNull(currentRow) || Util.isEmptyObject(currentRow)) {
                    currentRow = buildFilterCurrentRow(widget);
                }
            }
            ZhFilterPreferences.openFilterWidget(widget, currentRow);
        });
    };

    var buildFilterCurrentRow = function(widget) {
        // let filterFields = getFilterFields(widget);
        return getFilterRowWithDefaultValues(widget.fields);
    };

    // var getFilterFields = function(widget) {
    //     if (Array.isArray(widget.fields)) {
    //         return widget.fields.filter(field => {
    //             return (
    //                 (Util.getNestedProperty(field, ["filterProperties", "isVisible"]) !== false && field.isVisible) &&
    //                 (Util.getNestedProperty(field, ["filterProperties", "showOnForm"]) !== false && field.showOnForm)
    //             );
    //         });
    //     } else {
    //         return [];
    //     }
    // };

    var getFilterRowWithDefaultValues = function (filterFields) {
        let filterRow = {};
        filterFields.forEach(field => {
            let defaultValue = Util.getNestedProperty(field, ["filterProperties", "defaultValue"]) || field.defaultValue;
            if (!Util.isUndefinedOrNull(defaultValue)) {
                filterRow[field.name] = Util.evaluate(defaultValue);
            }
        });
        return filterRow;
    };
    
    this.validDate = function(dataInicial, dataFinal,  mensagem){
		if(!Util.isEmptyOrBlank(dataInicial) && !Util.isEmptyOrBlank(dataFinal)){
			var dateIni = moment(dataInicial, 'DD/MM/YYYY');
			var dateFin = moment(dataFinal, 'DD/MM/YYYY');

			var valid = dateIni <= dateFin;
			if(!valid){
			    if(!mensagem)
			        mensagem = ScreenService.i18n("A data inicial deve ser menor ou igual a data final");
			        
				ScreenService.showMessage(mensagem);
			}
			return valid;
		}
		return true;
	};
	
	this.toggleReadOnlySelect = function (checkboxField, selectField) {

		if (checkboxField.checked) {
			selectField.readOnly = true;
			selectField.validations = {};
			selectField.setValue('');
		} else {
			selectField.readOnly = false;
			selectField.validations = {
				required: true
			};
		}
	};
    
    this.enabledDisableWidgets = function(widget, flag){
        
        if(!Util.isEmptyOrBlank(widget.getParent())){
            widget.getParent().widgets.forEach(function(tab){ 
    			tab.enabled = flag; 
    		});
    		
    		if (!Util.isEmptyOrBlank(widget.getParent().getParent())){
    			widget.getParent().getParent().widgets.forEach(function(tab){
    				tab.enabled = flag; 
    			});
    			
    			if (!Util.isEmptyOrBlank(widget.getParent().getParent().getParent())){
        			widget.getParent().getParent().getParent().widgets.forEach(function(tab){
        				tab.enabled = flag; 
        			});	
        		}
    		}
        }
    };
    
     this.retiraCabecalhoReport = function (widget){
        var metadata = widget.beforeGenerateReport.arguments[0].metadata;
        metadata.filter = [];
    };
    
    /*
        *  
        * Função utilizada para esconder mensagem de 'Filtro aplicado' quando utilizando apenas filtro default.
        * Chamada atráves do events da widget: FilterAfterApply.
        *
    */
    this.hideFilterApplied = function(widget){
        if(!Util.isEmptyOrBlank(widget._filterInfo)){
            widget._filterInfo = widget._filterInfo.filter(a=>(a.value != ""));
            widget._filterInfo = widget._filterInfo.filter(a=>(a.value != "Original"));
        }
    };
    
    this.getRowTarget = function(args){
        var nossoTargetHtmlElement = args.e.target;
        var row = $(nossoTargetHtmlElement).closest('.zh-new-grid')[0].__scope.row;
        return row;
    };

    //Função justInteger que aceita ponto e vírgula
    this.justInteger = function(selector, retryCount) {
        if (!retryCount || retryCount > 5)
            retryCount = 5;
        if (!selector) {
            throw {
                "message": "Selector is not defined!"
            };
        }
        setTimeout(function() {
            var elem = $(selector);
            if (elem[0]) {
                var length = selector.split(',').length;
                for (var i = 0; i < length; i++) {
                    if (elem.hasOwnProperty(i)) {
                        elem[i].onkeypress = avoidNonInteger;
                        elem[i].onpaste = preventEvent;
                    }
                }
            } else {
                if (!retryCount) {
                    console.error("Selector not found!");
                } else {
                    retryCount--;
                    this.justInteger(selector, retryCount);
                }
            }
        }.bind(this), 500);
    };

    function avoidNonInteger(e) {
        var integerNumbers = [44, 46, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
        var numberCode = e.which;
        if (integerNumbers.indexOf(numberCode) < 0) {
            preventEvent(e);
        }
    }

    function preventEvent(e) {
        e.preventDefault();
    }
    
    /**
     * Função responsável por chamar a maskCurrency sempre que o 
     * usuário desfocar do campo onde ela é aplicada.
     * Função deve ser usada no evento FieldOnBlur, junto com a 
     * removeMask, essa no FieldOnFocus.
     * 
     * @param {Field} field - field.
     * @param {Char} decimal - Caráctere que será usado para a casa decimal.
     * @param {Char} thousand - Caráctere que será usado para as unidades de milhar.
     * @param {Number} precision - Quantidade de casas decimais.
     */

    this.callMaskCurrency = function(field, decimal, thousand, precision = 2) {
        var value = field.value();
        var params = {
            "decimal": decimal,
            "thousand": thousand,
            "precision": precision
        };
        field.value(this.maskCurrency(value, params));
    };

    var maskFunction = function(element, viewValue, params, callBack) {
        // do something
    };

   /**
     * Função responsável por aplicar uma máscara do type currency
     * em algum field.
     * 
     * @param {Value} value - value do field.
     * @param {Object} params - Objeto param da máscara com {decimal, thousand, precision}.
     */

    this.maskCurrency = function(value, params) {
        var thousands = Util.isEmptyOrBlank(params.thousands) ? params.thousand : params.thousands;
        if (!Util.isEmptyOrBlank(value)) {

            if (value[0] === '.') {
                value = '0' + '.' + value.split('.').join('');
            } else if (value[0] === ',') {
                value = '0' + ',' + value.split(',').join('');
            }
            value = numberFormatBr(value);
            var newValue = "";
            value = value.split('.').join('');
            var dec = value.split(',')[1];
            var mil = [];
            var int = value.split(',')[0];

            if (dec === undefined) {
                dec = "";
                dec = dec.padEnd(params.precision, "0");
                int = value;
            } else if (dec.length < params.precision) {
                dec = dec.padEnd(params.precision, "0");
            } else if (dec.length > params.precision) {
                dec = parseFloat('1.' + dec).toFixed(params.precision).toString().split('.')[1];
            }

            var counter = angular.copy(int.length / 3);

            for (var i = 1; i <= counter; i++) {
                if (i != counter)
                    mil.push(thousands + int.substr(-3));
                else
                    mil.push(int.substr(-3));
                int = int.substr(0, (int.length - 3));
            }

            if (int != "") mil.push(int);

            mil = mil.reverse().join('');
            newValue = mil + params.decimal + dec;
            var regex = '^(^([0-9]{1,3}\\' + thousands + ')((\\d{3}\\' + thousands + '{1})|(\\d{3}\\' + params.decimal + '\\d{2,}))*$)|^(\\d{0,3}\\' + params.decimal + '\\d{2,})*$';
            regex = new RegExp(regex, 'g');
            if (regex.test(newValue))
                return newValue;
            else
                return 0;
        }
    };

    maskEngine.registerMask('gopMaskCurrency', maskFunction, this.maskCurrency);

    /**
     * Função responsável por retirar os pontos da máscara no
     * field para o usuário digitar.
     * 
     * @param {Args} args - Args do field.
     */

    this.removeMask = function(args) {
        var field = args.owner.field;
        var value = field.value();
        if (/^(.*)(\.+).*(,+)(.*)$/gm.test(field.value())) {
            field.value(value.split('.').join(''));
        }
    };

    /**
     * Função que retira os '.' das casas de milhar dos números
     * e troca as ',' dos decimais por '.'
     * Dessa forma o campo fica compativel com o BD do oracle.
     * 
     * @param {String} field - Fields de valor numérico.
     */

    this.parseFloat = function(field) {
        if (/^(.*)(\.+).*(,+)(.*)$/gm.test(field)) {
            var loop = field.toString().split('.');
            for (var i = 0; i < loop.length - 1; i++) {
                field = field.toString().replace('.', '');
            }
        }
        if (/\,/g.test(field))
            field = field.toString().replace(',', '.');
        return parseFloat(field);
    };
    

    var numberFormatBr = function(value) {
        if (/^(.*)(\.+).*(,+)(.*)$/gm.test(value)) {
            var loop = value.toString().split('.');
            for (var i = 0; i < loop.length - 1; i++) {
                value = value.toString().replace('.', '');
            }
        }
        if (/^\d{1,}\.{1}\d*$/g.test(value))
            value = value.toString().replace('.', ',');
        return value.toString();
    };
    
     /*
      * Usar no FieldOnKeyup
      *
      */
    
    this.lengthCurrency = function(args) {
        var value;
        var fieldValue;
        var field = args.owner.field;
        // var allowedKeys = [8, 9, 37, 38, 39, 40, 46, 44, 108, 188, 194];
        var allowedKeys = ['Tab', 'ArrowLeft', 'ArrowRight', 'ArrowUp', 'ArrowDown', 'End', 'Home', 'Shift', 'Control', 'Delete', 'Backspace', '.', ',', '-', 'Alt'];
        
        if (field.inlineEditable && (field.widget.mode === "list" || field.widget.mode === "inline_edit")) {
            fieldValue = field.widget.currentRow[field.name + '_inline'];
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue.toString();
        } else {
            fieldValue = field.value();
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue.toString();
        }
        var cleanNumber = value.split('-').join('').split('.').join('').split(',').join('');
        var realLength = value.split('-').join('').split('.').join('').split(',')[0].length;
        var last = Math.max(...[value.lastIndexOf(','), value.lastIndexOf('.')]);
        var separador;
        var decimalLength = value.split('-').join('').split('.').join('').split(',')[1] ? value.split('-').join('').split('.').join('').split(',')[1].length : 0;
        var precision = "d{0,5}";
        var precisionLength = 0;
        var regex = "^(\\d{1," + field.maxLengthCurrency + "})$";
        
        if(field.mask && field.mask.params && field.mask.params.precision){
            precision = "d{1,"+ field.mask.params.precision+"}";
            regex = "^(\\d{1," + field.maxLengthCurrency + "})(\\,|\\.){1}(\\"+precision+")$";
            precisionLength = field.mask.params.precision;
        }
        var validation = new RegExp(regex, "gm");
        
        if (last > -1){
            separador = value.substring(0, last).length >= field.maxLengthCurrency-precisionLength ? ',' : value[last];
            value = value.substr(0,last).concat(separador+value.substr(last+1, value.length));
            value = value.split('.').join('');
            last = value.lastIndexOf(',');
            realLength = value.substr(0,last).length;
            decimalLength = value.substr(last+1, value.length).length;
        }
        
        
        if((args.e.keyCode === 45 && value.includes('-'))){
            args.e.preventDefault();
        }else{
            if(event.target.selectionStart == event.target.selectionEnd){
                if (realLength >= field.maxLengthCurrency && precisionLength >= decimalLength && (!validation.test(value) && !value.endsWith(',') && !value.endsWith('.'))) {
                    if (!allowedKeys.includes(args.e.key) || precisionLength == 0)
                        args.e.preventDefault();
                } else if(realLength == field.maxLengthCurrency && precisionLength == decimalLength){
                    if (!allowedKeys.includes(args.e.key) || precisionLength == 0)
                        args.e.preventDefault();
                } else if(cleanNumber.length == field.maxLengthCurrency + precisionLength){
                    if (!allowedKeys.includes(args.e.key) || precisionLength == 0)
                        args.e.preventDefault();
                }
            }
        }
    };
    
    /*
     * Aceita [- . , 0 1 2 3 4 5 6 7 8 9]
     *
     */
    
     this.justDigits = function(selector, retryCount) {
        if (!retryCount || retryCount > 5)
            retryCount = 5;
        if (!selector) {
            throw {
                "message": "Selector is not defined!"
            };
        }
        setTimeout(function() {
            var elem = $(selector);
            if (elem[0]) {
                var length = selector.split(',').length;
                for (var i = 0; i < length; i++) {
                    if (elem.hasOwnProperty(i)) {
                        elem[i].onkeypress = avoidNonDigits;
                        elem[i].onpaste = preventEvent;
                    }
                }
            } else {
                if (!retryCount) {
                    console.error("Selector not found!");
                } else {
                    retryCount--;
                    this.justDigits (selector, retryCount);
                }
            }
        }.bind(this), 500);
    };

    function avoidNonDigits(e) {
        var digits = [44, 45, 46, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
        var numberCode = e.which;
        if (digits.indexOf(numberCode) < 0) {
            preventEvent(e);
        }
    }
    
    /*
        Função para formatação de campos numéricos decimais e completar com zero se necessário
    */
    
    function setValue (event, value){
        event.target.value = value;
    }
    
    this.checkFieldLength = function(field, convertToFloat = false) {
        var maxLength = field._maxSize;
        var precision = field._precision;
        var modeToTrunc = field._modeToTrunc ? field._modeToTrunc : 'END';
        var caracter = field._caracter;
        var allowNegative = field._allowNegative;
        var value = event.target.value;
        var oldValue = value;
        var absNumber = value.split('-').join('');
        value = allowNegative ? '-'+absNumber : absNumber;
        var obj = treatStrWithNaN(value, true, [',','.','-'],caracter);
        // field.setValue(obj.normalizedNumber);
        setValue(event, obj.normalizedNumber);
        templateManager.updateTemplate();
        var qtCharacteresNaN = obj.count; 
        value = obj.normalizedNumber;
        if(qtCharacteresNaN === oldValue.length){
            // field.setValue("");
            setValue(event, "");
            templateManager.updateTemplate();
        } else {
            if(!isNumber(event) &&  qtCharacteresNaN > 1 && value.length >= oldValue.length){
                let normalizedNumber = value.split('');
                normalizedNumber.pop();
                normalizedNumber = normalizedNumber.join('');
                // field.setValue(normalizedNumber);
                setValue(event, normalizedNumber);
                templateManager.updateTemplate();
            }
            if (value) {
                if(field.validToSave && !isValidNumberMask(value, maxLength, precision)){
                    var formatValue = formatarNumero(value, maxLength, precision, modeToTrunc, caracter);
                    formatValue = convertToFloat ? parseFloat(formatValue) : formatValue;
                    // field.clearValue();
                    setValue(event, "");
                    // field.setValue(formatValue);
                    setValue(event, formatValue);
                    field.reload();
                }
                field.validToSave = true;
            }else{
                if(!field.validToSave){
                    // field.clearValue();
                    setValue(event, "");
                    field.validToSave = true;
                }else{
                    field.validToSave = false;
                }
            }
        }
    };
    
    function isValidNumberMask(value, maxLength, precision){
        var formatNumber = getPartsNumber(value, precision, maxLength);
        var number = value && typeof value.toString === 'function' ? value.toString().split(',').join('').split('.').join('') : "";
        if(number.length <= maxLength && formatNumber.length === value.length){
            return true;
        }else{
            return false;
        }
    }
    
    var getPartsNumber = function(number, precision, maxLength,caracter) {
        var formatNumber = '';
        number = number.split(',').join('.');
        var numberClean = number.split('.').join('');
        var lastCharacter = number[number.length-1];
        var precisionUser = number.split('.').length > 1 ?  number.split('.')[1].length : 0; 
        if (typeof number === 'string' && typeof precision === 'number') {
            if(typeof maxLength === 'number' && (number.length-1)-precisionUser <= maxLength-precision && 
                precisionUser <= precision && 
            (
                (precisionUser !== 0) || (precisionUser === 0 && number.length <= maxLength-precision) || 
                (precisionUser === 0 && 
                    (number.length <= maxLength-precision || (numberClean.length <= maxLength-precision && lastCharacter === '.')))
            )
            ){
                formatNumber = number;
            }else{
                var part1 = number.substring(0, number.length - precision);
                var part2 = number.substring(number.length - precision, number.length);
                if (typeof caracter === 'string') {
                    formatNumber = part1 + caracter + part2;
                } else {
                    formatNumber = part1 + '.' + part2;
                }
            }
        }
        return formatNumber;
    };
    
    function isNumber(event){
        return (event.keyCode > 47 && event.keyCode < 58) || (event.keyCode > 95 && event.keyCode < 106);
    }
    
    function treatStrWithNaN(str, verifyAndNormalizeNumber, allowChar, caracter = '.'){
        let obj = {
            count: 0,
            normalizedNumber: ""
        };
        allowChar = Array.isArray(allowChar) ? allowChar : [];
        if(typeof str === 'string' && str.length){
            str.split('').forEach(function(caracter){
                obj.count += isNaN(caracter) ? 1 : 0;
                if(verifyAndNormalizeNumber) {
                    obj.normalizedNumber += isNaN(caracter) && allowChar.indexOf(caracter) === -1 ? "" : caracter;
                }
            });
        } 
        if(obj.normalizedNumber && caracter){
            obj.normalizedNumber = obj.normalizedNumber.split('.').join(caracter);
        }
        return obj;
    }
    
    var formatarNumero = function(value, maxLength, precision, modeToTrunc, caracter) {
        var arrayFromString;
        var number = value && typeof value.toString === 'function' ? value.toString().split(',').join('').split('.').join('') : "";
        if (number && typeof caracter === 'string') {
            number = number.split(caracter).join('');
        }
        if (typeof maxLength === 'number' && typeof precision === 'number') {
            if (number && number.length > maxLength) {
                do {
                    arrayFromString = number.split('');
                    if (modeToTrunc === 'END') {
                        number = number.substring(0, maxLength);
                    } else {
                        arrayFromString.shift();
                        number = arrayFromString.join('');
                    }
                } while (number.length > maxLength);
                number = getPartsNumber(number, precision, maxLength,caracter);
            } else if (number.length > precision) {
                number = getPartsNumber(number, precision, maxLength,caracter);
            } else if (number.length === precision){
                number = getPartsNumber(number.padStart(maxLength, '0'), precision);
            }
        } else if (typeof precision === 'number') {
            number = getPartsNumber(number, precision, maxLength,caracter);
        } else if (typeof maxLength === 'number') {
            do {
                arrayFromString = number.split('');
                if (modeToTrunc === 'END') {
                    number = number.substring(0, maxLength);
                } else {
                    arrayFromString.shift();
                    number = arrayFromString.join('');
                }
            } while (number.length > maxLength);
        }
        number = caracter ? parseFloat(number.split(',').join('.')).toString().split('.').join(caracter) : parseFloat(number.split(',').join('.')).toString();
        return number;
    };
    
    this.fillZero = function(field){
        let value = event.target.value;
        let precision = field._precision;
        if(value){
            let valueFloat = parseFloat(value.split(',').join('.'));
            valueFloat = valueFloat.toFixed(precision);
            valueFloat = valueFloat.split('.').join(',');
            setValue(event, valueFloat);
        }
    };
    function isSemicolon(event){
        return event.keyCode === 194 || event.keyCode === 190 || event.keyCode === 188 || event.keyCode === 110; //ponto ou virgula
    }
    this.onKeyDown = function(field){
        var obj = treatStrWithNaN(event.target.value, true, null, ',');
        var element = event.target;
        var isNumberValue = isNumber(event);
        if(!isKeysNavigationOrDelete(event) && !isNumberValue && !isSemicolon(event)){
            event.returnValue = false;
        } else {
            if(obj.normalizedNumber.length === field._maxSize){
                if(isNumber(event)){
                    if(element.selectionStart === element.selectionEnd){
                        event.returnValue = false;
                    }
                } else if(!isKeysNavigationOrDelete(event)){
                    event.returnValue = false;
                }
            }
        }
    };
    
    function isKeysNavigationOrDelete(event){
        return event.keyCode === 8 || event.keyCode === 46 || event.keyCode === 13 || (event.keyCode > 36 && event.keyCode < 41);
    }
    
    this.latinize = function(a){
        var Latinize = {
            "Á": "a", "á": "a", "Â": "a", "â": "a", "Ã": "a", "ã": "a",
            "É": "e", "é": "e", "Ê": "e", "ê": "e",
            "Í": "i", "í": "i", "Î": "i", "î": "i", 
            "Ó": "o", "ó": "o", "Ô": "o", "ô": "o", "Õ": "o", "õ": "o",
            "Ú": "u", "ú": "u", "Û": "u", "û": "u",
            "Ç": "c", "ç": "c"
            
            
        };
        return a.replace(/[^A-Za-z0-9\[\] ]/g, function(a) {
    		return Latinize[a] || a;
    	});
    };
    
    /*
        Inserir essa função apos aplicar o filtro, seja no FilterAfterApply ou em um OnChange caso seja uma 
        field que filtra a tela ao inves de um popup de filtro
        
        @params
        stripeWidget => nome da widget de stripe ou o objecto em si
        row (opcional) => row contendo os dados que vão para o filtro do relatorio
        fieldsToUse (opcional) => array com nome de fields a serem exibidas no filtro do relatorio
        ['NMFILIAL']
        default:
            ele exibe todas as fields visiveis do stripe no filtro do relatorio
        Issue:
            stripeWidget por default procurar por uma widget com template widget/stripe.html
    */
    this.newFilterReport = function(stripeWidget, row, fieldsToUse){
        stripeWidget = setDefaultValuesConfig(stripeWidget);
        let config = stripeWidget.floatingControlConfig;
        let dataSet = [];
        row = row || stripeWidget.currentRow;
        let arrayKeysRangeFields = Object.keys(config.rangeFields);
        let allFieldsRange = arrayKeysRangeFields.join('_');
        if(Array.isArray(config.rangeFields) && config.rangeFields.length){
            arrayKeysRangeFields.forEach( rangeField => {
                let dataSets = {
                    $I: {
                        propertyToGet: 'INDEX',
                        value: config.rangeFields[rangeField].itemsToReplace,
                    },
                    $V: {
                        propertyToGet: 'NAMEFIELD',
                        value: row
                    }
                }; 
                let fieldsRange = rangeField.split('_');
                let treatStr = config.rangeFields[rangeField].format.replace(/\$[A-Z]+\d+/g, function(currentMatch){
                    let cKey = currentMatch.match(/(\$[A-Za-z]+)/g)[0];
                    let arrayValues = dataSets[cKey];

                    let index = currentMatch.replace(cKey, '');
                    let currentItem =  arrayValues.propertyToGet == 'INDEX' ? arrayValues.value[index] : arrayValues.value[fieldsRange[index]];
                    return ScreenService.i18n(currentItem);
                });
                dataSet.push(
                    {
                        label: treatLabel(config.rangeFields[rangeField].label),
                        value: treatStr
                    }
                );
            });
        }
        if(Array.isArray(fieldsToUse) && fieldsToUse.length){
            fieldsToUse.forEach( fieldName => {
                let rangeContainsCurrentField = new RegExp(`(?<!([A-Z]+))(${fieldName})(?!([A-Z]+))`, 'gm').test(allFieldsRange);
                if(!rangeContainsCurrentField){
                    let field = stripeWidget.getField(fieldName);
                    dataSet.push(
                        {
                            label: treatLabel(field.label),
                            value: row[field.name]
                        }
                    );
                }
            });
        } 
        if(!config.ignoreFieldsStripe){
            stripeWidget.fields.forEach( field => {
                if(field.isVisible){
                    dataSet.push(
                        {
                            label: treatLabel(field.label),
                            value: row[field.name]
                        }
                    );
                }
            });
        }
        config.dataset = dataSet; 
    };
    
    /*
        Handles filter for floating control report
        USAGE:
            Widget with floating control report:
                event: BeforeGenerateReport => call function setFilterReport
            Event or function after initial filter screen => call function newFilterReport
        DEFAULT (PRE SET) STRUCT IN WIDGET STRIPE (PREFERENCE)
            "floatingControlConfig":{
                "dataset": [],
                "ignoreFieldsStripe": false,
                "rangeFields": []
            }
    */

   function setDefaultValuesConfig(stripeWidget){
        if(!stripeWidget || stripeWidget == 'DEFAULT'){
            stripeWidget = templateManager.container.widgets.filter( widget => 
                widget.template && widget.template.toLowerCase().indexOf('stripe') > -1
            )[0];
        } else {
            stripeWidget = typeof stripeWidget === 'string' ? templateManager.container.getWidget(stripeWidget) : stripeWidget;
        }
        stripeWidget.floatingControlConfig = stripeWidget.floatingControlConfig || {};
        stripeWidget.floatingControlConfig.dataset = stripeWidget.floatingControlConfig.dataset || [];
        stripeWidget.floatingControlConfig.rangeFields = stripeWidget.floatingControlConfig.rangeFields || [];
        stripeWidget.floatingControlConfig.ignoreFieldsStripe = stripeWidget.floatingControlConfig.hasOwnProperty('ignoreFieldsStripe') ? stripeWidget.floatingControlConfig.ignoreFieldsStripe : false;
        return stripeWidget;
    }
    
     function treatLabel(label){
        return ScreenService.i18n(label).split(':').join('');  
    }
    
    this.setFilterReport = function(args, stripeWidget){
        stripeWidget = setDefaultValuesConfig(stripeWidget);
        args.e.metadata.filter = stripeWidget.floatingControlConfig.dataset;
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('gopUtil', gopUtil);
});

// FILE: ./assets/js/services/Constants.js
if (typeof Constants === "undefined") {
	var Constants = {
		successNotification: {
			type: "success"
		},

		successMessage: {
			type: "success",
			icon: "checked"
		},

		errorNotification: {
			type: "error"
		},

		errorMessage: {
			type: "error",
			icon: "error"
		},

		alertMessage: {
			type: "alert",
			icon: "exclamation"
		},

		infoMessage: {
			type: "information",
			icon: "information"
		},

		confirmationMessage: {
			type: "question",
			icon: "question"
		}
	};
}

// FILE: ./assets/js/services/Ger03000_ConclusaoPrepOrca_Service.js
var Ger03000_ConclusaoPrepOrca_Service = function (restEngine){
    
	this.autenticacao = function (data){
	    
        return restEngine.post("gop#/ger03000_autenticacao", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.checkAcesso = function (data){
	    
        return restEngine.post("gop#/ger03000_checkAcesso", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.checkEtapas = function (data){
	    
        return restEngine.post("gop#/ger03000_checkEtapas", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.checkOrcamento = function (data){
	    
        return restEngine.post("gop#/ger03000_checkOrcamento", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.getOrcamento = function (data){
	    
        return restEngine.post("gop#/ger03000_getOrcamento", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.checkExistAprova = function (data){
	    
        return restEngine.post("gop#/ger03000_checkExistAprova", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};

};

Configuration(function(ContextRegister){
	ContextRegister.register('Ger03000_ConclusaoPrepOrca_Service', Ger03000_ConclusaoPrepOrca_Service);
});

// FILE: ./assets/js/services/Gop01100_ComparacaoVersoesOrcamento_Service.js
var Gop01100_ComparacaoVersoesOrcamento_Service = function (restEngine){
    
    this.getParametro = function(){
		return restEngine.post('gop#/crud/paramorcaoperac', {
            "requestType": "FilterData",
            "filter": ""
        });
	};  
    
	this.getDataDetalhamento = function (data){
        return restEngine.post("gop#/gop01100_getDataDetalhamento", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.getVersoes = function (data){
        return restEngine.post("gop#/gop01100_getVersoes", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
};

Configuration(function(ContextRegister){
	ContextRegister.register('Gop01100_ComparacaoVersoesOrcamento_Service', Gop01100_ComparacaoVersoesOrcamento_Service);
});

// FILE: ./assets/js/services/gop10000_CalculoOrcamentoOperacional_Service.js
var gop10000_CalculoOrcamentoOperacional_Service = function (restEngine){
	
	this.getParametro = function(){
		return restEngine.post('gop#/crud/paramorcaoperac', {
            "requestType": "FilterData",
            "filter": ""
            });
	};
	
	this.getItemMes = function(filtro){
		return restEngine.post('gop#/crud/gop10000_item_mes', {
            "requestType": "FilterData",
            "filter": filtro
            });
	};
	
	this.atualizaMesBase = function(data){
	    return restEngine.post('gop#/gop10000_atualizaMesBase', {
            "requestType": "DataSet",
            "dataset": data
            });
	};
	
	this.processaCalculoComensais = function(data, parametros){
	    return restEngine.post('gop#/gop10000_processaCalculoComensais', {
            "requestType": "DataSet",
            "dataset": data,
            "parametros": parametros
            });
	};
	
	this.atualizaItemTotal = function(data){
	    return restEngine.post('gop#/gop10000_atualizaItemTotal', {
            "requestType": "DataSet",
            "dataset": data
	    });
	};
	
	this.updateDetalheDespesaMo = function(data){
	    return restEngine.post('gop#/gop10000_updateDetalheDespesaMo', {
            "requestType": "DataSet",
            "dataset": data
	    });
	};

	
	this.updateMeses = function(data){
	    return restEngine.post('gop#/gop10000_updateMeses', {
            "requestType": "DataSet",
            "dataset": data
        });
	};
	
	this.getDataSourceItemMes = function(filtro){
		return restEngine.post('gop#/crud/gop10000_item_mes', {
            "requestType": "FilterData",
            "filter": filtro
        });
	};
	
	this.updateTotalDetalhe = function(data){
	    return restEngine.post('gop#/gop10000_updateTotalDetalhe', {
            "requestType": "DataSet",
            "dataset": data
        });
	};
	
	this.updateDetalheImposto = function(data){
	    return restEngine.post('gop#/gop10000_updateDetalheImposto', {
            "requestType": "DataSet",
            "dataset": data
	    });
	};

	this.updateTotalImposto = function(data){
	    return restEngine.post('gop#/gop10000_updateTotalImposto', {
            "requestType": "DataSet",
            "dataset": data
	    });
	};
	
	this.importSpreadsheet = function(row){
	    return restEngine.post('gop#/gop10000_importSpreadsheet', {
            "requestType": "Row",
            "row": row
	    });
	};
};

Configuration(function(ContextRegister){
	ContextRegister.register('gop10000_CalculoOrcamentoOperacional_Service', gop10000_CalculoOrcamentoOperacional_Service);
});

// FILE: ./assets/js/services/gop11000_AprovacaoOrcaOperac_Service.js
var gop11000_AprovacaoOrcaOperac_Service = function (restEngine){
	
	this.saveAprova = function (data){
	    
        return restEngine.post("gop#/gop11000_saveAprova", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.updateAprova = function (data){
	    
        return restEngine.post("gop#/gop11000_updateAprova", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.updateConclusao = function (data){
	    
        return restEngine.post("gop#/gop11000_updateConclusao", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
};

Configuration(function(ContextRegister){
	ContextRegister.register('gop11000_AprovacaoOrcaOperac_Service', gop11000_AprovacaoOrcaOperac_Service);
});

// FILE: ./assets/modules/cadastro/assets/js/services/ger80700_orcamentoMo_Service.js
var ger80700_orcamentoMo_Service = function (ScreenService, templateManager, restEngine) {
    
	 this.excluiDespesa = function (data){
	    
        return restEngine.post("gop_cadastro#/ger80700_excluiDespesa", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};

};

Configuration(function(ContextRegister, eventEngine) {
	ContextRegister.register('ger80700_orcamentoMo_Service', ger80700_orcamentoMo_Service);
});


// FILE: ./assets/modules/cadastro/assets/js/services/gop20000_conta_contabil_por_despesaService.js
var gop20000_conta_contabil_por_despesaService = function (ScreenService, templateManager, restEngine) {
    
        this.atualiza = function(dataset){
        var parametroRestEngine  = {
            "requestType": "DataSet",
            "dataset": dataset
        };
        
        return restEngine.post('gop_cadastro#/gop20000_conta_contabil_por_despesa/atualiza', parametroRestEngine)
            .then(function(response){
                return response;
            }.bind(this))
            .catch(function(error){
                throw error;
            }
        );
    };
};

Configuration(function(ContextRegister, eventEngine) {
	ContextRegister.register('gop20000_conta_contabil_por_despesaService', gop20000_conta_contabil_por_despesaService);
});










// FILE: ./assets/modules/relatorio/assets/js/services/gop13000_verificacao_de_pendenciasService.js
var gop13000_verificacao_de_pendenciasService = function (ScreenService, templateManager, restEngine) {

	this.dataParametro = function(data){
	    return restEngine.post("gop_relatorio#/gop13000_verificacao_parametro",{
	       "requestType": "FilterData",
	       "filter": data
	    }).then(function(response){
	        return response;
	    }).catch(function(error){
	        throw error;
	    });
	};

	this.dataOperacional = function(data){
	    return restEngine.post("gop_relatorio#/gop13000_verificacao_operacional",{
	       "requestType": "FilterData",
	       "filter": data
	    }).then(function(response){
	        return response;
	    }).catch(function(error){
	        throw error;
	    });
	};

	
    
};

Configuration(function(ContextRegister, eventEngine) {
	ContextRegister.register('gop13000_verificacao_de_pendenciasService', gop13000_verificacao_de_pendenciasService);
});


// FILE: ./assets/modules/parametro/assets/js/services/fai02800_lancamento_de_despesas_com_mao_de_obra_Service.js
var fai02800_lancamento_de_despesas_com_mao_de_obra_Service = function (ScreenService, templateManager, restEngine) {

	this.atualizaLancDesp = function (data){
	    
        return restEngine.post("gop#/fai02800_atualizaLancDesp", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};

};

Configuration(function(ContextRegister, eventEngine) {
	ContextRegister.register('fai02800_lancamento_de_despesas_com_mao_de_obra_Service', fai02800_lancamento_de_despesas_com_mao_de_obra_Service);
});

// FILE: ./assets/modules/parametro/assets/js/services/gop00100_parametros_orcamento_Service.js
var gop00100_parametros_orcamento_Service = function (ScreenService, templateManager, restEngine) {

	this.atualizaFilial = function (data){
	    
        return restEngine.post("gop_parametro#/gop00100_atualizaFilial", {
            "requestType": "DataSet",
            "dataset": data
        }
        );
	};
	
};

Configuration(function(ContextRegister, eventEngine) {
	ContextRegister.register('gop00100_parametros_orcamento_Service', gop00100_parametros_orcamento_Service);
});

// FILE: ./assets/modules/parametro/assets/js/services/gop00200_controle_de_unidades_por_gerenteService.js
var Gop00200_controle_de_unidades_por_gerenteService = function (ScreenService, templateManager, restEngine) {

	this.excluirGrupoGerente = function (data){
	    
        return restEngine.post("gop_parametro#/gop00200_excluir_grupo_gerente", {
            "requestType": "DataSet",
            "dataset": [data]
        });
	};
	
	this.excluirOperadorGrupo = function (data){
	    
        return restEngine.post("gop_parametro#/gop00200_excluir_operador_grupo", {
            "requestType": "DataSet",
            "dataset": [data]
        });
	};
	
	this.saveOperadorGrupo = function (data){
	    
        return restEngine.post("gop_parametro#/gop00200_save_operador_grupo", {
            "requestType": "DataSet",
            "dataset": [data]
        });
	};

};

Configuration(function(ContextRegister, eventEngine) {
	ContextRegister.register('Gop00200_controle_de_unidades_por_gerenteService', Gop00200_controle_de_unidades_por_gerenteService);
});

// FILE: ./assets/modules/parametro/assets/js/services/gop02000_calendario_prep_orca_Service.js
var gop02000_calendario_prep_orca_Service = function (ScreenService, templateManager, restEngine) {

	this.existeConcpreorcagop = function(filter){
		return restEngine.post("gop_parametro#/crud/gop02000_existe_concpreorcagop", {
            "requestType": "FilterData",
            "filter": filter
        });
	};
	
};

Configuration(function(ContextRegister, eventEngine) {
	ContextRegister.register('gop02000_calendario_prep_orca_Service', gop02000_calendario_prep_orca_Service);
});

// FILE: ./assets/modules/parametro/assets/js/services/gop21000_percentual_por_item_operacional_Service.js
var gop21000_percentual_por_item_operacional_Service = function (ScreenService, templateManager, restEngine) {

	this.verificaInitUnidade = function(CDFILIAL) {
        return restEngine.post("gop#/initUnidade_validation", {
            "requestType": "FilterData",
            "filter": [
                {
                    name: "P_CDFILIAL",
                    operator: "=",
                    value: CDFILIAL
                }
            ]
        });
    };
    
    this.saveItemOrcamento = function (data){
	    
        return restEngine.post("gop_parametro#/gop21000_percentual_save_item", {
            "requestType": "DataSet",
            "dataset": [data]
        });
	};
	
	this.saveItemOrcamentoUnidade = function (data){
	    
        return restEngine.post("gop#/gop21000_percentual_save_item_unidade", {
            "requestType": "DataSet",
            "dataset": [data]
        });
	};
	
	this.saveMo = function (data){
	    
        return restEngine.post("gop#/gop21000_percentual_save_item_mo_prod", {
            "requestType": "DataSet",
            "dataset": [data]
        });
	};
	
	this.saveMoUnidade = function (data){
	    
        return restEngine.post("gop#/gop21000_percentual_save_item_mo_prod_unidade", {
            "requestType": "DataSet",
            "dataset": [data]
        });
	};
	
	this.saveProd = function (data){

        return restEngine.post("gop#/gop21000_percentual_save_item_mo_prod", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	
	};
	
	this.saveProdUnidade = function (data){
	    
	    return restEngine.post("gop#/gop21000_percentual_save_item_mo_prod_unidade", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
        
	};
	
	this.cadAutoPercentUnidade = function(CDFILIAL_ORIG, CDFILIAL_DEST){
	    return restEngine.post("gop#/cadAutoPercentUnidade", {
            "requestType": "FilterData",
            "filter": [
                {
                    name: "P_CDFILIAL_ORIG",
                    operator: "=",
                    value: CDFILIAL_ORIG
                },
                {
                    name: "P_CDFILIAL_DEST",
                    operator: "=",
                    value: CDFILIAL_DEST
                }
            ]
        });
	};
	
	this.updateProdUnidade = function(data){
	    
        return restEngine.post("gop_parametro#/gop21000_percentual_update_item_mo_prod_unidade", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.updateProd = function(data){
	    
        return restEngine.post("gop_parametro#/gop21000_percentual_update_item_mo_prod", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.updateItemGeral = function(data){
	    
        return restEngine.post("gop_parametro#/gop21000_percentual_update_item_geral", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.updateItemUnidade = function(data){
	    
        return restEngine.post("gop_parametro#/gop21000_percentual_update_item_unidade", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};

};

Configuration(function(ContextRegister, eventEngine) {
	ContextRegister.register('gop21000_percentual_por_item_operacional_Service', gop21000_percentual_por_item_operacional_Service);
});

// FILE: ./assets/modules/parametro/assets/js/services/gop22000_percentual_de_correcao_Service.js
var gop22000_percentual_de_correcao_Service = function (ScreenService, templateManager, restEngine) {

	this.saveFilial = function(data){
	    
        return restEngine.post("gop_parametro#/gop22000_saveFilial", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};

	this.updateFilial = function(data){
	    
        return restEngine.post("gop_parametro#/gop22000_updateFilial", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.updateFilialCadAuto = function(data){
	    
        return restEngine.post("gop_parametro#/gop22000_updateFilialCadAuto", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(response){
                return response;
        }).catch(function(error){
                throw error;
            }
        );
	};
	
	this.verificaFilial = function(data){
	    return restEngine.post("gop_parametro#/gop22000_verificaFilial",{
	       "requestType": "DataSet",
	       "dataset": data
	    }).then(function(response){
	        let IDCNTRRESC = "";
	        try {
	           IDCNTRRESC = response.messages[0].message[0].IDCNTRRESC; 
	        } finally {
	            return IDCNTRRESC || "";
	        }
	        return ;
	    }).catch(function(error){
	        throw error;
	    });
	};

    
};

Configuration(function(ContextRegister, eventEngine) {
	ContextRegister.register('gop22000_percentual_de_correcao_Service', gop22000_percentual_de_correcao_Service);
});

// FILE: ./assets/js/controllers/ger03000_ConclusaoPrepOrcaController.js
function ger03000_ConclusaoPrepOrcaController(ScreenService, General, templateManager, gopUtil, ZhFilterPreferences, restEngine, Ger03000_ConclusaoPrepOrca_Service, gop11000_AprovacaoOrcaOperac_Service, MessageService) {

    var required = {
		required: true
	};
    
    /**
     * Abre a widget popUp (widgetDescricao) para descrever a conclusão.
    */
    this.openPopupDesc = function() {
        var descricaoWidget = gopUtil.getWidgetContainer('widgetDescricao');
        var mainWidget = gopUtil.getWidgetContainer("widgetMainConclusao");

        if (mainWidget.hasCheckedRows()) {
            descricaoWidget.getField('DSMOTIVO').isVisible = false;
        }
        else {
            descricaoWidget.currentRow.DSMOTIVO = mainWidget.currentRow.DSOBSERVACAO;
            if (Util.isEmptyOrBlank(descricaoWidget.currentRow.DSMOTIVO)) {
                descricaoWidget.getField('DSMOTIVO').isVisible = false;
            }
            else {
                descricaoWidget.getField('DSMOTIVO').isVisible = true;
            }
        }
        ScreenService.openPopup(descricaoWidget);
        descricaoWidget.currentRow.DSOBSERVACAO = "";
        descricaoWidget.reload();
    };
    
    /**
     * Abre a widget modal (widgetLogin) para logar com o usuário e senha de um
     * gerente que tenha acesso à aprovação de orçamentos.
    */
    this.openModalLog = function() {
        var modalWidget = gopUtil.getWidgetContainer('widgetLogin');
        modalWidget.currentRow.USER = "";
        modalWidget.currentRow.PASSWORD = "";
        ScreenService.openPopup(modalWidget);
    };

    this.fecharModal = function() {
        var mainWidget = gopUtil.getWidgetContainer('widgetMainConclusao');
        ScreenService.closePopup(true);
        mainWidget.list();
    };

    this.activateWidget = function() {
        var mainWidget = gopUtil.getWidgetContainer('widgetMainConclusao');
        mainWidget.activate();
        mainWidget.reload();
    };

    this.setFilter = function(mainWidget) {
        if (mainWidget.isValid()) {
            if(Array.isArray(mainWidget.appliedFilter.FILTER_ROW.CDFILIAL) === true){
                mainWidget.dataSourceFilter = [
                    {
                        "name": "P_CDFILIAL",
                        "operator": "=",
                        "value": mainWidget.appliedFilter.FILTER_ROW.CDFILIAL.join('_')
                    },
                    {
                        "name": "P_NRANOORCAGOP",
                        "operator": "=",
                        "value": mainWidget.appliedFilter.FILTER_ROW.NRANOORCAGOP
                    },
                    {
                        "name": "P_IDSTATUS",
                        "operator": "=",
                        "value": mainWidget.appliedFilter.FILTER_ROW.IDSTATUS
                    },
                    {
                        "name": "P_CDOPERADOR",
                        "operator": "=",
                        "value": getLocalVar('CDOPERADOR')
                    }
                ];
            } else{
                 mainWidget.dataSourceFilter = [
                    {
                        "name": "P_CDFILIAL",
                        "operator": "=",
                        "value": 'T'
                    },
                    {
                        "name": "P_NRANOORCAGOP",
                        "operator": "=",
                        "value": mainWidget.appliedFilter.FILTER_ROW.NRANOORCAGOP
                    },
                    {
                        "name": "P_IDSTATUS",
                        "operator": "=",
                        "value": mainWidget.appliedFilter.FILTER_ROW.IDSTATUS
                    },
                    {
                        "name": "P_CDOPERADOR",
                        "operator": "=",
                        "value": getLocalVar('CDOPERADOR')
                    }
                ];
            }
            mainWidget.reload();
        }
    };

    this.setDate = function(row) {
        if (row.NRCALENPREORCAGOP) {
            var dataInicial = moment(row.DTINICIALPREP, "DD/MM/YYYY");
            var dataFinal = dataInicial.add(row.NRDIAAREALIZA, "days");
            row.DTFINALPREP = moment(dataFinal, "eee mmm dd yyyy GMT-0200 (BRST)").format("DD/MM/YYYY");
        } else if (row.NRORCAMENTOGOP) {
            row.DSETAPA = "Orçamento";
        }
    };

    this.getFilter = function(mainWidget) {
        var array = [];
        mainWidget.dataSourceFilter.forEach(function(filter) {
            if (filter.name == "P_DTINICIALPREP" && !Util.isEmptyOrBlank(filter.value)) {
                array.push(filter.value);
            }
            if (filter.name == "P_DTFINALPREP" && !Util.isEmptyOrBlank(filter.value)) {
                array.push(filter.value);
            }
        });
        if (Util.isEmptyOrBlank(array)) {
            array.push("");
            array.push("");
        }
        return array;
    };

    this.setGrid = function(mainWidget) {
        var newDataSet = [];

        mainWidget.dataSource.data.forEach(function(row) {
            var periodo = gopUtil.getPeriodo(row.IDPERIODOORCA);
            this.setDate(row);
            periodo.forEach(function(p) {
                if (p.value == row.NRPERIODOORCA) {
                    row.NMPERIODOORCA = p.name;
                }
            });

            var dataInicialFiltroMoment = moment(this.getFilter(mainWidget)[0], 'DD/MM/YYYY');
            var dataFinalFiltroMoment = moment(this.getFilter(mainWidget)[1], 'DD/MM/YYYY');

            if (!Util.isEmptyOrBlank(dataInicialFiltroMoment)) {
                var dataInicialPrepMoment = moment(row.DTINICIALPREP, 'DD/MM/YYYY');
                if (dataInicialPrepMoment.isAfter(dataInicialFiltroMoment) && dataInicialPrepMoment.isBefore(dataFinalFiltroMoment)) {
                    newDataSet.push(row);
                }
            }
            if (!Util.isEmptyOrBlank(newDataSet)) {
                mainWidget.dataSource.data = newDataSet;
            }
        }.bind(this));
    };
    
    this.startFilter = function(widget){
       if(!widget.currentRow.NRANOORCAGOP){
            this.setDateFilter(widget);
       }
        gopUtil.startFilter(widget,widget.currentRow);
        
    };
    
    this.setDateFilter = function(widget){
        var year = moment().year();
        widget.currentRow.NRANOORCAGOP = year;
    };
    
    this.onChangeCheckbox = function (checkbox, select) {
		if (checkbox.value() == 'T') {
			select.readOnly = true;
			select.validations = {};
			select.setValue([]);
			select.widget.currentRow[select.name] = [];
			
			if (select.name == 'CDFILIAL') {
				select.widget.getField('CDFILIAL').reload().then(function() {
					if (select.widget.currentRow.CDGRUPFILI) {
						select.widget.currentRow.CDFILIAL = select.widget.getField('CDFILIAL').dataSource.data.map(function (filial) {
							return filial.CDFILIAL;
						}).join("_");
					}
					else
						select.widget.currentRow.CDFILIAL = "T";
				});
			}
		} else {
			select.readOnly = false;
			select.validations = required;
			if (select.name == 'CDFILIAL') {
				select.widget.getField('CDFILIAL').setValue('');
			}
		}
	};

    this.getMesPeriodo = function(row) {
        var mes = [];
        var mesInicio;
        var mesFinal;
        var array = [
            { "name": "MENSAL", "plus": 0 },
            { "name": "BIMESTRAL", "plus": 1 },
            { "name": "TRIMESTRAL", "plus": 2 },
            { "name": "QUADRIMESTRAL", "plus": 3 },
            { "name": "SEMESTRAL", "plus": 5 },
            { "name": "ANUAL", "plus": 11 },
        ];

        if (row.IDPERIODOORCA) {
            var idperiodo = row.IDPERIODOORCA;
            array.forEach(function(periodo) {
                if (idperiodo == periodo.name) {
                    if (row.NRPERIODOORCA != 1) {
                        mesInicio = row.NRPERIODOORCA + periodo.plus;
                    } else {
                        mesInicio = row.NRPERIODOORCA;
                    }
                    mesFinal = mesInicio + periodo.plus;
                }
            });
        }
        mes.push(mesInicio);
        mes.push(mesFinal);
        return mes;
    };

    this.checkRow = function(mainWidget) {
        if (mainWidget.currentRow.IDSTATUS == "PENDENTE") {
            mainWidget.dataSource.checkedRows.length = 1; //Impedir que abra o swipe
            this.openPopupDesc();
        } else {
            mainWidget.dataSource.checkedRows.length = 0; //Possibilita que abra o swipe
            mainWidget.lastCheckedRows = 0; //Possibilita que abra o swipe
            if (mainWidget.currentRow.DSETAPA == "Orçamento") {
                mainWidget.getField("NMGRUPGERENTEOP").isVisible = false;
            } else {
                mainWidget.getField("NMGRUPGERENTEOP").isVisible = true;
            }
        }
    };

    this.checkOrcamento = function() {
        var mainWidget = gopUtil.getWidgetContainer("widgetMainConclusao");
        var descricaoWidget = gopUtil.getWidgetContainer('widgetDescricao');

        if (mainWidget.getCheckedRows().length > 0) {
            if (Util.isEmptyOrBlank(descricaoWidget.currentRow.DSOBSERVACAO)) {
                mainWidget.currentRow.DSOBSERVACAO = " ";
            }

            mainWidget.getCheckedRows().forEach((row) => {
                row.IDSTATUS = "CONCLUIDO";
                row.__is_new = (!row.NRCONCPREORCAGOP);
                row.DSOBSERVACAO = descricaoWidget.currentRow.DSOBSERVACAO;
                mainWidget.currentRow = row;
                mainWidget.currentRow.__using_unit = mainWidget.paramsNewCode.__using_unit || false;
				mainWidget.currentRow.__new_code_pk = mainWidget.paramsNewCode.__new_code_pk;
				mainWidget.currentRow.__table_pk = mainWidget.paramsNewCode.__table_pk;
				mainWidget.currentRow.__new_code_interval = mainWidget.paramsNewCode.__new_code_interval || 1;
				mainWidget.currentRow.__new_code_length = mainWidget.paramsNewCode.__new_code_length || 5;
				mainWidget.currentRow.__new_code_column_size = mainWidget.paramsNewCode.__new_code_column_size || 10;
				mainWidget.currentRow.__cast_number = mainWidget.paramsNewCode.__cast_number || false;
                if (!mainWidget.currentRow.__columns_to_return && mainWidget.paramsNewCode && mainWidget.paramsNewCode.__columns_to_return)
                	mainWidget.currentRow.__columns_to_return = mainWidget.paramsNewCode.__columns_to_return;
                restEngine.post('gop#/crud/concpreorcagop/save', {
                    "requestType": "Row",
                    "row": row
                }).then(function() {
                    mainWidget.reload();
                    mainWidget.list();
                    ScreenService.closePopup(true);
                    this.checkAprova(row);
                }.bind(this), function(erro) {
                    ScreenService.showMessage(erro);
                });
            });
            
            mainWidget.unSelectAll();
            
        } else {
            Ger03000_ConclusaoPrepOrca_Service.checkEtapas([mainWidget.currentRow]).then(function(response) {
                if (response.dataset.ETAPAS[0].ETAPAS == 1) {
                    Ger03000_ConclusaoPrepOrca_Service.checkOrcamento([mainWidget.currentRow]).then(function(response) {
                        if (response.dataset.ORCAMENTOS[0].ORCAMENTOS != 0) {
                            this.conclui(mainWidget);
                        } else {
                            ScreenService.showMessage("Conclusão não permitida! Favor realizar a geração do orçamento");
                            ScreenService.closePopup();
                        }
                    }.bind(this), function(retorno) {
                        MessageService.showMessageByCode("MSG_ERRO", {"PARAMETRO": retorno.data.error});
                    });
                } else {
                    this.conclui(mainWidget);
                }
            }.bind(this), function(retorno) {
                MessageService.showMessageByCode("MSG_ERRO", {"PARAMETRO": retorno.data.error});
            });
        }
    };

    this.conclui = function(mainWidget) {
        var descricaoWidget = gopUtil.getWidgetContainer('widgetDescricao');

        if (descricaoWidget.isValid()) {
            mainWidget.currentRow.IDSTATUS = "CONCLUIDO";

            mainWidget.currentRow.__is_new = (!mainWidget.currentRow.NRCONCPREORCAGOP);

            if (!Util.isEmptyOrBlank(descricaoWidget.currentRow.DSOBSERVACAO)) {
                mainWidget.currentRow.DSOBSERVACAO = descricaoWidget.currentRow.DSOBSERVACAO;
            } else {
                mainWidget.currentRow.DSOBSERVACAO = " ";
            }
            General.widgetSave(mainWidget).then(function() {
                mainWidget.list();
                mainWidget.reload();
                ScreenService.closePopup(true);
                this.checkAprova();
            }.bind(this), function(erro) {
                console.log(erro);
            });
        }
    };

    this.checkAprova = function(row = null) {
        var mainWidget = gopUtil.getWidgetContainer("widgetMainConclusao");

        mainWidget.currentRow = row ? row : mainWidget.currentRow;
        Ger03000_ConclusaoPrepOrca_Service.checkEtapas([mainWidget.currentRow]).then(function(response) {
            if (response.dataset.ETAPAS[0].ETAPAS == 0) {
                restEngine.post('gop#/crud/paramorcaoperac', {
                    "requestType": "FilterData",
                    "filter": ""
                }).then(function(callback) {
                    var parametros = callback.dataset.paramorcaoperac[0];
                    if (parametros.IDAPROVORCACONC == "S") {
                        this.checkOper(mainWidget, row);
                    }
                }.bind(this), function(erro) {
                    console.log(erro);
                });
            }
        }.bind(this), function(retorno) {
            console.log(retorno.data.erro);
        });
    };

    this.checkOper = function(mainWidget, row = null) {
        mainWidget.currentRow = row ? row : mainWidget.currentRow;
        mainWidget.currentRow.CURRENTCDFILIAL = mainWidget.currentRow.CDFILIAL;
        mainWidget.currentRow.CDOPERADOR = mainWidget.dataSourceFilter.filter(a => a.name == "P_CDOPERADOR")[0].value;

        Ger03000_ConclusaoPrepOrca_Service.checkAcesso([mainWidget.currentRow]).then(function(response) {
            if (response.dataset.RESULT[0].IDACESSOAPROV == 'S') {
                this.aprova(mainWidget, row);
            }
            else {
                this.openModalLog();
            }
        }.bind(this), function(retorno) {
            console.log(retorno.data.erro);
        });
    };

    this.autentica = function(modalWidget) {
        var mainWidget = gopUtil.getWidgetContainer("widgetMainConclusao");

        if (modalWidget.isValid()) {
            modalWidget.currentRow.CDFILIAL = mainWidget.dataSourceFilter.filter(a => a.name == "P_CDFILIAL")[0].value;
            Ger03000_ConclusaoPrepOrca_Service.autenticacao([modalWidget.currentRow]).then(function(response) {
                if (response.dataset.RESULT[0].AUTENTIC) {
                    ScreenService.closePopup();
                    Ger03000_ConclusaoPrepOrca_Service.checkEtapas([mainWidget.currentRow]).then(function(response) {
                        this.aprova(mainWidget);
                    }.bind(this), function(retorno) {
                        MessageService.showMessageByCode("MSG_ERRO", {"PARAMETRO": retorno.data.error});
                    });
                }
                else {
                    ScreenService.showMessage("Usuário e senha inválidos");
                }
            }.bind(this), function(retorno) {
                console.log(retorno.data.erro);
            });
        }
    };

    this.aprova = function(mainWidget, row = null) {
        mainWidget.currentRow = row ? row : mainWidget.currentRow;
        mainWidget.currentRow.IDAPROVACAOGOP = "APROVADO";

        Ger03000_ConclusaoPrepOrca_Service.getOrcamento([mainWidget.currentRow]).then(function(response) {
            mainWidget.currentRow.NRORCAMENTOGOP = response.dataset.ORCAMENTO[0].NRORCAMENTOGOP;
            Ger03000_ConclusaoPrepOrca_Service.checkExistAprova([mainWidget.currentRow]).then(function(retorno) {
                if (Util.isEmptyOrBlank(retorno.dataset.EXISTE[0].NRAPROVAORCAGOP)) {
                    gop11000_AprovacaoOrcaOperac_Service.saveAprova([mainWidget.currentRow]).then(function(response) {
                        ScreenService.closePopup(true);
                        mainWidget.list();
                        mainWidget.reload();
                    }.bind(this), function(retorno) {
                        MessageService.showMessageByCode("MSG_ERRO", {"PARAMETRO": retorno.data.error});
                        ScreenService.closePopup();
                    });
                } else {
                    mainWidget.currentRow.IDAPROVACAOGOP = "APROVADO";
                    mainWidget.currentRow.NRAPROVAORCAGOP = retorno.dataset.EXISTE[0].NRAPROVAORCAGOP;

                    gop11000_AprovacaoOrcaOperac_Service.updateAprova([mainWidget.currentRow]).then(function(response) {
                        ScreenService.closePopup(true);
                        mainWidget.list();
                        mainWidget.reload();
                    }.bind(this), function(retorno) {
                        MessageService.showMessageByCode("MSG_ERRO", {"PARAMETRO": retorno.data.error});
                        ScreenService.closePopup();
                    });
                }
            }.bind(this), function(retorno) {
                console.log(retorno.data.erro);
            });
        }, function(retorno) {
            console.log(retorno.data.erro);
        });
    };

    this.includeCode = function(campo) {
        var userFinal = gopUtil.includeCodeZero(campo.value(), 12);
        campo.widget.currentRow[campo.name] = userFinal;
    };

    this.changeStatus = function(field) {
        if (Util.isEmptyOrBlank(field.value())) {
            field.value("T");
        }
    };

    this.validaOrcamento = function(row) {
        if (row.IDSTATUS === "CONCLUIDO") {
            row.__notSelectable = true;
            return true;
        }
        return false;
    };

    this.setInlineEditable = function(element, row, condition) {
        if (element.field && condition) {
            if (element.context.children[1]) {
                element.context.children[1].style.visibility = 'collapse';
            }
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ger03000_ConclusaoPrepOrcaController', ger03000_ConclusaoPrepOrcaController);
});

// FILE: ./assets/js/controllers/gop00240_PreparacaoDoOrcamento.js
var gop00240_PreparacaoDoOrcamento = function(ScreenService, restEngine, General, templateManager, gopUtil, metaDataFactory) {
    var self = this;

    /**
     * Todas essas funções servem para setar o valor dos parâmetros necessários para o dataSourceFilter do campo ""Unidades",
     * e também para assegurar que se o campo for vazio, será passado TODOS os dados filtrados pelo dataSource;
     * @param {object} widget principal
     * ("widgetMain").
     */
    this.concatenaAgFilial = function(row) {

        if (!row.CDGRUPFILI) {
            row.P_CDGRUPFILI = 'T';
        } else {
            row.P_CDGRUPFILI = row.CDGRUPFILI.join('_');
        }
    };
    
    this.concatenaSegmento = function(row) {

        if (!row.CDSEGMER) {
            row.P_CDSEGMER = 'T';
        } else {
            row.P_CDSEGMER = row.CDSEGMER.join('_');
        }
    };
    
    this.concatenaOperador = function(row) {

        if (!row.CDOPERADOR) {
            row.P_CDOPERADOR = 'T';
        } else {
            row.P_CDOPERADOR = row.CDOPERADOR.join('_');
        }
    };
    
    this.todasUnidades = function(row) {

        if (!row.CDFILIAL) {
            row.CDFILIAL = 'T';
        }
    };
    /**
     * Como é um field dependente de outros três campos,
     * chama as funções para setar os parâmetros que o dataSourceFilter precisa para buscar os dados
     * do campo select;
     * @param {object} field "Unidades"
     * ("CDFILIAL").
     */
    this.createDataSourceFilterFiliais = function(field) {

        var row = field.widget.currentRow;
        self.concatenaAgFilial(row);
        self.concatenaSegmento(row);
        self.concatenaOperador(row);
        var filtroUnidades = {
            'P_CDGRUPFILI': row.P_CDGRUPFILI,
            'P_CDSEGMER': row.P_CDSEGMER,
            'P_CDOPERADOR': row.P_CDOPERADOR
        };
        General.onClickDependentField(field, filtroUnidades);
    };
    
    this.limpaFiliais = function(widget){
       // widget.currentRow.CDSEGMER = [];
        widget.currentRow.CDFILIAL = [];
    };
    
    /** 
     * Ao entrar na widget será chamado o dataSource 
     * e setado o valor no campo "Tipo de Período";
     * 
     * @param {object} widget principal
     * ("widgetMain").
     */
     
    this.periodoOrcamento = function(widget) {
        
        restEngine.post('gop#/crud/paramorcaoperac', {
            "requestType": "FilterData",
            "filter": ""
        }).then(function(callback) {
            var parametros = callback.dataset.paramorcaoperac[0];
            widget.currentRow.IDPERIODOORCA = parametros.IDPERIODOORCA;
            if (widget.currentRow.IDPERIODOORCA == "ANUAL") {
                widget.getField('NRPERIODOORCA').readOnly = true;
                widget.currentRow.NRPERIODOORCA = "1";
                widget.getField('NRPERIODOORCA').validations = {};
            }else{
                widget.getField('NRPERIODOORCA').readOnly = false;
            }
            widget.reload();

        }.bind(this), function(erro) {
            ScreenService.showMessage(erro);
        });
    };
    /**
     * Verifica o que foi carregado no campo "Tipo de Período" e então a partir desse valor,
     * será definido qual funçaõ deverá ser chamada do arquivo gopUtil para setar o dataSource do Período;
     * 
     * @param {object} widget principal
     * ("widgetMain").
     */
    this.setPeriodo = function(widget) {
        
        var id = widget.currentRow.IDPERIODOORCA;

        if (id != "" && id != null) {
            if (id != "ANUAL") {
                if (id == "MENSAL") {
                    widget.getField('NRPERIODOORCA').dataSource.data = gopUtil.getDataMensal();
                } else if (id == "BIMESTRAL") {
                    widget.getField('NRPERIODOORCA').dataSource.data = gopUtil.getDataBimestral();
                } else if (id == "TRIMESTRAL") {
                    widget.getField('NRPERIODOORCA').dataSource.data = gopUtil.getDataTrimestral();
                } else if (id == "QUADRIMESTRAL") {
                    widget.getField('NRPERIODOORCA').dataSource.data = gopUtil.getDataQuadrimestral();
                } else if (id == "SEMESTRAL") {
                    widget.getField('NRPERIODOORCA').dataSource.data = gopUtil.getDataSemestral();
                }

                widget.getField("NRPERIODOORCA").readOnly = false;
                widget.getField("NRPERIODOORCA").validations = {
                    "required": {
                        "message": "Campo obrigatório"
                    }
                };
            } else {
                widget.getField("NRPERIODOORCA").readOnly = true;
            }
            
        } else {
            
            widget.getField("NRPERIODOORCA").readOnly = true;
            widget.getField("NRPERIODOORCA").validations = {
                "required": false
            };
        }
    };
    
    this.gerarOrcamento = function(widget) {
        if (widget.isValid()) {
            if(widget.currentRow.CDOPERADOR[0] == "T"){
                widget.currentRow.CDOPERADOR = null;
            }
            if (widget.currentRow.IDPERIODOORCA == "ANUAL") {
                widget.currentRow.NRPERIODOORCA = "1";
            }
            ScreenService.confirmMessage(ScreenService.i18n('Caso exista orçamento para o período informado, deseja substituí-lo?'), 'question',
                function(success) {
                    widget.currentRow.IDEXCLUIR = 'S';
                    self.getDados(widget); //Chama a procedure
                },
                function(error) {
                    widget.currentRow.IDEXCLUIR = 'N';
                    self.getDados(widget); //Chama a procedure
                }
            );
        }
    };
    
    /**
     * Como a procedure reconhece os valores marcados no campo 'TPORCAMENTO' por string,
     * então atribuímos a letra "S" aos valores selecionados no campo (ou seja, que estão no array do campo),
     * e o restante das posições existentes no campo não selecionadas recebem a letra "N";
     * Depois da string montada enviamos ela para a row;
     * 
     * @param {object} widget principal
     * ("widgetMain").
     */
    this.tipoOrcamentoString = function(widget) {

        var matriz = ["N", "N", "N", "N", "N", "N", "N", "N", "N", "N", "N", "N"];
        var tamanhoCampo = widget.currentRow.TPORCAMENTO;
        for (var i = 0; i < matriz.length; i++) {
            matriz[tamanhoCampo[i]] = 'S';
        }
        var stringTpOrca = matriz.toString();
        widget.currentRow.stringTpOrca = stringTpOrca.replace(/,/g, "");
    };

    /**
     * Atribui valores que ainda não estão na currentRow que serão necessários para a procedure,
     * Envia os valores da row para o backend;
     * 
     * @param {object} widget principal
     * ("widgetMain").
     */
    this.getDados = function(widget) {
        
        var row = widget.currentRow;
        var cdoperador = row.P_CDOPERADOR;
        
        row.CDOPERADOR = cdoperador.split('_');
        if(widget.currentRow.IDPERIODOORCA == "ANUAL"){
            row.VALUE_PERIOD = 1;
        }else{
            gopUtil.getPeriodo(widget.currentRow.IDPERIODOORCA).forEach(function(periodo){
                if(periodo.name == widget.currentRow.NRPERIODOORCA){
                    row.VALUE_PERIOD = parseInt(periodo.value);
                }
            });
        }

        self.todasUnidades(row);
        self.tipoOrcamentoString(widget);
        
        General.callRestEngine('gop#/getDados', "Row", row).then(function(response){
           
            if (Util.isEmptyOrBlank(response.dataset.errorList)){
                ScreenService.showMessage("Procedimento realizado com sucesso!");
            }else{
                ScreenService.showMessage("Algumas unidades não puderam ser geradas, favor verificar o log.").then(function(){
                this.showError(widget, response.dataset.errorList, 'Lista de Erros');
                }.bind(this));
            }
            
        }.bind(this),function(erro){
            ScreenService.showMessage("Não foi possível completar esta solicitação");
            
        }.bind(this));
        
    };
    
    this.resetFilial = function (widget){
        
        if(widget.currentRow.CDFILIAL){
            widget.currentRow.CDFILIAL = "";
        }  
    };
    
    this.showError = function (widget, dataset, popupName) {
        var messagesWidget = {
            "name": "messagesWidget",
            "label": "Mensagens",
            "messagesDataset": "",
            "template": "widget/form.html",
            "container": {},
            "isVisible": true,
            "popupNoBlock": false,
            "fields": [
                {
                    "label": "",
                    "name": "CONTENT",
                    "readOnly": true,
                    "template": "field/textarea.html",
                    "textRows": 14,
                    "isVisible": true,
                    "class": 12,
                    "descriptionOrder": true,
                    "showOnView": true,
                    "showOnForm": true,
                    "showOnList": true
                }
            ],
            "actions": [
                {
                    "name": "ok",
                    "label": "OK",
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup();"
                        }
                    ]
                },
                {
                    "name": "exportar",
                    "label": "Exportar",
                    "idVisible": true,
                    "icon": "export",
                    "showAsAction": "always",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "General.generateFileByDataSet(args.owner.widget.messagesDataset);"
                        }
                    ]
                }
            ],
            "events": []
        };
        messagesWidget.container = widget.container;
		var messagesWidgetInitialized = metaDataFactory.widgetFactory(messagesWidget);
		var content = '';
		if (popupName !== undefined)
			messagesWidgetInitialized.label = popupName;
		messagesWidgetInitialized.messagesDataset = dataset;
		dataset.forEach(function (message) {
			content += message + '\n';
		});
		messagesWidgetInitialized.getField('CONTENT').setValue(content);
		ScreenService.openPopup(messagesWidgetInitialized);
	};

};
Configuration(function(ContextRegister) {
    ContextRegister.register('gop00240_PreparacaoDoOrcamento', gop00240_PreparacaoDoOrcamento);
});

// FILE: ./assets/js/controllers/gop01100_ComparacaoVersoesOrcaController.js
function gop01100_ComparacaoVersoesOrcaController(ScreenService, General, templateManager, gopUtil, metaDataFactory, ZhReports, localStorageEngine, Gop01100_ComparacaoVersoesOrcamento_Service, MessageService){
    
    this.verificaPeriodoAnual = function(filterWidget){
        
        if(filterWidget.currentRow.IDPERIODOORCA == "ANUAL"){
            filterWidget.getField("NRPERIODOORCA").validations = {};
            filterWidget.getField("NRPERIODOORCA").readOnly = true;
            filterWidget.getField("NRPERIODOORCA").isVisible = false;
            filterWidget.getField("NRPERIODOORCA").value("1");
        }else{
            filterWidget.getField("NRPERIODOORCA").validations = { "required" : true};
            filterWidget.getField("NRPERIODOORCA").readOnly = false;
            filterWidget.getField("NRPERIODOORCA").isVisible = true;
        }
    };


    this.openFilterSelect = function(){
        
        var filterWidget = gopUtil.getWidgetContainer("filterSelect");
        
        Gop01100_ComparacaoVersoesOrcamento_Service.getParametro().then(function(retorno){
            
            var periodos = gopUtil.getPeriodo(retorno.dataset.paramorcaoperac[0].IDPERIODOORCA);
            
            filterWidget.getField("NRPERIODOORCA").dataSource.data = periodos;
            filterWidget.currentRow.IDPERIODOORCA = retorno.dataset.paramorcaoperac[0].IDPERIODOORCA;
            
            this.verificaPeriodoAnual(filterWidget);
            
         }.bind(this),function(erro){
             ScreenService.showMessage(erro);
         });
         
    };

    
    this.isset = function(target){
        
        if( target == undefined || target =='undefined' || target === "" || target === null || target ==="null"){
            return false;
        }else{
            return true;
        }
    };

    this.versaoEdit = function(args){
        
        var widget  = args.owner.field.widget;
        var unidade = widget.getField("NMFILIAL").value();  
        var ano     = widget.getField("NRANOORCAGOP").value();  
        var periodo = widget.getField("NRPERIODOORCA").value();  

        if(!this.isset(unidade) || !this.isset(ano) || !this.isset(periodo)){
            MessageService.showMessageByCode('REQUIRED_FIELDS');
            args.e.stopPropagation();
        }
        
    };
    
    this.setDataSourceVersoes = function(filterWidget){
        
        if(filterWidget.getField("VERSOES").value()) 
            return true;
        var unidades = filterWidget.getField("CDFILIAL");
        var ano      = filterWidget.getField("NRANOORCAGOP");
        var tipoOrca = filterWidget.getField("IDPERIODOORCA");
        var periodo  = filterWidget.getField("NRPERIODOORCA");
        
        if((unidades.value() != "" && unidades.value() != undefined) && (ano.value() != "" && ano.value() != undefined) && (tipoOrca.value() != "" &&tipoOrca.value() != undefined)&& (periodo.value() != "" && periodo.value() !=undefined )){    
            var dataVersoes = [];
            var nrversao;
            
            Gop01100_ComparacaoVersoesOrcamento_Service.getVersoes([filterWidget.currentRow]).then(function(response){
                for(nrversao = 1; nrversao <= response.dataset.RESULT[0]; nrversao++){
                    dataVersoes.push({
                        "VERSOES": "Versão " + nrversao,
                        "value": nrversao,
                        "align": "right"
                    });
                }
                filterWidget.getField("VERSOES").readOnly        = false;
                filterWidget.getField("VERSOES").validations     = { "required": true };
                filterWidget.getField("VERSOES").dataSource.data = dataVersoes;
                templateManager.updateTemplate();
            });
        } else {

            templateManager.updateTemplate();
        }    
    };
    
    
    this.limitaSelecao = function(args,selectAll){
        var widget = args.owner; 
        let data = widget.dataSource.data;
        let numeroItems = data.length;
        if(!selectAll){
            if(widget.getCheckedRows().length == 5){
                args.e.preventDefault(); 
                MessageService.showMessageByCode('LIMIT_VERSION');
            }
        } else {   
            if(numeroItems <= 5) return;
            args.e.preventDefault();
            let filteredRows = [];
            data.every( (item, index) => { 
                 if(index === 5) return false; 
                 filteredRows.push(item);
                 return true;
            });
            widget.dataSource.selectAll(filteredRows);
            MessageService.showMessageByCode('LIMIT_VERSION');
        }
    };
    
    
    this.limpaVersao = function (widget){
        widget.getField("VERSOES").clearValue();
    };


     this.gerarRelatorio = function(widget){
         
        var array = [];

        if (widget.isValid()) {
            
            widget.currentRow.VERSOES.forEach(function(versao){
    	        array.push(versao.split(" ")[1]);
        	});   
        	
        	var row = widget.currentRow;

            row.CDVERSAOGOP = '_' + array.join("_") + '_';


            var reportName          = widget.currentRow.RELATORIO;
            widget.currentRow.NRORG = getLocalVar("NRORG");
            row.FORMAT              = "pdf";
            let language            = window.lang || " _ ";
            let languageParts       = language.split("_");
            languageParts[1]        = languageParts[1].toUpperCase();
            language                = languageParts.join("_");
            row.LOCALE              = language;
            row.TIMEZONE            = Intl.DateTimeFormat().resolvedOptions().timeZone;
            row.NRANOVERSAOGOP      = row.NRANOORCAGOP;
            
            General.openReport("gop#/gop01100_gerarRelatorio", reportName, row).then(function() {

            });
        }
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('gop01100_ComparacaoVersoesOrcaController', gop01100_ComparacaoVersoesOrcaController);
});

// FILE: ./assets/js/controllers/gop10000_CalculoOrcamentoOperacionalController.js
function gop10000_CalculoOrcamentoOperacionalController(ScreenService, General, maskEngine, templateManager, restEngine, localStorageEngine, gopUtil, gop10000_CalculoOrcamentoOperacional_Service, MessageService){
    
    const self = this;
    
    /**
     * Função retorna um array com o nome
     * dos fields referentes aos meses do ano.
     * @param {} 
    **/
    this.getArrayCampos = function(){
        
        var arrayMeses = gopUtil.getDataMensal();
	    var arrayCampos = [];
        
        arrayMeses.forEach(function(mes){
            arrayCampos.push(mes.name.latinize().toUpperCase());
        });
        
        return arrayCampos;
    };
    
    /**
     * Abre o detalhamento das despesas
     * com produtos nos detalhamentos, ou seja,
     * a widget que funciona como parent é a última widget ativa
     * 
     * @param {args} widget ativa
    **/
    this.backEdit = function(args){
        
        var lastActiveWid = gopUtil.getWidgetContainer(args.owner.widget.retornoWidget);
        
		General.backOrCancel(args);
        args.owner.widget.unSelectAll();
	    lastActiveWid.reload();
        lastActiveWid.activate(); 
	};
	
	
	this.back = function(widget){
	    var lastActiveWid = gopUtil.getWidgetContainer(widget.retornoWidget);
		
        if(lastActiveWid.getParent()){
            lastActiveWid.getParent().reload();
        }
        widget.unSelectAll();
		lastActiveWid.reload();
        ScreenService.closeSwipe();
        lastActiveWid.activate();
	};
	
	this.atualizaMesBase = function(widget){
	    
	    if(widget.isValid()){
    	    var dataMesBase = 01 +"/"+ widget.currentRow.MESBASE +"/"+ widget.currentRow.ANOBASE;
    	    var retornoWidget = gopUtil.getWidgetContainer(widget.retornoWidget);
    	    var rows = retornoWidget.getCheckedRows();
            
            rows.forEach(function(row){
                row.DTMESBASEITEM = dataMesBase;
                row.__is_new = false;
            });
        
            gop10000_CalculoOrcamentoOperacional_Service.atualizaMesBase(rows).then(function(){
                ScreenService.closePopup(true);
                ScreenService.closeSwipe();
                retornoWidget.unSelectAll();
                retornoWidget.activate();
                retornoWidget.reload();
                widget.applyDefaultValue();
            }, function(erro){
                if(erro.indexOf("MSG_ERRO") !== -1){
                    MessageService.showMessageByCode("MSG_ERRO", {"PARAMETRO": erro.replace("MSG_ERRO", "")});
                } else {
                    ScreenService.showMessage(erro);
                }
            });
        }
	};
	
    this.processaPercentual = function(widget){
        
        var rows = widget.getCheckedRows();
        var parametros = Util.clone(widget.dataSourceFilter);
        
        parametros.push({name: "P_PROCESSA_PERCENTUAL", operator: "=", value: "S"});
        if(rows.length == 0){
            ScreenService.showMessage("Unidade deve ser selecionada.");
        }else{
            ScreenService.confirmMessage("Esta operação recalcula todos os valores do item orçamentário com base no percentual cadastrado para cada item sobre o valor do faturamento. Deseja prosseguir?").then(function(){
                this.executaCalculo(widget, parametros);
            }.bind(this));
        }
    };
    
    this.processaMaoObra = function(widget){
        
        var rows       = widget.getCheckedRows();
        var parametros = Util.clone(widget.dataSourceFilter);
        
        if(rows.length == 0 ){
            ScreenService.showMessage("Unidade deve ser selecionada");
        }else{
            this.executaCalculo(widget,parametros);
        }
    };
  
    this.processaDeprec = function(widget){
        
        var rows       = widget.getCheckedRows();
        var parametros = Util.clone(widget.dataSourceFilter);
        
        if(rows.length == 0 ){
            ScreenService.showMessage("Unidade deve ser selecionada");
        }else{
            this.executaCalculo(widget,parametros);
        }
    };
    	
	this.processaCalculo = function(widget){
	    
	    var rows = widget.getCheckedRows();
	    var parametros = Util.clone(widget.dataSourceFilter);
	    
	    parametros.push({name: "P_PROCESSA_PERCENTUAL", operator: "=", value: "N"});
	    if(rows.length == 0){
	        ScreenService.showMessage("Unidade deve ser selecionada.");
	    }else{
	        if(this.validaMesBase(rows)){
    	        ScreenService.confirmMessage("Esta operação recalcula todos os valores do Faturamento. Deseja prosseguir?").then(function(){
    	            this.executaCalculo(widget, parametros);
    	        }.bind(this));
	        }else{
	            ScreenService.showMessage("Favor informar mês base para prosseguir com o processamento.");
	        }
	    }
	};
	
	this.validaMesBase = function(rows){
	    
	    var bool = true;
	    
	    rows.forEach(function(row){
	        if(Util.isEmptyOrBlank(row.DTMESBASEITEM))
	            bool = false;
	    });
	    return bool;
	};
	
	this.executaCalculo = function(widget, parametros){
	    
	    gop10000_CalculoOrcamentoOperacional_Service.processaCalculoComensais(widget.getCheckedRows(), parametros).then(function(){
            ScreenService.closePopup();
            widget.unSelectAll();
            widget.reload();
            widget.list();
            ScreenService.successNotification("Cálculo realizado com sucesso");
        }, function(erro){
            ScreenService.showMessage("Ocorreu um erro ao realizar a operação!");
        });
	};
	
    this.openPopupFiltro = function(flag){
        var filterSelect = gopUtil.getWidgetContainer('filterSelect');
        if(flag)
            filterSelect.currentRow.P_TODOS = "N";

         gop10000_CalculoOrcamentoOperacional_Service.getParametro().then(function(retorno){
            filterSelect.getField("NRPERIODOORCA").dataSource.data = gopUtil.getPeriodo(retorno.dataset.paramorcaoperac[0].IDPERIODOORCA);
            filterSelect.currentRow.IDPERIODOORCA = retorno.dataset.paramorcaoperac[0].IDPERIODOORCA;
            filterSelect.currentRow.IDORCDESPENTROP = retorno.dataset.paramorcaoperac[0].IDORCDESPENTROP;
            filterSelect.currentRow.NRMESINIORCA = retorno.dataset.paramorcaoperac[0].NRMESINIORCA;
            gopUtil.checkPeriodo(filterSelect);
            ScreenService.openPopup(filterSelect);
         }.bind(this));
    };
    
    this.openPopupMesBase = function(NAME){
        var callWidget = gopUtil.getWidgetContainer(NAME);
        var widget = gopUtil.getWidgetContainer("REPLICA_MES");
        var rows = callWidget.getCheckedRows();
        
        if(rows != 0){
            widget.retornoWidget = NAME;
            ScreenService.openPopup(widget);
        }else{
            ScreenService.showMessage("Unidade deve ser selecionada.");
        }
    };
    
    this.setStripeLabel = function(filterWidget){
        
        var stripeWidget = gopUtil.getWidgetContainer("stripeWidget");
        var ano = filterWidget.currentRow.NRANOORCAGOP;
        var nrperiodo = filterWidget.currentRow.NRPERIODOORCA;
        var mesInicial = filterWidget.currentRow.NRMESINIORCA;
        var periodo = "";
            
        filterWidget.getField("NRPERIODOORCA").dataSource.data.forEach(function(dados){
            if(dados.value ===  nrperiodo){
                periodo = dados.name;
            }
        });
        if(mesInicial != 1){
            ano = ano + '/' + (gopUtil.parseFloat(ano) + 1);
        }
        
        if(Util.isEmptyOrBlank(periodo))
            periodo = "Anual";
        stripeWidget.getField("LABEL_STRIPE").label = "Ano: " + ano + " - Período: "+ periodo + " - Gerente Operacional: " + localStorageEngine.getLocalVar('NMOPERADOR');
    };

    /**
     * Função que monta o filtro dos dataSourceFilter das widgets e chama a
     * função responsável por setá-los.
     * @param {widget} widget ativa
     *  Chamada na action "Busca" da widget de filtro "filterSelect"
    **/
    this.busca = function(filterWidget){
        
        if(filterWidget.isValid()) {
            let tabCalculo = gopUtil.getWidgetContainer('TAB_CALCULO_OPERACIONAL');
            var resumoWidget = gopUtil.getWidgetContainer("GOP10000_RESUMO");
            var idperiodo = filterWidget.currentRow.IDPERIODOORCA;
            var nrperiodo = idperiodo == "ANUAL" ? 1 : filterWidget.currentRow.NRPERIODOORCA;
            var filtro = [
    		    { "name": "P_NRANOORCAGOP", "operator": "=", "value": filterWidget.currentRow.NRANOORCAGOP},
    			{ "name": "P_IDPERIODOORCA", "operator": "=", "value": idperiodo},
    			{ "name": "P_NRPERIODOORCA", "operator": "=", "value": nrperiodo},
    			{ "name": "P_CDOPERADOR", "operator": "=", "value": localStorageEngine.getLocalVar("CDOPERADOR")},
    			{ "name": "P_CDGRUPFILI", "operator": "=", "value": Util.isEmptyOrBlank(filterWidget.currentRow.CDGRUPFILI) ? "T" : ("_" + filterWidget.currentRow.CDGRUPFILI.join("_"))},
    			{ "name": "P_CDSEGMER", "operator": "=", "value": Util.isEmptyOrBlank(filterWidget.currentRow.CDSEGMER) ? "T" : ("_" + filterWidget.currentRow.CDSEGMER.join("_"))},
    			{ "name": "P_CDAREAGERE", "operator": "=", "value": Util.isEmptyOrBlank(filterWidget.currentRow.CDAREAGERE) ? "T" : ("_" + filterWidget.currentRow.CDAREAGERE.join("_"))},
    			{ "name": "P_CDFILIAL", "operator": "=", "value": Util.isEmptyOrBlank(filterWidget.currentRow.CDFILIAL) ? "T" : ("_" + filterWidget.currentRow.CDFILIAL.join("_"))}
    		 ];
            
            filterWidget.backUpRow = Util.clone(filterWidget.currentRow);
        
        
    		resumoWidget.dataSourceFilter = filtro; 
            this.setDataSourceFilterWidgets(filtro);
            this.ocultaMesesWidgets();
            this.setStripeLabel(filterWidget);
            
            ScreenService.closePopup(true);
            ScreenService.closeSwipe(true);
            tabCalculo.reload();
            resumoWidget.reload();
        }
    };
    
    /**
     * Função que chama a função de montagem dos dataSourceFilter das widgets
     * @param {filtro} array[obj]
    **/
    this.setDataSourceFilterWidgets = function(filtro){

        this.pushFiltroDataSourceFilter("GOP10000_FATURAMENTO", "FATURAMENTO", filtro);
        this.pushFiltroDataSourceFilter("GOP10000_IMPOSTO_FATURAMENTO", "IMPOSTO", filtro);
        this.pushFiltroDataSourceFilter("GOP10000_IMPOSTO_RECUPERAR", "IMPOSTOARECUPERAR", filtro);
        this.pushFiltroDataSourceFilter("GOP10000_IMPOSTO_LUCRO", "IMPOSTOSOBLUCRO", filtro);
        this.pushFiltroDataSourceFilter("GOP10000_CONSUMO", "CONSUMO", filtro);
        this.pushFiltroDataSourceFilter("GOP10000_DESPESA_MO", "MAOOBRA", filtro);
        this.pushFiltroDataSourceFilter("GOP10000_DESPESA_MATRIZ", "DESPMATRIZ", filtro);
        this.pushFiltroDataSourceFilter("GOP10000_DESPESA_DIVERSA", "DESPDIVERSA", filtro);
        this.pushFiltroDataSourceFilter("GOP10000_DESPESA_LANCENT", "DESPENTRADA", filtro);
        this.pushFiltroDataSourceFilter("GOP10000_DESPESA_DEPRECIACAO", "DEPRECIACAO", filtro);
        this.pushFiltroDataSourceFilter("GOP10000_TAXA_ADMINISTRATIVA", "TAXAADM", filtro);
        this.pushFiltroDataSourceFilter("GOP10000_DESPESA_INVESTIMENTO", "INVESTIMENTO", filtro);
    };
    
    /**
     * Função que complementa o filtro os dataSourceFilter das widgets com as 
     * particularidades de cada widget.
     * 
     * @param {filtro} Array[Objeto]
     *
     * Ex.: If nmWidget == GOP10000_FATURAMENTO then  P_IDTIPOORCAOP = "FATURAMENTO";
    **/
    this.pushFiltroDataSourceFilter = function(nmWidget, tpOrcamento, filtro){
        
        var widget = gopUtil.getWidgetContainer(nmWidget);
        var novoFiltro = Util.clone(filtro);
        novoFiltro.push(
            {"operator": "ORDER_BY",
             "name": [
                {"name": "CDFILIAL", "direction": "ASC" }
            ]
		});

        novoFiltro.push({"name": "P_IDTIPOORCAOP", "operator": "=",	"value": tpOrcamento});
        widget.dataSourceFilter = novoFiltro;
        widget.reload();
    };
    
    /**
     * Chama a função ocultaMes para cada aba da tela
     * 
     * @param {}
    **/
    this.ocultaMesesWidgets = function(){
        
        var impostoWidget = gopUtil.getWidgetContainer("GOP10000_IMPOSTO");
        var despesaWidget = gopUtil.getWidgetContainer("GOP10000_DESPESA");

        this.ocultaMes("GOP10000_FATURAMENTO");
        impostoWidget.widgets.forEach(function(widget){
            this.ocultaMes(widget.name);
        }.bind(this));
        this.ocultaMes("GOP10000_CONSUMO");
        despesaWidget.widgets.forEach(function(widget){
            this.ocultaMes(widget);
        }.bind(this));
        this.ocultaMes("GOP10000_TAXA_ADMINISTRATIVA");
        this.ocultaMes("GOP10000_DESPESA_DETALHAMENTO");
        this.ocultaMes("GOP10000_DESPESA_DETALHAMENTO_MES");
        
        var resumoWidget = gopUtil.getWidgetContainer("GOP10000_RESUMO");
        resumoWidget.activate();
    };
    
    this.detalheOrcamo = function(args){
        
        if(args.row.QTDPRODUTO > 0) {
            var widget               = args.owner;
            var filterWidget         = gopUtil.getWidgetContainer("filterSelect");
            var detalheOrcaMoTab     = gopUtil.getWidgetContainer("GOP10000_DETALHAMENTO_ORCAMO_TAB");
            var detalheOrcaMo        = gopUtil.getWidgetContainer("GOP10000_DETALHAMENTO_ORCAMO");
            
            var detalheMoTab      = gopUtil.getWidgetContainer("GOP10000_ORCAMO_ORCAMENTO_DETALHE_MES_TAB");
            var detalheMo         = gopUtil.getWidgetContainer("GOP10000_ORCAMO_ORCAMENTO_DETALHE_MES");
            
            var fieldName  = $(event.target).closest('.zh-standard-column').attr('data-zh-field-name');
            var arrayMeses = ["JANEIRO", "FEVEREIRO", "MARCO", "ABRIL", "MAIO", "JUNHO", "JULHO", "AGOSTO", "SETEMBRO", "OUTUBRO", "NOVEMBRO", "DEZEMBRO"];
            var nrperiodo  = filterWidget.currentRow.IDPERIODOORCA == "ANUAL" ? 1 : filterWidget.currentRow.NRPERIODOORCA;
            if (!arrayMeses.includes(fieldName)){
                return 0;
            }
            var mes = this.getNumeroMes(fieldName);
            var idtipoorcaop = widget.dataSourceFilter.filter(f=>f.name == "P_IDTIPOORCAOP")[0].value;
            var filtroImpostoDetalhe = [
                { "name": "P_NRANOORCAGOP", "operator": "=", "value": widget.currentRow.NRANOORCAGOP},
                { "name": "P_CDFILIAL","operator":"=","value":widget.currentRow.CDFILIAL},
    			{ "name": "P_IDPERIODOORCA", "operator": "=", "value": widget.currentRow.IDPERIODOORCA},
    			{ "name": "P_NRPERIODOORCA", "operator": "=", "value": nrperiodo},
    			{ "name": "P_CDOPERADOR", "operator": "=", "value": localStorageEngine.getLocalVar("CDOPERADOR")},
        		{ "name": "P_NRORCAMENTOGOP", "operator": "=",	"value": widget.currentRow.NRORCAMENTOGOP},
                { "name": "P_MES", "operator": "=",	"value": mes},
                { "name": "P_VALTOTAL","operator":"=","value":widget.currentRow[fieldName]}
            ];
            
            if(this.validaOrcamento(args.row)){
                detalheMoTab.getField('VRTOTALMES').readOnly = true;
                detalheMo.getField('VRORCAMENTO').inlineEditable = false;
            }else{
                detalheMoTab.getField('VRTOTALMES').readOnly = false;
                detalheMo.getField('VRORCAMENTO').inlineEditable = true;
            }
            
            detalheOrcaMoTab.getField('VRORCAMENTO_LABEL').label = gopUtil.initialUpperCase(fieldName);
            detalheOrcaMoTab.currentRow.VRTOTALMES = widget.currentRow[fieldName];
            detalheOrcaMo.retornoWidget    = widget.name;
            detalheOrcaMo.dataSourceFilter = filtroImpostoDetalhe;
            detalheOrcaMo.reload();
            ScreenService.openSwipe(detalheOrcaMoTab).then(function(){
                gopUtil.underBlockWidget();
            }.bind(this));
            
            this.readOnlyRows(widget, detalheOrcaMo, 1);
        }
        
    };
    
    this.detalheMo = function (args){
        
         var widget            = args.owner;
         var detalheOrcaMoTab  = gopUtil.getWidgetContainer("GOP10000_DETALHAMENTO_ORCAMO_TAB");
         var detalheOrcaMo     = gopUtil.getWidgetContainer("GOP10000_DETALHAMENTO_ORCAMO");
         var detalheMoTab      = gopUtil.getWidgetContainer("GOP10000_ORCAMO_ORCAMENTO_DETALHE_MES_TAB");
         var detalheMo         = gopUtil.getWidgetContainer("GOP10000_ORCAMO_ORCAMENTO_DETALHE_MES");
         
         
        var filtroImpostoDetalhe = [
                { "name": "P_CDFILIAL","operator":"=","value":detalheOrcaMo.currentRow.CDFILIAL},
        		{ "name": "P_NRORCAMENTOGOP", "operator": "=",	"value": widget.currentRow.NRORCAMENTOGOP},
        		{ "name": "P_CDORCAMENTOMO" , "operator": "=", "value": detalheOrcaMo.currentRow.CDORCAMENTOMO},
                { "name": "P_MES", "operator": "=",	"value": detalheOrcaMo.currentRow.MES},
                { "name": "P_VRITEMMO","operator":"=","value":detalheOrcaMo.currentRow.VRORCAMENTO}
            ];
        
        detalheMoTab.getField('VRORCAMENTO_LABEL').label = detalheOrcaMo.currentRow.NOME;
        detalheMoTab.currentRow.VRTOTALMES               = detalheOrcaMo.currentRow.VRORCAMENTO;
        detalheMo.dataSourceFilter = filtroImpostoDetalhe;
        detalheMo.reload();

        detalheMoTab.backUpRow      = angular.copy(detalheMoTab.currentRow.VRTOTALMES);
        detalheMo.dataSource.backUp = angular.copy(detalheMo.dataSource.data);  

        
        ScreenService.openSwipe(detalheMoTab).then(function(){
            gopUtil.underBlockWidget();
        }.bind(this));
        
    };
    
    this.cancelEditOrcaMO = function(widget){
        
        widget.getParent().currentRow.VRTOTALMES = widget.getParent().backUpRow;
        
        widget.reload();
        widget.list();
        this.controlaBotoesEdit(widget,false);
        
    };
    
    this.updateOrcaMo = function(widget, rows){
        
        gop10000_CalculoOrcamentoOperacional_Service.updateDetalheOrcaMo(rows).then(function(response){
            widget.list();
            widget.formTemplate = widget.template;
            widget.dataSource.reload().then(function(){
                widget.reload();
            });
        });
    };
    
    
	this.backOrcamentoMo = function(args){
        
        var widgetToActivate = gopUtil.getWidgetContainer('GOP10000_DESPESA_MO');
        
        ScreenService.closeSwipe().then(function(){
            widgetToActivate.activate();
            widgetToActivate.reload();
        });
	};
	
	this.backDetalheOrcaMo = function(widget){
	    
        var widgetToActivate = gopUtil.getWidgetContainer('GOP10000_DETALHAMENTO_ORCAMO');
      
        ScreenService.closeSwipe().then(function(){
            widgetToActivate.activate();
            widgetToActivate.reload();
        });


	};
	
	this.totalItens = function(widget){
	    
	    var vrTotaItens  = 0;
	    
	    widget.dataSource.data.forEach(function(row){
	        vrTotaItens += General.valueToFloat(row.VRORCAMENTO);
	    });
	    
	    return vrTotaItens;
	    
	};
	
	
    this.moveRowImpInv = function(args){
        
        if(args.row.QTDPRODUTO > 0){
            var widget = args.owner;
            var filterWidget = gopUtil.getWidgetContainer("filterSelect");
            var impostoDetalheWidgetTotal = gopUtil.getWidgetContainer("GOP10000_DETALHAMENTO_IMPINV_TAB");
            var impostoDetalheWidget = gopUtil.getWidgetContainer("GOP10000_DETALHAMENTO_IMPINV");
            var impostoDescriptionWidget = gopUtil.getWidgetContainer("GOP10000_IMPOSTO_FATURAMENTO");
            var fieldName = $(event.target).closest('.zh-standard-column').attr('data-zh-field-name');
            var arrayMeses = ["JANEIRO", "FEVEREIRO", "MARCO", "ABRIL", "MAIO", "JUNHO", "JULHO", "AGOSTO", "SETEMBRO", "OUTUBRO", "NOVEMBRO", "DEZEMBRO"];
            var nrperiodo = filterWidget.currentRow.IDPERIODOORCA == "ANUAL" ? 1 : filterWidget.currentRow.NRPERIODOORCA;
            if (!arrayMeses.includes(fieldName)){
                return 0;
            }
            var mes = this.getNumeroMes(fieldName);
            var idtipoorcaop = widget.dataSourceFilter.filter(f=>f.name == "P_IDTIPOORCAOP")[0].value;
            var filtroImpostoDetalhe = [
                { "name": "P_NRANOORCAGOP", "operator": "=", "value": filterWidget.currentRow.NRANOORCAGOP},
    			{ "name": "P_IDPERIODOORCA", "operator": "=", "value": filterWidget.currentRow.IDPERIODOORCA},
    			{ "name": "P_NRPERIODOORCA", "operator": "=", "value": nrperiodo},
    			{ "name": "P_CDOPERADOR", "operator": "=", "value": localStorageEngine.getLocalVar("CDOPERADOR")},
        		{ "name": "P_IDTIPOORCAOP", "operator": "=", "value": idtipoorcaop},
        		{ "name": "P_NRORCAMENTOGOP", "operator": "=",	"value": widget.currentRow.NRORCAMENTOGOP},
                { "name": "P_MES", "operator": "=",	"value": mes}
            ];
            var msg;
            
            if(widget.name.indexOf("GOP10000_IMPOSTO") != -1){
                msg = ScreenService.i18n("Total de Imposto de ");
                impostoDetalheWidget.label = ScreenService.i18n("Impostos");
            }else{
                msg = ScreenService.i18n("Total de Investimento de ");
                impostoDetalheWidget.label = ScreenService.i18n("Investimentos");
            }
            
            impostoDescriptionWidget.descriptionOrder = widget.currentRow.CDFILIAL + " - " + widget.currentRow.NMFILIAL;
            
            this.setDescriptionOrder(impostoDetalheWidget, widget);
    
            impostoDetalheWidgetTotal.getField('VRORCAMENTO_LABEL').label = msg + gopUtil.initialUpperCase(fieldName);
            impostoDetalheWidgetTotal.currentRow.VRTOTALMES = widget.currentRow[fieldName];
            impostoDetalheWidget.retornoWidget = widget.name;
            impostoDetalheWidget.dataSourceFilter = filtroImpostoDetalhe;
            impostoDetalheWidget.reload();
            ScreenService.openSwipe(impostoDetalheWidgetTotal).then(function(){
                gopUtil.underBlockWidget();
            }.bind(this));
            
            this.readOnlyRows(widget, impostoDetalheWidget, 1);
        }
    };
    
    this.moveRowLancEnt = function(args, widget){
        
        var filterWidget = gopUtil.getWidgetContainer("filterSelect");
        
        if(filterWidget.currentRow.IDORCDESPENTROP === "D"){
            this.moveRowDespesaItem(args, widget);
        }else{
            this.moveRowDespesa(args, widget, "gop10000_orcamento_detalhe_operacao");
        }
    };
    
    this.moveRowDespesaItem = function(args, widget){
        
        if(args.row.QTDPRODUTO > 0) {
            var filterWidget = gopUtil.getWidgetContainer("filterSelect");
            var despesaDetalheWidget = gopUtil.getWidgetContainer("GOP10000_DESPESA_DETALHAMENTO");
            var idtipoorcaop = widget.dataSourceFilter.filter(f=>f.name == "P_IDTIPOORCAOP")[0].value;
            var filtroDespesaDetalhe = [
                { "name": "P_NRANOORCAGOP", "operator": "=", "value": filterWidget.currentRow.NRANOORCAGOP},
    			{ "name": "P_IDPERIODOORCA", "operator": "=", "value": filterWidget.currentRow.IDPERIODOORCA},
    			{ "name": "P_NRPERIODOORCA", "operator": "=", "value": filterWidget.currentRow.NRPERIODOORCA},
    			{ "name": "P_CDOPERADOR", "operator": "=", "value": localStorageEngine.getLocalVar("CDOPERADOR")},
        		{ "name": "P_IDTIPOORCAOP", "operator": "=", "value": idtipoorcaop}
            ];
            
            this.setDescriptionOrder(despesaDetalheWidget, widget);
            
            if(widget.name == "GOP10000_DESPESA_MO"){
                despesaDetalheWidget.dataSource.name = "gop#/crud/gop10000_orcamento_detalhe_despesa_mo";
            }else{
                despesaDetalheWidget.dataSource.name = "gop#/crud/gop10000_orcamento_detalhe_despesa";
            }
            
            /*if(widget.name == "GOP10000_DESPESA_MATRIZ"){
                filtroDespesaDetalhe.push({name: "P_IDDESPPREV", operator: "=", value: 'MATRIZ' });
                //filtroDespesaDetalhe.push({name: "P_NRPERCITEMGOP", operator: "=", value: 5});
                
            }else if(widget.name == "GOP10000_DESPESA_DIVERSA"){
                filtroDespesaDetalhe.push({name: "P_IDDESPPREV", operator: "=", value: 'DIVERSA' });
                //filtroDespesaDetalhe.push({name: "P_NRPERCITEMGOP", operator: "=", value: 6});
                
            }else if(widget.name == "GOP10000_DESPESA_LANCENT"){
                filtroDespesaDetalhe.push({name: "P_IDDESPPREV", operator: "=", value: 'LANCENTRADA' });
                //filtroDespesaDetalhe.push({name: "P_NRPERCITEMGOP", operator: "=", value: 7});
            } */
            
            despesaDetalheWidget.retornoWidget = widget.name;
            filtroDespesaDetalhe.push({
    			"name": "P_NRORCAMENTOGOP", "operator": "=",	"value": widget.currentRow.NRORCAMENTOGOP
    		});
            despesaDetalheWidget.dataSourceFilter = filtroDespesaDetalhe;
            despesaDetalheWidget.beginWidgetName = widget.name;
            despesaDetalheWidget.reload();
            despesaDetalheWidget.parent = widget;
            ScreenService.openSwipe(despesaDetalheWidget).then(function(){
                gopUtil.underBlockWidget();
            });
        }
    };
    
    this.moveRowDespesa = function(args, widget, dataSourceName){
        
        if(args.row.QTDPRODUTO > 0) {
            var filterWidget = gopUtil.getWidgetContainer("filterSelect");
            var despesaDetalheWidget = gopUtil.getWidgetContainer("GOP10000_DESPESA_DETALHAMENTO_MES");
            var idtipoorcaop = widget.dataSourceFilter.filter(f=>f.name == "P_IDTIPOORCAOP")[0].value;
            var filtroDespesaDetalhe = [
                { "name": "P_NRANOORCAGOP", "operator": "=", "value": filterWidget.currentRow.NRANOORCAGOP},
    			{ "name": "P_IDPERIODOORCA", "operator": "=", "value": filterWidget.currentRow.IDPERIODOORCA},
    			{ "name": "P_NRPERIODOORCA", "operator": "=", "value": filterWidget.currentRow.NRPERIODOORCA || 1},
    			{ "name": "P_CDOPERADOR", "operator": "=", "value": localStorageEngine.getLocalVar("CDOPERADOR")},
        		{ "name": "P_IDTIPOORCAOP", "operator": "=", "value": idtipoorcaop}
            ];
            
            this.setDescriptionOrder(despesaDetalheWidget, widget);
            despesaDetalheWidget.dataSource.name = "gop#/crud/" + dataSourceName;
            despesaDetalheWidget.retornoWidget = widget.name;
            filtroDespesaDetalhe.push(
                {"name": "P_NRORCAMENTOGOP", "operator": "=",	"value": widget.currentRow.NRORCAMENTOGOP}
            );
            despesaDetalheWidget.dataSourceFilter = filtroDespesaDetalhe;
            despesaDetalheWidget.reload();
            ScreenService.openSwipe(despesaDetalheWidget).then(function(){
                gopUtil.underBlockWidget();
            }.bind(this));
            
            this.readOnlyRows(widget, despesaDetalheWidget, 1);
        }
    };
    
    this.getFiltroItemMes = function(row, mes){
        
        var filtroMes = [
		    {name: "P_NRANOORCAGOP",  operator: "=", value: row.NRANOORCAGOP},
			{name: "P_IDPERIODOORCA", operator: "=", value: row.IDPERIODOORCA},
			{name: "P_NRPERIODOORCA", operator: "=", value: row.NRPERIODOORCA},
			{name: "P_CDFILIAL",      operator: "=", value: row.CDFILIAL},
			{name: "P_MES",           operator: "=", value: mes},
			{name: "P_NRITEMORCAOP",  operator: "=", value: row.NRITEMORCAOP}
		];
		return filtroMes;
    };
    
    this.getFiltroMaoObra = function(row, mes){
        
        var filtro = [
            {name: "P_CDOPERADOR", operator: "=", value: localStorageEngine.getLocalVar("CDOPERADOR")},
            {name: "P_NRORCAMENTOGOP", operator: "=", value: row.NRORCAMENTOGOP},
            {name: "P_NRIDCODIGO", operator: "=", value: row.CODIGO},
            {name: "P_NRANOORCAGOP", operator: "=", value: row.NRANOORCAGOP},
            {name: "P_NRPERIODOORCA", operator: "=", value: row.NRPERIODOORCA},
            {name: "P_IDPERIODOORCA", operator: "=", value: row.IDPERIODOORCA},
            {name: "P_MES", operator: "=", value: mes}
        ];
        return filtro;
    };
    
    this.getChangedMeses = function(data, array){
        
        var changed = [];
        
        array.forEach(function(mes){
            if(!Util.isEmptyOrBlank(data['changed_' + mes])){
                changed.push(mes);
            }
        });
        return changed;
    };
    
    this.getValueMeses = function(arrayName, arrayObj){
        var dataValue = [];
        arrayName.forEach(function(name){
            arrayObj.forEach(function(obj){
                if(name == obj.name.latinize().toUpperCase())
                    dataValue.push(obj.value);
            });
        });
        return dataValue;
    };
    
    this.saveTotalImposto = function(detalheWidget){
        
        var detalheWidgetTotal = detalheWidget.getParent();
        var data = detalheWidget.dataSource.data;
        var codigos = "";
        var parametros;
        
        detalheWidget.dataSource.data.forEach(function(row){
            if(Util.isEmptyOrBlank(codigos)){
                codigos = "'" + row.NRITEMORCAOP + "'";
            }else{
                codigos += ", '" + row.NRITEMORCAOP + "'";
            }
        });
        
        parametros = {nritemorcaop: codigos, MES: data[0].MES, VRTOTALMES: parseFloat(detalheWidgetTotal.currentRow.VRTOTALMES)};
        gop10000_CalculoOrcamentoOperacional_Service.updateTotalImposto([parametros]).then(function(){
            detalheWidget.reload();
            detalheWidget.getParent().backUpRow = Util.clone(detalheWidget.getParent().currentRow);
            this.editaTotalImposto(false);
            detalheWidget.list();
        }.bind(this));
    };
     
    this.saveDetalhamentoImposto = function(widget){
        
        var data = widget.dataSource.data;
        var rows = data.filter(row => row.VRORCAMENTO_inline !=='undefined');
        var soma = 0;
        
        widget.dataSource.data.forEach(function(row){
	        soma += General.valueToFloat(row.VRORCAMENTO);
	    });
	    
        rows.forEach(function(row){
            row.VRORCAMENTO = General.valueToFloat(row.VRORCAMENTO);
        });
        
        widget.getParent().currentRow.VRTOTALMES = soma.toFixed(2);
        widget.getParent().backUpRow.VRTOTALMES  = soma.toFixed(2); //para verificação de edição do total manualmente
        widget.dataSource.backUp            = widget.dataSource.data;
        this.updateImposto(widget, rows);
        this.controlaBotoesEdit(widget,false);
	    
    };
    
    this.updateImposto = function(widget, rows){
        
        gop10000_CalculoOrcamentoOperacional_Service.updateDetalheImposto(rows).then(function(response){
            widget.list();
            widget.formTemplate = widget.template;
            ScreenService.successNotification('Salvo com sucesso');
            widget.dataSource.reload().then(function(){
                widget.reload();
            });
        });
    };

	this.saveMeses = function(widget){
	    
        if(widget.isValid()){
            var arrayMeses = gopUtil.getDataMensal();
    	    var arrayCampos = this.getArrayCampos();
            var rows = this.getRows(widget);
            ScreenService.successNotification("Operação realizada com sucesso");
            rows = this.parseToFloatRows(rows);
            rows.forEach(function(data){
                var changed = this.getChangedMeses(data, arrayCampos);
                data.changedFields = this.getValueMeses(changed, arrayMeses);
            }.bind(this));
            
            gop10000_CalculoOrcamentoOperacional_Service.updateMeses(rows).then(function(response){
                widget.list();
                widget.formTemplate = widget.template;
                widget.dataSource.reload().then(function(){
                    widget.reload();
                });
            });
        }
	};
	
	/**
     * Função para tratar os valores editados na widget de Detalhamento de Produtos.
     * @param {widget} widget ativa
     *  Chamada na action "saveEditaTotal" e no event
     * "WidgetOnSaveInlineEdit", ambos da widget "GOP10000_DETALHAMENTO_MENSAL"
    **/
    
    this.saveDetalhamento = function(widget){
        
        var consumoWidget   = gopUtil.getWidgetContainer('GOP10000_CONSUMO');
        var totalWidget     = widget.getParent();
        var soma            = 0;
        
        this.saveTotalDetalhamentoMes(widget, consumoWidget);
        this.controlaBotoesEdit(widget,false);

        widget.currentRow.NRITEMORCAOP = totalWidget.currentRow.NRITEMORCAOP;
        widget.dataSource.data.forEach(function(row){
	        soma += General.valueToFloat(row.VRORCAMENTO);
	    });
	    
        widget.getParent().currentRow.VRTOTALMES = soma.toFixed(2);
        widget.getParent().backUpRow.VRTOTALMES  = soma.toFixed(2); 
        widget.dataSource.backUp            = widget.dataSource.data; //para verificação de edição do total manualmente
	       
    };
    /**
     * Função que efetivamente salva o total manualmente alterado
     * @param {widget, consumoWidget, soma} widget ativa, "GOP10000_CONSUMO"
     *  Chamada nas funções "confirmaIdaltManual" e "verificaSomaSaveTotalMes".
     **/
    this.saveTotalDetalhamentoMes = function(widget, consumoWidget){
        
	    var vrtotalmes = General.valueToFloat(widget.getParent().currentRow.VRTOTALMES);
	    var data = {}; 
	    data.itensDetalhe = [];

        var nritorcamesgop;

	    if(widget.beginWidgetName == "GOP10000_DESPESA_MO"){
	 
            nritorcamesgop = this.getMesesDespesaMo(widget.dataSource.data);
            data.itemorca  = 'MAOOBRA';

        }else{
            
            nritorcamesgop = widget.getParent().currentRow.NRITORCAMESGOP;
            widget.getParent().currentRow.__is_new = false;
            widget.getParent().currentRow.VRTOTALMES = vrtotalmes;
            widget.getParent().backUpRow.VRTOTALMES = vrtotalmes; //para verificação de edição do total manualmente
            data.itemorca ='DEMAISDESPESAS';
        }
        
        for(var index=0; index < widget.dataSource.data.length; index++){
            var item            = widget.dataSource.data[index];
            data.itensDetalhe[index] = item;
            data.itensDetalhe[index].VRORCAMENTO = General.valueToFloat(data.itensDetalhe[index].VRORCAMENTO);
        }

        gop10000_CalculoOrcamentoOperacional_Service.updateTotalDetalhe([data]).then(function(){
            this.afterUpdateTotal(widget, consumoWidget);
        }.bind(this));
	};
	
	this.getMesesDespesaMo = function(dataSource){
	    var stringNritorcamesgops;
	    dataSource.forEach(function(row){
            if(Util.isEmptyOrBlank(stringNritorcamesgops)){
                stringNritorcamesgops = row.NRITORCAMESGOP;
            }else{
                stringNritorcamesgops += ", " + row.NRITORCAMESGOP;
            }
	    });
	    return stringNritorcamesgops;
	};
	
	this.afterUpdateTotal = function(widget, consumoWidget){
	    widget.list();
	    ScreenService.successNotification('Salvo com sucesso');
        widget.formTemplate = widget.template;
        consumoWidget.reload();
        widget.reload();
        widget.getParent().list();  
        widget.getParent().childTemplate = "widget/form.html";
        this.controlaBotoesEdit(widget,false);
	};

	this.verificaIdaltManual = function(widget, soma){
	    var dataIdaltmanual = widget.getParent().dataSource.data[0].IDALTMANUAL;
	    var total = General.valueToFloat(widget.getParent().currentRow.VRTOTALMES);
	    var flag = true;
	    if(dataIdaltmanual == 'S' || widget.getParent().currentRow.VRTOTALMES != widget.getParent().backUpRow.VRTOTALMES){
    	    if(soma > total){
    	        flag = false;
    	    }
	    }
	    return flag;
	};
	
	this.saveOrcaMo = function(widget){
	    
        var rows = this.getRows(widget);
        rows.forEach(function(row){
            row.VRORCAMENTO = General.valueToFloat(row.VRORCAMENTO);
        });
            
        if(widget.name == "GOP10000_ORCAMO_ORCAMENTO_DETALHE_MES"){
            rows = this.parseToFloatRows(rows);
            gop10000_CalculoOrcamentoOperacional_Service.updateDetalheDespesaMo(rows).then(function(retorno){
                this.afterSaveItens(widget);
            }.bind(this));
        }else{
    	    gop10000_CalculoOrcamentoOperacional_Service.atualizaItemTotal(rows).then(function(retorno){
                this.afterSaveItens(widget);
            }.bind(this));
        }
	};
	
	
	this.AtualizaTotal = function(widget){
	    
	    var valorTotal = 0;
	    
	    widget.dataSource.data.forEach(function(row){
	        valorTotal += Number(row.VRORCAMENTO);
	    });
	    
	    widget.getParent().currentRow.VRTOTALMES = valorTotal;
	    
	};

    this.afterSaveItens = function(widget){
        
        var vrTotalDetalhes = 0;
        widget.list();
        widget.formTemplate     = widget.template;
        
        var widgetTotalAnterior = gopUtil.getWidgetContainer("GOP10000_DETALHAMENTO_ORCAMO");
        
        widgetTotalAnterior.reload();
        ScreenService.successNotification('Salvo com sucesso');
        widget.dataSourceFilter.filter(function(row){
        	return row.name === 'P_VRITEMMO';
        })[0].value =widget.getParent().currentRow.VRTOTALMES;
        
   
        widget.reload();
        
        this.controlaBotoesEdit(widget,false);
	};
	
	this.recalculaTotaisMO = function(){
	    
	   var totalorcamoTab      = 0;
	   var totalorcadetalheTab = 0;
	    
	   var orcamoTab      = gopUtil.getWidgetContainer('GOP10000_DETALHAMENTO_ORCAMO_TAB');
	   var orcamo         = gopUtil.getWidgetContainer('GOP10000_DETALHAMENTO_ORCAMO');
	   var orcadetalheTab = gopUtil.getWidgetContainer('GOP10000_ORCAMO_ORCAMENTO_DETALHE_MES_TAB');
	   var orcadetalheMes = gopUtil.getWidgetContainer('GOP10000_ORCAMO_ORCAMENTO_DETALHE_MES');
	   
	   
	   orcamo.dataSource.data.forEach(function(row){
        	totalorcamoTab += Number(row.VRORCAMENTO);
       });
        
       totalorcamoTab = (totalorcamoTab - Number(orcamo.currentRow.VRORCAMENTO));
                        
       orcadetalheMes.dataSource.data.forEach(function(row){
        	totalorcadetalheTab += Number(row.VRORCAMENTO);
       });
     
       totalorcamoTab += totalorcadetalheTab;    
        
	  
	   orcamoTab.currentRow.VRTOTALMES      = totalorcamoTab;
	   orcadetalheTab.currentRow.VRTOTALMES = totalorcadetalheTab; 
	   
	   return;
	   
	};
	
	this.controlaBotoesEdit = function(widget,bool){
	    
	  
	    if (!Util.isEmptyOrBlank(widget.getAction('ACT_saveEditaTotal'))){
	         widget.getAction('ACT_saveEditaTotal').isVisible = bool;
	    }
	  
	    if(!Util.isEmptyOrBlank(widget.getAction("ACT_cancelEditaTotal"))){
	        widget.getAction("ACT_cancelEditaTotal").isVisible = bool;
	    }
        
        if(!Util.isEmptyOrBlank(widget.getAction("ACT_back"))){
           widget.getAction("ACT_back").isVisible = !bool;
        }
        
	};
    
    this.editaTotal = function(widget){
        
        /****
        Ao editar o campo de total do item imposto o valor digitado deve ser distruibuido entre 
        os itens de detalhamento. Esta distribuição pode ser em partes proporcionais
        ao percencual ja existente no item ou em partes iguais (em casos onde não há percentual informado)
        ****/
        
        var detalheWidget = widget.widgets[0];
        var vrTotalRecemModificado = General.valueToFloat(detalheWidget.getParent().currentRow.VRTOTALMES);
        var percentual    = 0;
        var percentualAnt = 0;
        var totalPercent  = 0;
        
        for(var indexPercent =0; indexPercent < detalheWidget.dataSource.data.length; indexPercent++){
            totalPercent += Number(detalheWidget.dataSource.data[indexPercent].VRPERCENTUAL);
        }
            
        
        for(var index = 0; index < detalheWidget.dataSource.data.length; index++){
            

            percentual = Number(detalheWidget.dataSource.data[index].VRPERCENTUAL);
            
            if(vrTotalRecemModificado === 0){
                
                detalheWidget.dataSource.data[index].VRORCAMENTO        = 0;
                detalheWidget.dataSource.data[index].VRORCAMENTO_inline = 0;
                detalheWidget.dataSource.data[index].VRPERCENTUAL       = 0;
                
            }else{ 
                
                if(totalPercent === 0 ||  percentual === null || percentual === undefined){
                
                    detalheWidget.dataSource.data[index].VRORCAMENTO        = (vrTotalRecemModificado / detalheWidget.dataSource.data.length);
                    detalheWidget.dataSource.data[index].VRORCAMENTO_inline = detalheWidget.dataSource.data[index].VRORCAMENTO;
                    detalheWidget.dataSource.data[index].VRPERCENTUAL       = (detalheWidget.dataSource.data[index].VRORCAMENTO * 100) / vrTotalRecemModificado;
                    
                }else{
                    
                    if(percentualAnt >100){
                        
                        detalheWidget.dataSource.data[index].VRORCAMENTO        = 0;
                        detalheWidget.dataSource.data[index].VRORCAMENTO_inline = 0;
                        detalheWidget.dataSource.data[index].VRPERCENTUAL       = 0;
                        
                    }else{
                        
                        detalheWidget.dataSource.data[index].VRORCAMENTO        = (vrTotalRecemModificado / 100) * percentual;
                        detalheWidget.dataSource.data[index].VRORCAMENTO_inline = detalheWidget.dataSource.data[index].VRORCAMENTO;
                        detalheWidget.dataSource.data[index].VRPERCENTUAL       = (detalheWidget.dataSource.data[index].VRORCAMENTO * 100) / vrTotalRecemModificado;
                        
                    }
                }
                
            }
             percentualAnt = percentual;
        }
        
        detalheWidget.activate();
        
        this.controlaBotoesEdit(detalheWidget,true);
    };
    
    this.editaItem = function(detalheWidget){
        
        var vritemAcumulado = 0;
        
        for(var index = 0; index < detalheWidget.dataSource.data.length;index++){
            
            if(detalheWidget.dataSource.data[index].CODIGO === detalheWidget.currentRow.CODIGO){
                 vritemAcumulado += General.valueToFloat(detalheWidget.currentRow.VRORCAMENTO);
            }else{
                vritemAcumulado += General.valueToFloat(detalheWidget.dataSource.data[index].VRORCAMENTO);  
            }
        }
        
        detalheWidget.getParent().currentRow.VRTOTALMES = vritemAcumulado ;
        
        /*Apos a edição de um item especifico, é necessario realizar o recalculo dos percentuais de todos os itens, com base no novo total*/
        for(var item = 0; item < detalheWidget.dataSource.data.length;item++){
            detalheWidget.dataSource.data[item].VRPERCENTUAL = (General.valueToFloat(detalheWidget.dataSource.data[item].VRORCAMENTO) * 100) / vritemAcumulado;
        }

        this.controlaBotoesEdit(detalheWidget,true);
        
    };
    
    this.editaItemImposto = function(){
        
        var detalheWidget   = gopUtil.getWidgetContainer('GOP10000_DETALHAMENTO_IMPINV');
        var vritemAcumulado = 0;
        
        for(var index = 0; index < detalheWidget.dataSource.data.length;index++){
            
            if(detalheWidget.dataSource.data[index].CODIGO === detalheWidget.currentRow.CODIGO){
                vritemAcumulado += General.valueToFloat(detalheWidget.currentRow.VRORCAMENTO);
            }else{
                vritemAcumulado += General.valueToFloat(detalheWidget.dataSource.data[index].VRORCAMENTO);  
            }
        }
        
        if(vritemAcumulado != detalheWidget.getParent().backUpRow.VRTOTALMES){

            detalheWidget.getParent().currentRow.VRTOTALMES = vritemAcumulado ;
            
            /****
            Ao realizar a edição de um item especifico, é necessario realizar o
            recalculo dos percentuais deste e de todos os itens de detalhamento
            com base no novo total
            ****/
              
            for(var item = 0; item < detalheWidget.dataSource.data.length;item++){
                detalheWidget.dataSource.data[item].VRPERCENTUAL = (General.valueToFloat(detalheWidget.dataSource.data[item].VRORCAMENTO) * 100) / vritemAcumulado;
            }

            this.controlaBotoesEdit(detalheWidget,true);
        }
    };
    
    
    /**
     * Função retorna à tela anterior ao detalhamento de produtos (consumo ou despesa).
     * @param {args} widget ativa
     * Chamada na 'action' "back" da widget 
     * GOP10000_DETALHAMENTO_MENSAL 
    **/
    this.backConsDesp = function(widget){
        var widgetToActivate = gopUtil.getWidgetContainer('GOP10000_DESPESA_DETALHAMENTO');
        
        if(widget.lastActiveWidget.name != "GOP10000_DESPESA_DETALHAMENTO"){
            if(widget.getParent().getField('VRORCAMENTO_LABEL').label.indexOf('Consumo') != -1){
                widgetToActivate = gopUtil.getWidgetContainer('GOP10000_CONSUMO');
            }
        } else {
            if (widget.name == 'GOP10000_DETALHAMENTO_MENSAL') {
                widget.getParent().lastActiveWidget = gopUtil.getWidgetContainer('GOP10000_DESPESA');
            }
            
        }
        ScreenService.closeSwipe().then(function(){
            widgetToActivate.activate();
            widgetToActivate.reload();
        });
    };
    
    this.voltar = function(widget){
        
        if(widget.lastActiveWidget){
            widget.lastActiveWidget.activate();
        }
        ScreenService.closePopup();
    };
    
    this.cancelEditaTotal = function(widget){
        
        widget.getParent().currentRow.VRTOTALMES = widget.getParent().backUpRow.VRTOTALMES;
        widget.dataSource.data = angular.copy(widget.dataSource.backUp);
        widget.reload();
    
        this.controlaBotoesEdit(widget,false);
    };
    
    this.setDetalhamento = function(detalheWidget, dataSource, dataSourceTotal, label){
        
        detalheWidget.label = label;
        detalheWidget.dataSource.name = "gop#/crud/" + dataSource;
        detalheWidget.getParent().dataSource.name = "gop#/crud/" + dataSourceTotal;
        
    };
    
    this.openSwipeDespesaMo = function(detalheWidget, row, mes, field, msg){
        var detalheWidgetTotal = detalheWidget.getParent();
        var filtro = this.getFiltroMaoObra(row, mes);
        
        this.setDetalhamento(detalheWidget, "gop10000_item_mes_detalhe_mo", "gop10000_itorcamesgop_mo", "Despesas");
        detalheWidget.dataSourceFilter = filtro;
        detalheWidget.reload().then(function(){
            detalheWidgetTotal.dataSourceFilter = [
                { "name": "P_NRITORCAMESGOP", "operator": "=", "value": detalheWidget.dataSource.data[0].NRITORCAMESGOP},
                { "name": "P_NRIDCODIGO", "operator": "=", "value": row.CODIGO},
                { "name": "P_NRORCAMENTOGOP", "operator": "=", "value": row.NRORCAMENTOGOP}
                
            ];

            detalheWidgetTotal.reload().then(function(){
                var total_itens = 0;

                for (var index  = 0; index < detalheWidget.dataSource.data.length; index++){
                    total_itens += detalheWidget.dataSource.data[index].VRORCAMENTO;
                }
                detalheWidgetTotal.currentRow.VRTOTALMES = total_itens.toFixed(2);
            });
        });
        this.openDetalhamentoSwipe(detalheWidget, field, msg);
    };
    
    this.openSwipeDespesa = function(detalheWidget, row, mes, field, msg){
        var detalheWidgetTotal = detalheWidget.getParent();
        var filtro = this.getFiltroItemMes(row, mes);
        
        this.setDetalhamento(detalheWidget, "gop10000_item_mes_detalhe", "gop10000_itorcamesgop", "Produtos");
        gop10000_CalculoOrcamentoOperacional_Service.getDataSourceItemMes(filtro).then(function(callback) {
            var retorno = callback.dataset.gop10000_item_mes[0];
            if(Boolean(retorno)){
                if(retorno.QTDEDETMES > 0){
                    detalheWidgetTotal.currentRow.NRITORCAMESGOP = retorno.NRITORCAMESGOP;
                    detalheWidget.dataSourceFilter = [
                        { "name": "P_NRITORCAMESGOP", "operator": "=", "value": retorno.NRITORCAMESGOP }
                    ];
                    detalheWidgetTotal.dataSourceFilter = [
                        { "name": "P_NRITORCAMESGOP", "operator": "=", "value": retorno.NRITORCAMESGOP}
                        
                    ];
                    detalheWidget.reload().then(function(){
                        detalheWidgetTotal.currentRow.VRTOTALMES = 0;
                        detalheWidgetTotal.reload().then(function(){
                        
                            var total_itens = 0;
                            for (var index  = 0; index < detalheWidget.dataSource.data.length; index++){
                                total_itens += Number(detalheWidget.dataSource.data[index].VRORCAMENTO);
                            }
                            detalheWidgetTotal.currentRow.VRTOTALMES = total_itens.toFixed(2);
                        });

                    });
                    this.openDetalhamentoSwipe(detalheWidget, field, msg);
                }
            }
        }.bind(this), function(erro) {
            ScreenService.showMessage(erro);
            console.log(erro);
        });
                
    };
    

    this.cellClick = function(widget){
        
        var consumoWidget = gopUtil.getWidgetContainer("GOP10000_CONSUMO");
        var detalheWidget = gopUtil.getWidgetContainer("GOP10000_DETALHAMENTO_MENSAL");
        if(!event)
            return 0;
        var fieldName = $(event.target).closest('.zh-standard-column').attr('data-zh-field-name');
        var arrayMeses = ["JANEIRO", "FEVEREIRO", "MARCO", "ABRIL", "MAIO", "JUNHO", "JULHO", "AGOSTO", "SETEMBRO", "OUTUBRO", "NOVEMBRO", "DEZEMBRO"];
        var field = widget.getField(fieldName);
        
        if (!arrayMeses.includes(fieldName)){
            return 0;
        }

        detalheWidget.beginWidgetName = (widget.name == consumoWidget.name) ? widget.name : widget.beginWidgetName;

        if(widget.mode != 'inline_edit'){
            var row = widget.currentRow;
            var mes = this.getNumeroMes(fieldName);

            var inline = true;
            var msg = ScreenService.i18n("Total de Consumo de ");
            
            if(widget.name == 'GOP10000_DESPESA_DETALHAMENTO'){
                msg = ScreenService.i18n("Total de Despesa de ");
                consumoWidget.descriptionOrder = "";
                
                if(field.inlineEditable){
                   inline = false;
                }
            }
            
            if(inline){
                if(widget.beginWidgetName == "GOP10000_DESPESA_MO"){
                    this.openSwipeDespesaMo(detalheWidget, row, mes, field, msg);
                }else{
                    this.openSwipeDespesa(detalheWidget, row, mes, field, msg);
                }
            }   
        }
    };
    
    this.openDetalhamentoSwipe = function(detalheWidget, field, msg){
        var detalheWidgetTotal = detalheWidget.getParent();
        var mes   = field.name.toLowerCase();
        var array = mes.split("");
        
        array[0] = array[0].toUpperCase();
        mes = array.join("");

        this.readOnlyRows(field.widget, detalheWidget, 1);
        ScreenService.openSwipe(detalheWidgetTotal).then(function(){
            detalheWidgetTotal.getField('VRORCAMENTO_LABEL').label = msg + gopUtil.initialUpperCase(field.name);
            gopUtil.underBlockWidget();
        });
    };
    
    /**
     * Função que mantém o template da widget como "widget/grid.html", pois ao cancelar o inline
     * no swipe, o 'formTemplate' da widget é alterado para "widget/form.html".
     * @param {widget} widget ativa
     * Chamada na 'action' cancel da widget
     * GOP10000_DESPESA_DETALHAMENTO 
    **/
    this.cancelInline = function(widget){
        widget.reload();
        widget.list();
        widget.formTemplate = widget.template;
    };
    
    /**
     * Função que retorna o número de um mês a partir do nome(string)
     * @param {nmMes} nmMes String
    **/
    this.getNumeroMes = function(nmMes){
        var calendario = { 
            "JANEIRO": 1, "FEVEREIRO": 2, "MARCO": 3,
            "ABRIL": 4, "MAIO": 5, "JUNHO": 6, 
            "JULHO": 7, "AGOSTO": 8, "SETEMBRO": 9,
            "OUTUBRO": 10, "NOVEMBRO": 11, "DEZEMBRO": 12
        };
        return calendario[nmMes] ? calendario[nmMes] : [];

    };
    
    /**
     * Função que retorna o número de meses por "etapa" do período passado
     * como parâmetro 
     * @param {idperiodo} idperiodo String filtro.IDPERIODOORCA
     * Chamada na função 'ocultaMes'
    **/
    this.getNrMeses = function(idperiodo){
        if(idperiodo == "MENSAL"){
            return 1;
        }else if(idperiodo == "BIMESTRAL"){
            return 2;
        }else if(idperiodo == "TRIMESTRAL"){
            return 3;
        }else if(idperiodo == "QUADRIMESTRAL"){
           return 4;
        }else if(idperiodo == "SEMESTRAL"){
            return 6;
        }else if(idperiodo == "ANUAL"){
            return 12;
        }
    };
    
    /**
     * Função que altera a visibilidade dos campos de acordo
     * com o período filtrado.
     * 
     * @param {widget} widget ativa
     * Chamada no evento "WidgetAfterLoadDataSource"
     * das widgets com o grid detalhado por Mês.
    **/
    this.ocultaMes = function(widgetName){
        var widget = gopUtil.getWidgetContainer(widgetName);
        var filterSelect = gopUtil.getWidgetContainer("filterSelect");
        var idperiodo = filterSelect.currentRow.IDPERIODOORCA;
        var nrperiodo = gopUtil.parseFloat(filterSelect.currentRow.NRPERIODOORCA);
        var mesInicial = gopUtil.parseFloat(filterSelect.currentRow.NRMESINIORCA);
        var nrMeses = this.getNrMeses(idperiodo);
        
        this.setFieldsMeses(nrMeses, nrperiodo, mesInicial, widget);

        widget.getField("TOTAL").isVisible = (idperiodo != "MENSAL");
        widget.dataSource.data = Util.isEmptyOrBlank(widget.dataSourceFilter) ? [] : widget.dataSource.data;
       
        templateManager.updateTemplate();
    };
    
    /**
     * Função que altera o nome dos campos de acordo com o período filtrado.
     * 
     * @param {nrMeses, periodo, mesInicial, widget} quantidade de meses visíveis, 
     * NRPERIODO filtrado, NRMESINIORCA parametrizado, widget Widget ativa
     * 
     * Exemplo: If 
     *              nrMeses == 3 and nrperiodo == 2 and mesInicial == 4
     *          Then
     *              O primeiro field deve representar Abril;
     *              Existem 3 fields visíveis, e devem representar os meses de
     *              Abril, Maio e Junho na widget. 
    **/
    this.setFieldsMeses = function(nrMeses, nrperiodo, mesInicial, widget){
        var indice = widget.fields.indexOf(widget.fields.filter(function(obj){
            return obj.mesInicial;
        })[0]);
        
        var meses = [ {},
            {name: "JANEIRO",  label: "JAN"}, {name: "FEVEREIRO", label: "FEV"},
            {name: "MARCO",    label: "MAR"}, {name: "ABRIL",     label: "ABR"},
            {name: "MAIO",     label: "MAI"}, {name: "JUNHO",     label: "JUN"},
            {name: "JULHO",    label: "JUL"}, {name: "AGOSTO",    label: "AGO"},
            {name: "SETEMBRO", label: "SET"}, {name: "OUTUBRO",   label: "OUT"},
            {name: "NOVEMBRO", label: "NOV"}, {name: "DEZEMBRO",  label: "DEZ"}
        ];
        
        var mes = nrperiodo == 1 ? mesInicial : parseInt(moment(mesInicial, 'MM').add(nrMeses * (nrperiodo - 1), 'months').format('MM'));
       
        widget.fields[indice].name = meses[mes].name;
        widget.fields[indice].label = meses[mes].label;

        for(var y = 1; y < 12; y++){
            widget.fields[indice + y].isVisible = false;
        }
        
        if(nrMeses != 1){
            for (var i = 1; i < nrMeses; i++){
                widget.fields[indice + i].name = meses[gopUtil.parseFloat(moment(mes, "MM").add(i, 'months').format('MM'))].name; 
                widget.fields[indice + i].label = meses[gopUtil.parseFloat(moment(mes, "MM").add(i, 'months').format('MM'))].label;
                widget.fields[indice + i].isVisible = true;
            }
            this.totalizadorMes(widget, meses, nrMeses, mes);
        }
        
        widget.reload();
    };
    
    /**
     * Função que realiza a soma mensal por unidadee preenche
     * a propriedade TOTAL em de cada row do dataSource passado
     * @param {dataSource} dataSource da widget ativa
     * Chamada na função 'ocultaMes'
    **/
    this.totalizadorMes = function(widget, meses, nrMeses, mes){
        
        if(!meses){
            var filterSelect = gopUtil.getWidgetContainer("filterSelect");
            var idperiodo = filterSelect.currentRow.IDPERIODOORCA;
            var nrperiodo = gopUtil.parseFloat(filterSelect.currentRow.NRPERIODOORCA);
            meses = [ {},
                {name: "JANEIRO",  label: "JAN"}, {name: "FEVEREIRO", label: "FEV"},
                {name: "MARCO",    label: "MAR"}, {name: "ABRIL",     label: "ABR"},
                {name: "MAIO",     label: "MAI"}, {name: "JUNHO",     label: "JUN"},
                {name: "JULHO",    label: "JUL"}, {name: "AGOSTO",    label: "AGO"},
                {name: "SETEMBRO", label: "SET"}, {name: "OUTUBRO",   label: "OUT"},
                {name: "NOVEMBRO", label: "NOV"}, {name: "DEZEMBRO",  label: "DEZ"}
            ];
            nrMeses = this.getNrMeses(idperiodo);
            var mesInicial = gopUtil.parseFloat(filterSelect.currentRow.NRMESINIORCA);
            mes = nrperiodo == 1 ? mesInicial : parseInt(moment(mesInicial, 'MM').add(nrMeses * (nrperiodo - 1), 'months').format('MM'));
        }
        
        if(widget.dataSource.data){
            widget.dataSource.data.forEach(function(row){
                var stringMes;
                row.TOTAL = 0;
                
                for (var i = 0; i < nrMeses; i++){
                    stringMes = meses[gopUtil.parseFloat(moment(mes, "MM").add(i, 'months').format('MM'))].name;
                    row.TOTAL += Number(row[stringMes]);
                }
                if(this.validaOrcamento(row)){
                    row.__notSelectable = true;
                }
            }.bind(this));
        }
    };
    
    this.validaOrcamento = function(row){
        return (row.IDAPROVACAOGOP === "APROVADO" || row.CONCLUSAO === "CONCLUIDO");
    };
    
    /**
     * Função que retorna um dataSource com os 12 
     * meses do ano para o field 'MESBASE'.
     * @param {widget} widget ativa
     * Chamada ao abrir a widget
     * REPLICA_MES
    **/
    this.carregaMes = function(widget){
        var field = widget.getField("MESBASE");
        field.dataSource.data = gopUtil.getPeriodo("MENSAL");
    };
    
    /**
     * Função retorna um array com as rows alteradas. 
     * @param {widget} widget ativa
     * Chamada nas funções 'saveOrcaMo' e 'saveMeses'
    **/
    this.getRows = function(widget){
        var dataWidget  = widget.dataSource.data;
        var changedRows = dataWidget.filter(row => (Object.keys(row).filter(key => key.indexOf("changed_") !== -1)));
        var editedRows  = changedRows.filter(row => (row.CONCLUSAO != 'CONCLUIDO' && row.IDAPROVACAOGOP != 'APROVADO'));
        
        return editedRows;
    };
    
    /**
     * Função retorna um array das rows modificadas
     * com seus valores tipo float.
     * @param {widget} widget ativa
     * Chamada nas função 'saveMeses'
    **/
    this.parseToFloatRows = function (rows){
        var arrayCampos = this.getArrayCampos();
        
        rows.forEach(function(row){
            arrayCampos.forEach(function(campo){
                if(!Util.isEmptyOrBlank(row[campo+"_inline"])){
                    var campoInline = campo + "_inline";
                    if(typeof row[campoInline] == "string"){
                        for(var i = 0; i < (parseInt(row[campoInline].length / 3) - 1); i++){
                            row[campoInline] = row[campoInline].replace(".", "");
                        }
                        row[campoInline] = row[campoInline].replace(",", ".");    
                    }
                    row[campo] = General.valueToFloat(row[campoInline]);
                }
            });
        });
        return rows;
    };
    
    /**
     * Função que bloqueia o inlineEdit para as rows que
     * obedecem à condição passada.
     * @param {widget} widget ativa
    **/
    this.setInlineEditable = function(element, row, condition){
        if(element.field &&  condition){
            if(element.context.children[1]){
                // element.context.classList.remove('inline-editable');
                element.context.children[1].style.visibility = 'collapse';
            }
        }
    };
    
    this.setValueInlineEditable = function(widget, bool){
        widget.fields.forEach(function(field){
            if(!Util.isEmptyOrBlank(field.inlineEditable)){
                field.inlineEditable = bool;
            }
        });
        if(!Util.isEmptyOrBlank(widget.getParent().getField("VRTOTALMES"))){
            if(widget.getParent().name !=="GOP10000_DETALHAMENTO_ORCAMO_TAB"){
                 widget.getParent().getField("VRTOTALMES").readOnly = !bool;
            }
        }
    };
    
    this.readOnlyRows = function(parentWidget, activeWidget, flag){
        var bool = (parentWidget.currentRow.IDAPROVACAOGOP == 'APROVADO' || parentWidget.currentRow.CONCLUSAO == 'CONCLUIDO');
        
        if(flag == 1){
            this.setValueInlineEditable(activeWidget, !bool);
        }else if(flag == 2){
            activeWidget.fields.forEach(function(field){
                if(!Util.isEmptyOrBlank(field.inlineEditable)){
                    field.inlineEditable = !bool;
                }
            });
        }
    };
    
    this.enterConsumoWidget = function(widget){
        widget.reload();
        widget.unSelectAll(); //Porque as rows não são desmarcadas automaticamente.
        widget.currentRow = '';
    };

    this.setFormTemplate = function(widget){
        widget.formTemplate = widget.template;
        
        widget.list();
        widget.reload();
        
        this.cancelEditaTotal(widget); 
        
        /*if (widget.name ==='GOP10000_DETALHAMENTO_MENSAL'){
            this.cancelEditaTotal(widget); 
        }else{
            this.cancelEditaTotalImposto(widget);
        }*/
        
    };
    
    this.setDescriptionOrder = function(widget, mainWidget){
        widget.getParent().descriptionOrder = mainWidget.currentRow.CDFILIAL + " - " + mainWidget.currentRow.NMFILIAL;
    };
    
    this.backReplicaMes = function(widget){
        widget.lastActiveWidget.activate();
        ScreenService.closePopup();
    };
    
    this.habilitaCampos = function(widget){
        
        let parametros = gopUtil.getWidgetContainer('TAB_CALCULO_OPERACIONAL');
        var data       = widget.fields;
        var bool       = parametros.dataSource.data[0].IDPERORCAITEMDETAL =='S';
        
        
        for(var index in data){

            if(data[index].name === "VRORCAMENTO" || data[index].name ==="VRPERCENTUAL" ){
                data[index].isVisible =bool;
            }
        }
    };
    
    this.cloneCurrentBackUpRow = function(widget){
        

        var widgetTotalizadora = widget;
        var widgetItensDetalhe = widget.widgets[0];
    
    
        widgetTotalizadora.backUpRow = Util.clone(widgetTotalizadora.currentRow);
        
        widgetTotalizadora.backUpRow.VRTOTALMES  = 0;
        widgetTotalizadora.backUpRow.VRORCAMENTO = 0;
       
        for (var index  = 0; index < widgetItensDetalhe.dataSource.data.length; index++){
            /*Acumula o valor de todos os itens contidos no detalhamento*/                
            widgetTotalizadora.backUpRow.VRTOTALMES  += General.valueToFloat(widgetItensDetalhe.dataSource.data[index].VRORCAMENTO);
            widgetTotalizadora.backUpRow.VRORCAMENTO += General.valueToFloat(widgetItensDetalhe.dataSource.data[index].VRORCAMENTO);
        }
        
        /*Gera um BackUp também da Widget Filha, que sera usado no momento de cancelar uma alteração (seja no valor totaol ou do item)*/
       
        widgetItensDetalhe.dataSource.backUp = angular.copy(widgetItensDetalhe.dataSource.data);
    };
    
    this.setTotalResumo = function(args){
        var widget = args.owner;
        var data = widget.dataSource.data;
        var itens = [];
        var totais = [];
        var negativo;
        var soma = 0;
        
        data.filter(a=>a.CDFILIAL).map(a=>a.CDFILIAL).forEach(function(cdfilial){
            data.filter(row=>row.AUXPARENT == cdfilial).forEach(function(row){
                if(row.NOME == "Faturamento" || row.NOME == "Imposto a Recuperar"){
                	itens.push(row.TOTAL);
                }else{
                    if(row.TOTAL != 0)
                        negativo = row.TOTAL * -1;
                    else
                        negativo = 0;
                    itens.push(negativo);
                }
            });
            if(itens != 0){
                 soma = itens.reduce(function(x, y){
                    return Number(x) + Number(y);
                });
            }
            
            var indice = widget.dataSource.data.indexOf(widget.dataSource.data.filter(function(obj){
                return (obj.CDFILIAL == cdfilial);
            })[0]);
            totais.push({"indice": indice, "soma": soma});
            widget.dataSource.data.filter(a=>a.CDFILIAL == cdfilial);
            
            itens = [];
        });
        totais.forEach(function(total){
            widget.dataSource.data[total.indice].TOTAL = total.soma.toFixed(3).replace(".", ",");
        });
    };
    
    this.limpaFiliais = function(widget){
        widget.currentRow.CDFILIAL = [];
    };

    this.createDataSourceFilterFiliais = function(args) {
        var field = args.owner.field;
        var row = field.widget.currentRow;
        var filtroUnidades = [
            {"name": "P_CDGRUPFILI", "operator": "=", "value": Util.isEmptyOrBlank(row.CDGRUPFILI) ? 'T' : row.CDGRUPFILI.join("_")},
            {"name": 'P_CDSEGMER', "operator": "=", "value": Util.isEmptyOrBlank(row.CDSEGMER) ? 'T' : row.CDSEGMER.join('_')},
            {"name": 'P_CDAREAGERE', "operator": "=", "value": Util.isEmptyOrBlank(row.CDAREAGERE) ? 'T' : row.CDAREAGERE.join('_')},
            {"name": 'P_CDOPERADOR', "operator": "=", "value": localStorageEngine.getLocalVar('CDOPERADOR')}
        ];
        field.widget.getField("CDFILIAL").dataSourceFilter = filtroUnidades;
        field.reload();
    };
    
    this.validaCheckUnidade = function(widget){
        if(widget.currentRow.P_TODOS == "S"){
            widget.currentRow.CDFILIAL = "";
            widget.getField("CDFILIAL").readOnly = true;
        }else{
            widget.getField("CDFILIAL").readOnly = false;
        }
    };
    
    this.openDownloadSpreadsheetModel = function() {
		const popup = templateManager.container.getWidget("downloadSpreadsheet");
        ScreenService.openPopup(popup);
    };
    
    this.confirmDownloadSpreadsheet = function() {
        const widgetDownload = templateManager.container.getWidget("downloadSpreadsheet");
        
        if (widgetDownload.isValid()) {
            const row = widgetDownload.currentRow;
            
            let params = {
                "CDFILIAL": row.CDFILIAL,
                "DETALHAMENTO_PRODUTO": row.DETALHAMENTO_PRODUTO
            };
            
            General.openReport("gop#/gop10000_downloadSpreadsheetModel", "GOP10000", params).then(() => {
                widgetDownload.applyDefaultValue();
                ScreenService.closePopup();
            }).catch(() => {
                ScreenService.showMessage("Erro ao gerar planilha.", "error");
            });
        }
    };
    
    this.closeDownloadSpreadsheet = function() {
        const widgetDownload = templateManager.container.getWidget("downloadSpreadsheet");
        
        widgetDownload.applyDefaultValue();
        ScreenService.closePopup();        
    };
    
    this.openUploadSpreadsheetPopUp = function() {
        const popup = templateManager.container.getWidget("importSpreadsheet");
        ScreenService.openPopup(popup);
    };
    
    this.confirmImport = function(widget) {
        if(widget.isValid()) {
            ScreenService.confirmMessage("Os orçamentos existentes serão substituidos. Deseja prosseguir?").then(function() {
                gop10000_CalculoOrcamentoOperacional_Service.importSpreadsheet(widget.currentRow).then(function(callback) {
                    const widgetResumo = templateManager.container.getWidget("GOP10000_RESUMO");
                    const errors = Util.getNestedProperty(callback, ["dataset", "errors"]);
                    
                    if (Array.isArray(errors)) {
                        const message = self.buildErrorMessage(errors);
                        General.showErrorMessages(widgetResumo, message, ScreenService.i18n("Log de Erros:"));
                    } else {
                        widgetResumo.reload().then(function() {
                            ScreenService.closePopup(true);
                            widgetResumo.activate();
                            MessageService.successNotificationByCode("SUC_IMPORT");
                        });
                    }
                }).catch(self.handleSheetErrors);
            });
        }
    };
    
    this.buildErrorMessage = function(errors) {
        let messages = [];
        
        for (let error of errors) {
            messages.push('Unidade ' + error.CDFILIAL + ' - ' + error.DSMOTIVO);
        }
        
        return messages;
    };
    
    this.handleSheetErrors = function (callback) {
        const errorMessages = Util.getNestedProperty(callback, ["data", "dataset", "errors"]);

        if (Array.isArray(errorMessages)) {
            const widgetResumo = templateManager.container.getWidget("GOP10000_RESUMO");
            
            widgetResumo.reload().then(function() {
                let title = "Erros na planilha";
                
                General.showErrorMessages(widgetResumo, errorMessages, ScreenService.i18n(title));
            });
        } else {
            let message = "Tempo limite de conexão excedido.";
            
            if (callback.data && callback.data.error) {
                if (callback.data.error.toString().indexOf("Uninitialized string offset:") != -1) {
                    message = "Erro ao carregar planilha. Certifique-se de que usou o modelo de planilha para importação.";
                } else if (callback.data.error.toString().indexOf("failed to open stream: Permission denied") != -1) {
                    message = "Não foi possível carregar a planilha no servidor: Permissão negada. Por favor, contate um administrador do sistema.";
                }
            }
            
            ScreenService.showMessage(message, "error");
        } 
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('gop10000_CalculoOrcamentoOperacionalController', gop10000_CalculoOrcamentoOperacionalController);
});

// FILE: ./assets/js/controllers/gop11000_AprovacaoOrcaOperacController.js
function gop11000_AprovacaoOrcaOperacController(ScreenService, General, restEngine, templateManager, gopUtil, ZhFilterPreferences, gop11000_AprovacaoOrcaOperac_Service, MessageService){
 
    this.setFiltro = function(widget){

        widget.getField('CDFILIAL').defaultValue = (getLocalVar('CDFILIAL')); 
        widget.getField('NMFILIAL').defaultValue = (getLocalVar('CDFILIAL')+' | '+getLocalVar('NMFILIAL')); 
        gopUtil.startFilter(widget,widget.currentRow);
    };

    this.save = function(widget){
        General.saveCrud(widget).then(function(){
            widget.reload();
            widget.view();
            ScreenService.showMessage("Operação realizada com sucesso");
        }, function(erro){
            ScreenService.showMessage(erro);
        });
    };
    
   this.openPopupObservacao = function(widget, action){
        var descricaoWidget = templateManager.container.getWidget('widgetDescricao'); 
        
        if(action == "APROVA"){
            descricaoWidget.label = "Aprovação";
            descricaoWidget.actions[0].label = "Aprovar";
        }else if(action == "CANCELA"){
            descricaoWidget.label = "Cancelar Aprovação";
            descricaoWidget.actions[0].label = "Cancelar Aprovação";
        }else if(action == "REPROVA"){
            descricaoWidget.label = "Reprovação";
            descricaoWidget.actions[0].label = "Reprovar";
        }
        
        descricaoWidget.currentRow.DSOBSERVACAO = "";
        ScreenService.openPopup(descricaoWidget);
    };
    
    this.callAction = function(descricaoWidget){
        if(descricaoWidget.label == "Aprovação"){
            this.aprova(descricaoWidget);
        }else if(descricaoWidget.label == "Cancelar Aprovação" || descricaoWidget.label == "Reprovação"){
            this.reprova(descricaoWidget);
        }
    };
    
    this.aprova = function(descricaoWidget){
        var mainWidget = gopUtil.getWidgetContainer('widgetMainAprovacao');
        
        if(descricaoWidget.isValid()){    
            if(mainWidget.currentRow.NRAPROVAORCAGOP){
                this.update(descricaoWidget, "A");
            }else{
                if(descricaoWidget.isValid()){
                    mainWidget.currentRow.DSOBSERVACAO = descricaoWidget.currentRow.DSOBSERVACAO;
                    mainWidget.currentRow.IDAPROVACAOGOP = "APROVADO";
                    gop11000_AprovacaoOrcaOperac_Service.saveAprova([mainWidget.currentRow]).then(function(response){
                        mainWidget.reload();
                        mainWidget.list();
                        ScreenService.closePopup();
                        MessageService.notificationMessageByCode("SUC_APPROVE", null, Constants.successNotification);
                    });
                }
                this.invisibleAction(descricaoWidget);
            }
        }
    };
    
    this.reprova = function(descricaoWidget){
        
        var mainWidget = gopUtil.getWidgetContainer('widgetMainAprovacao');
        
        if(descricaoWidget.isValid()){
            mainWidget.currentRow.DSOBSERVACAO = descricaoWidget.currentRow.DSOBSERVACAO;
            mainWidget.currentRow.IDSTATUS = "PENDENTE";

            gop11000_AprovacaoOrcaOperac_Service.updateConclusao([mainWidget.currentRow]).then(function(response){
                mainWidget.reload();
                mainWidget.list();
                ScreenService.closePopup();
                MessageService.notificationMessageByCode("SUC_DISAPPROVE", null, Constants.successNotification);
            });
            
            if (mainWidget.currentRow.IDAPROVACAOGOP == "APROVADO") {
                this.update(descricaoWidget, "C");
            }else{
                this.update(descricaoWidget, "R");
            }
        }
         this.invisibleAction(descricaoWidget);
    };
    
    this.update = function(descricaoWidget, flag){
        var mainWidget = gopUtil.getWidgetContainer('widgetMainAprovacao');
        
        if(descricaoWidget.isValid()){
            if(flag == "C"){
                if(mainWidget.currentRow.IDAPROVACAOGOP == "PENDENTE"){
                  mainWidget.currentRow.IDAPROVACAOGOP = "APROVADO";
                }else if(mainWidget.currentRow.IDAPROVACAOGOP == "APROVADO"){
                    mainWidget.currentRow.IDAPROVACAOGOP = "PENDENTE";
                    mainWidget.currentRow.DSOBSERVACAO = descricaoWidget.currentRow.DSOBSERVACAO;
                }
            }else if(flag == "A"){
                mainWidget.currentRow.IDAPROVACAOGOP = "APROVADO";
                mainWidget.currentRow.DSOBSERVACAO = descricaoWidget.currentRow.DSOBSERVACAO;
            }else if(flag == "R"){
                mainWidget.currentRow.IDAPROVACAOGOP = "REPROVADO";
                mainWidget.currentRow.DSOBSERVACAO = descricaoWidget.currentRow.DSOBSERVACAO;
            }
            gop11000_AprovacaoOrcaOperac_Service.updateAprova([mainWidget.currentRow]).then(function(response){
                mainWidget.reload();
                mainWidget.list();
                ScreenService.closePopup();
            });
            
            this.invisibleAction(descricaoWidget);
        }
    };
    
    this.openWidget = function(){
        var mainWidget = gopUtil.getWidgetContainer('widgetMainAprovacao');
        mainWidget.activate();
    };
    
    this.invisibleAction = function(widget){
        var mainWidget = gopUtil.getWidgetContainer('widgetMainAprovacao');
        
        if(!Util.isEmptyOrBlank(widget.currentRow)){
            mainWidget.getAction('APROVA').isVisible = false;
            mainWidget.getAction('CANCELAR_APROVACAO').isVisible = false;
            mainWidget.getAction('REPROVA').isVisible = false;
            mainWidget.reload();
            widget.currentRow.DSOBSERVACAO = "";
        }
    };
    
    this.action = function(widget){
        widget.activate();
        if(widget.currentRow.IDAPROVACAOGOP != "APROVADO"){
            widget.getAction('APROVA').isVisible = true;
            widget.getAction('REPROVA').isVisible = true;
            widget.getAction('CANCELAR_APROVACAO').isVisible = false;
        }else{
            widget.getAction('CANCELAR_APROVACAO').isVisible = true;
            widget.getAction('APROVA').isVisible = false;
            widget.getAction('REPROVA').isVisible = false;
        }
    };
    
    this.setStripe = function(widget){
		let stripeWid = gopUtil.getWidgetContainer("widgetStripe");
		let filterRow = widget.appliedFilter.FILTER_ROW;
		
		stripeWid.currentRow.NRANOORCAGOP = !Util.isEmptyOrBlank(filterRow.NRANOORCAGOP) ? filterRow.NRANOORCAGOP : "TODOS";
		
		stripeWid.currentRow.UNIDADE = !Util.isEmptyOrBlank(filterRow.CDFILIAL) ? filterRow.NMFILIAL : "TODAS";
		
		stripeWid.currentRow.STATUS = !Util.isEmptyOrBlank(filterRow.IDAPROVACAOGOP) ? filterRow.IDAPROVACAOGOP : "TODOS";
		
		ScreenService.closePopup(true);
	};  
	
	this.setGrid = function(widget){
        widget.dataSource.data.forEach(function(row){
            var periodo = gopUtil.getPeriodo(row.IDPERIODOORCA);
            periodo.forEach(function(p){
               if(p.value == row.NRPERIODOORCA){
                   row.NRPERIODOORCA = p.name;
               }
           });
        });
    };

    
}
Configuration(function(ContextRegister) {
    ContextRegister.register('gop11000_AprovacaoOrcaOperacController', gop11000_AprovacaoOrcaOperacController);
});

// FILE: ./assets/modules/cadastro/assets/js/controllers/fai37000_faturamento_diversosController.js

function fai37000_faturamento_diversosController(ScreenService, General, templateManager, gopUtil) {
    
    this.validateNumber = function(widget){
        var CDFATUDIVER = widget.currentRow.CDFATUDIVER;
        if(isNaN(CDFATUDIVER)){
            if(CDFATUDIVER.latinize() != CDFATUDIVER){
                ScreenService.showMessage("Não são permitidos caracteres especiais no campo Código");
                return false;
            }else{
                return true;
            }
        }else{
            if(CDFATUDIVER.indexOf('.') != -1 || CDFATUDIVER.indexOf(',') != -1  ){
                ScreenService.showMessage("Caso o código seja um número, apenas inteiros.");
                return false;
            }else{
            
                if(CDFATUDIVER.length == 1){
                    widget.currentRow.CDFATUDIVER = "0"+CDFATUDIVER;
                }
                return true;
            }
        }
        
    };
    
    this.checkNaN = function(widget){
        var CDFATUDIVER = widget.currentRow.CDFATUDIVER;
        if(widget.isValid()){
           if(isNaN(CDFATUDIVER)){
               return true;
           }else{
               return true;
           }
        }
        
    };
    
    this.save = function(widget){
        if(this.checkNaN(widget) && this.validateNumber(widget)){
            General.saveCrud(widget).then(function(){
                    widget.reload();
            }, function(erro){
                if(erro.indexOf("ORA-0001") != -1){
                   ScreenService.showMessage(erro); 
                }else{
                    ScreenService.showMessage("Impossível adicionar. Registro já existe.");
                }
                    
            });  
        }
    };
    
    this.exclui = function(widget){
        
        var msg = ScreenService.i18n("Exclusão não permitida. A informação está sendo utilizada na tela de Laçamento de Faturamentos Diversos por Unidade.");
        
        if(widget.mode == "list"){
            gopUtil.deleteCrudCheckedRows(widget, msg);
        }else {
            gopUtil.deleteCrud(widget, msg);
        }
        
    };
    

}

Configuration(function(ContextRegister) {
    ContextRegister.register('fai37000_faturamento_diversosController', fai37000_faturamento_diversosController);
});

// FILE: ./assets/modules/cadastro/assets/js/controllers/fai63000_receitas_deducoesController.js
function fai63000_receitas_deducoesController(ScreenService, General, templateManager, gopUtil) {
    
    this.validateNumber = function(widget){
        var CDRECEXTRA = widget.currentRow.CDRECEXTRA;
        if(isNaN(CDRECEXTRA)){
            if(CDRECEXTRA.latinize() != CDRECEXTRA){
                ScreenService.showMessage("Não são permitidos caracteres especiais no campo Código");
                return false;
            }else{
                return true;
            }
        }else{
            if(CDRECEXTRA.indexOf('.') != -1 || CDRECEXTRA.indexOf(',') != -1  ){
                ScreenService.showMessage("Caso o código seja um número, apenas inteiros.");
                return false;
            }else{
            
                if(CDRECEXTRA.length == 1){
                    widget.currentRow.CDRECEXTRA = "000"+CDRECEXTRA;
                }else if(CDRECEXTRA.length == 2){
                    widget.currentRow.CDRECEXTRA = "00"+CDRECEXTRA;
                }else if(CDRECEXTRA.length == 3){
                    widget.currentRow.CDRECEXTRA = "0"+CDRECEXTRA;
                }
                return true;
            }
        }
        
    };
    
    this.checkNaN = function(widget){
        var CDRECEXTRA = widget.currentRow.CDRECEXTRA;
        if(widget.isValid()){
           if(isNaN(CDRECEXTRA)){
               if(!isNaN(CDRECEXTRA)){
                   this.validateNumber(widget);
               }else{
                   return true;
               }
           }else{
               return true;
           }
        }
        
    };
    
    this.save = function(widget){
        if(this.checkNaN(widget) && this.validateNumber(widget)){
            
            if(widget.currentRow.__is_new == false){
                if(Util.isEmptyOrBlank(widget.currentRow.NRRECEXTRA)){
                    widget.dataSource.data.forEach(function(row) {
                        if(row.CDRECEXTRA == widget.currentRow.CDRECEXTRA && row.DESCRECEXTRA == widget.backupRow.DESCRECEXTRA) {
                            widget.currentRow.NRRECEXTRA =  row.NRRECEXTRA;
                        }
                    });
                    
                }
                General.saveCrud(widget).then(function(){
                    widget.view();
                    if(Util.isEmptyOrBlank(widget.backupRow.NRRECEXTRA)){
                        widget.dataSource.data.pop();
                    }
                    widget.currentRow = widget.dataSource.data.filter(el=>el.CDRECEXTRA==widget.currentRow.CDRECEXTRA)[0];
                    widget.reload();
                    widget.updateSwipeTip(widget.currentRow);
                    ScreenService.successNotification("Operação realizada com sucesso");
                }, function(erro){
                   ScreenService.showMessage(erro);
                });
               
            }else {
                    gopUtil.saveCrudBackToView(widget).then(function(response){
                    widget.currentRow.__is_new = false;
                    widget.descriptionOrder = widget.currentRow.CDRECEXTRA + ' - ' + widget.currentRow.DESCRECEXTRA;
                    widget.currentRow = widget.dataSource.data.filter(el=>el.CDRECEXTRA==widget.currentRow.CDRECEXTRA)[0];
                    widget.reload();
                    widget.updateSwipeTip(widget.currentRow);
                    ScreenService.successNotification("Operação realizada com sucesso");
                   
                }, function(erro){
                    if(erro.indexOf("ORA-0001") != -1){
                      ScreenService.showMessage(erro); 
                    }else{
                        ScreenService.showMessage("Impossível adicionar. Registro já existe.");
                    }   
                });  
            }
        }
        
    };
    
    this.exclui = function(widget){
        if (widget.getCheckedRows().length == 0) {
            gopUtil.deleteCrud(widget).then(function(){
                ScreenService.closeSwipe();
            });
        }else {
            gopUtil.deleteCrudCheckedRows(widget);
        }
    };
    
    this.recarrega = function(field){
        field.dataSource.reload();
    };  
}

Configuration(function(ContextRegister) {
    ContextRegister.register('fai63000_receitas_deducoesController', fai63000_receitas_deducoesController);
});

// FILE: ./assets/modules/cadastro/assets/js/controllers/ger80700_orcamentoMOController.js
function ger80700_orcamentoMOController(General, ScreenService, gopUtil, ger80700_orcamentoMo_Service, templateManager, ZHPromise,  MessageService) {
    
    this.edit = function(widget) {
        General.edit(widget);
        if(widget.name == "ger80700_mainWidget")
            this.enabledDespesa(false);
    };
    
    this.exclui = function(widget) {
        General.deleteCrud(widget).then(function() {}, function(erro) {
            ScreenService.showMessage('Não é possível excluir o orçamento de mão de obra, pois possui orçamento gerado.');
        }.bind(this), function(erro) {
            ScreenService.showMessage(erro);
        });
    };
    
    this.save = function(widget) {
        if(widget.isValid()){
            if(!isNaN(widget.currentRow.CDORCAMENTOMO)){
                if(widget.currentRow.CDORCAMENTOMO < 0){
                    widget.currentRow.CDORCAMENTOMO = (widget.currentRow.CDORCAMENTOMO * -1);
                }
                widget.currentRow.CDORCAMENTOMO = ("0000000000" + widget.currentRow.CDORCAMENTOMO).slice(-10);
                
            }
            if(!widget.currentRow.CDFILIAL)
                widget.currentRow.CDFILIAL = 'T';
                
            gopUtil.saveCrudBackToView(widget).then(function(){
			    MessageService.notificationMessageByCode("SUC_SAVE", null, Constants.successNotification);
            }.bind(this),function(error){
                ScreenService.showMessage(error);
            });
            this.enabledDespesa(true);
        }
    };
    
    this.setFilter = function(widget){
        var filial = '';
        if(Util.isEmptyOrBlank(widget.appliedFilter.FILTER_ROW.CDFILIAL)){
            filial = "T";
        }else{
            filial = widget.appliedFilter.FILTER_ROW.CDFILIAL.join('_');
        }
        
        widget.dataSourceFilter = [
                {
                    "name"     : "P_GERAL",
                    "operator" : "=",
                    "value"    : widget.appliedFilter.FILTER_ROW.P_GERAL
                },
                {
                    "name"     : "P_CDFILIAL",
                    "operator" : "=",
                    "value"    : filial
                },
                {
                    "name"     : "TIPO",
                    "operator" : "=",
                    "value"    : widget.appliedFilter.FILTER_ROW.TIPO ? widget.appliedFilter.FILTER_ROW.TIPO : "T"
                }
            ];
        widget.reload();
    };
    
    this.validaCkUnidade = function(widget){
        if(widget.currentRow.P_TODOS == "S"){
            widget.currentRow.CDFILIAL = "";
            widget.currentRow.NMFILIAL = "";
            widget.getField("NMFILIAL").readOnly = true;
            widget.getField("NMFILIAL").validations = {};
        }else{
            widget.getField("NMFILIAL").readOnly = false;
            widget.getField("NMFILIAL").validations = { "required" : {}};
        }
    };

    this.setValuesDespesa = function(widget){
       var mainWidget = widget.getParent();
       
       widget.dataSourceFilter = [
            {
                "name": "P_CDFILIAL",
                "operator": "=",
                "value": mainWidget.currentRow.CDFILIAL
            },
            {
                "name": "P_CDORCAMENTOMO",
                "operator": "=",
                "value": mainWidget.currentRow.CDORCAMENTOMO
            }
        ];
        
        widget.currentRow.CDORCAMENTOMO = mainWidget.currentRow.CDORCAMENTOMO;
        widget.currentRow.CDFILIAL = mainWidget.currentRow.CDFILIAL;
        
        if(mainWidget.currentRow.IDTPORCAMENTOMO == 'M'){
            widget.getField('VRMENSDESPORCMO').isVisible = true;
        }else{
            widget.getField('VRMENSDESPORCMO').isVisible = false;
        }
        
        if(mainWidget.currentRow.IDTPORCAMENTOMO == 'D'){
            widget.getField('VRUNITDESPORCMO').isVisible = true;
            widget.getField('QTDIADESPFUNORC').isVisible = true;
            widget.getField('VRDIARIODESP').isVisible = true;
            widget.getField('QTHORAMESORC').isVisible = true;
            widget.getField('IDORCBASEDIAUTIL').isVisible = true;
        }else{
            widget.getField('VRUNITDESPORCMO').isVisible = false;
            widget.getField('QTDIADESPFUNORC').isVisible = false;
            widget.getField('VRDIARIODESP').isVisible = false;
            widget.getField('QTHORAMESORC').isVisible = false;
            widget.getField('IDORCBASEDIAUTIL').isVisible = false;
        }
        
        if(mainWidget.currentRow.IDTPORCAMENTOMO == 'P'){
            widget.getField('VRPERCDESPORCMO').isVisible = true;
        }else{
            widget.getField('VRPERCDESPORCMO').isVisible = false;
        }
        widget.reload();
   };
   
    this.saveDespesa = function(widget){
        var mainWidget = widget.getParent();
        
        if(widget.isValid()){
            widget.currentRow.VRMENSDESPORCMO = General.valueToFloat(widget.currentRow.VRMENSDESPORCMO);
            widget.currentRow.VRUNITDESPORCMO = General.valueToFloat(widget.currentRow.VRUNITDESPORCMO);
            widget.currentRow.VRDIARIODESP = General.valueToFloat(widget.currentRow.VRDIARIODESP);
            widget.currentRow.VRPERCDESPORCMO = General.valueToFloat(widget.currentRow.VRPERCDESPORCMO);
            
            if(mainWidget.currentRow.IDTPORCAMENTOMO !== 'D')
                widget.currentRow.IDORCBASEDIAUTIL = 'N';
            if(widget.currentRow.VRPERCDESPORCMO.isVisible == false)
                widget.currenRow.VRPERCDESPORCMO = null;
            if(!widget.currentRow.CDFILIAL)
                widget.currentRow.CDFILIAL = 'T';
             
            if(!widget.currentRow.__is_new){
                if(widget.currentRow.CDDESPADMFAT != widget.backupRow.CDDESPADMFAT){ 
                    widget.currentRow.__is_new = true;
                    gopUtil.saveCrudBackToView(widget).then(function(){
                        ger80700_orcamentoMo_Service.excluiDespesa([widget.backupRow]).then(function(response){
                            widget.view();
                            widget.reload();
                            MessageService.notificationMessageByCode("SUC_SAVE", null, Constants.successNotification);
                        });
                        
                    }, function(){
                        widget.currentRow.__is_new = false;
                    });
                }else{
                    gopUtil.saveCrudBackToView(widget).then(function(response){
                        MessageService.notificationMessageByCode("SUC_SAVE", null, Constants.successNotification);
                    });
                }
            }else{
                gopUtil.saveCrudBackToView(widget).then(function(response){
                        MessageService.notificationMessageByCode("SUC_SAVE", null, Constants.successNotification);
                });
            }
       }
   };
   
    this.adicionaDespesa = function(widget){
        var mainWidget = widget.getParent();
        
        widget.newRow();
        widget.currentRow.CDORCAMENTOMO = mainWidget.currentRow.CDORCAMENTOMO;
        widget.currentRow.CDFILIAL = mainWidget.currentRow.CDFILIAL;
   };
    
    this.enabledDespesa = function(value){
        var despesaWidget = templateManager.container.getWidget("ger80700_despesa");
        despesaWidget.enabled = value;
    };
    
    this.setValorDiario = function (widget){
        var row = widget.currentRow;
        if(row.QTDIADESPFUNORC && row.VRUNITDESPORCMO)
            row.VRDIARIODESP = row.QTDIADESPFUNORC * General.valueToFloat(row.VRUNITDESPORCMO);
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ger80700_orcamentoMOController', ger80700_orcamentoMOController);
});


// FILE: ./assets/modules/cadastro/assets/js/controllers/gop20000_conta_contabil_por_despesaController.js
function gop20000_conta_contabil_por_despesaController(ScreenService, General, templateManager,gopUtil,gop20000_conta_contabil_por_despesaService, ZhKeyboardService, eventEngine, MessageService) {

    
    this.buscaFiltroPrincipal = function (widget) {
        
        var mainWidget         = gopUtil.getWidgetContainer('gop20000_conta_contabil_despesa');
        var filtroTela         = gopUtil.getWidgetContainer("filterSelectContaContabil"); 
        var row                = filtroTela.currentRow;    
        var CDFILIAL           = "";
        var TCDFILIAL          = "";
        var PADRAO             = "";
        var CDDESPADMFAT       = "";
        var CDCONTCTBL         = "";
        
        if (filtroTela.isValid()){
            
            if(row.CDFILIAL != null && row.CDFILIAL !="" && row.TCDFILIAL !='T'){
            
                row.CDFILIAL.forEach(function(unidade){
                    CDFILIAL += "_"+unidade;
                });
                
                CDFILIAL +='_';

            }else{
                CDFILIAL = "";
            }
    
            if(row.TCDFILIAL != null && row.TCDFILIAL != ""){
                TCDFILIAL = row.TCDFILIAL;
            }else{
                TCDFILIAL = null;
            }
            
            
            if(row.PADRAO != null && row.PADRAO != ""){
                PADRAO = row.PADRAO;
            }else{
                PADRAO = null;
            }
        
            if(row.DSDESPADMFAT != null && row.DSDESPADMFAT !=""){
                
                row.DSDESPADMFAT.forEach(function(MO){
                    CDDESPADMFAT += "_"+MO;
                });
                
                CDDESPADMFAT +="_";
    
            }else{
                CDDESPADMFAT = "T";
            }
            
            if(row.NMCONTCTBL != null && row.NMCONTCTBL !=""){
                
                row.NMCONTCTBL.forEach(function(CTB){
                    CDCONTCTBL += "_"+CTB;
                });
    
            }else{
                CDCONTCTBL = "T";
            }
        
            var filtro = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": CDFILIAL
                },
                {
                    "name":"TCDFILIAL",
                    "operator":"=",
                    "value":TCDFILIAL
                },
                {
                    "name":"PADRAO",
                    "operator":"=",
                    "value":PADRAO
                },
                {
                    "name":"CDDESPADMFAT",
                    "operator":"=",
                    "value":CDDESPADMFAT 
                },
                {
                    "name":"CDCONTCTBL",
                    "operator":"=",
                    "value":CDCONTCTBL
                }
                
            ];
        
            mainWidget.dataSourceFilter = filtro;
            ScreenService.closePopup(true);
            mainWidget.reload();
            
        }
    
    };
    
    this.openPopupFiltro = function (widget) {
        var filtroContainer = gopUtil.getWidgetContainer("filterSelectContaContabil");  
        filtroContainer.reload();
        ScreenService.openPopup(filtroContainer);
    };
    
    this.edit = function (widget) {
        
        widget.backupRow = angular.copy(widget.currentRow);
        widget.edit();
        
    };
    
    this.afterInitContainer = function() {
        let gop20000_conta_contabil_despesa = gopUtil.getWidgetContainer('gop20000_conta_contabil_despesa');
        var ref = this;
        gop20000_conta_contabil_despesa.openFilterShortcut = eventEngine.bindEvent(ZhKeyboardService, 'mod+lPressed', function(eventData) {
            return ref.openPopupFiltro(eventData);
        });
    };
    
    this.beforeCloseContainer = function(args) {
        let gop20000_conta_contabil_despesa = gopUtil.getWidgetContainer('gop20000_conta_contabil_despesa');
        if (gop20000_conta_contabil_despesa.openFilterShortcut >= 0)
            eventEngine.unbindEvent(ZhKeyboardService, 'mod+lPressed', gop20000_conta_contabil_despesa.openFilterShortcut);
    };
    
    this.desabilitaUnidade = function (widget) {
        var padrao = widget.getField('PADRAO').checked;
        var todas = widget.getField('TCDFILIAL').checked;
        var requiredField = {"required":{"message":"Campo obrigatório"}};
        
        widget.getField('NMFILIAL').readOnly = todas;
        
        if(padrao || todas){
            if(padrao){
                widget.getField('NMFILIAL').validations = {};
            }
            if(todas){
                widget.getField('NMFILIAL').validations = {};
                widget.currentRow.CDFILIAL = "";
                widget.currentRow.NMFILIAL = "";
            } 
        }
        else{
            widget.getField('NMFILIAL').validations = requiredField;
        }
    };
    
    this.save = function (mainWidget){
        
        if(mainWidget.isValid()){
        
            if(mainWidget.currentRow.__is_new){
                this.saveCrudCtbMo(mainWidget);
            }else{
                gop20000_conta_contabil_por_despesaService.atualiza([mainWidget.currentRow,mainWidget.backupRow]).then(function(){
                    mainWidget.reload();
                    mainWidget.view();
                    ScreenService.successNotification("Registro salvo com sucesso.");
                },function(response_error){
                    if(response_error.indexOf("MSG_ERRO") !== -1){
                        MessageService.showMessageByCode("MSG_ERRO", {"PARAMETRO": response_error.replace("MSG_ERRO", "")});
                    } else {
                    ScreenService.showMessage(response_error.data.error); 
                    }
                });
            }
        }
	};
	this.saveCrudCtbMo = function (mainWidget){
	    
        if((Util.isEmptyOrBlank(mainWidget.getField('CDFILIAL').value()) ||(mainWidget.getField('CDFILIAL').value() ==="T"))) {
            
            mainWidget.currentRow.CDFILIAL = 'T';
            mainWidget.currentRow.NMFILIAL = 'Geral | Aplicado a todas as unidades';
            
        }
        gopUtil.saveCrudBackToView(mainWidget).then(function(){
            ScreenService.successNotification("Registro salvo com sucesso.");
        });
	};
	
	this.clearFiltro = function(widget){
        widget.currentRow = {
            "PADRAO": "T",
            "TCDFILIAL": ""
        };
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('gop20000_conta_contabil_por_despesaController', gop20000_conta_contabil_por_despesaController);
});

// FILE: ./assets/modules/relatorio/assets/js/controllers/gop13000_verificacao_de_pendencias_Controller.js
function gop13000_verificacao_de_pendencias_Controller(ScreenService, General, templateManager, gopUtil, restEngine, ZhFilterPreferences, gop13000_verificacao_de_pendenciasService){
    
    /*Funções de Filtro e  relativas à StripeWidget*/
    
    this.openPopupFiltro = function(){
        var paramWidget = gopUtil.getWidgetContainer('widgetVerificacaoParam');
        var operacWidget = gopUtil.getWidgetContainer('widgetVerificacaoOperac');
        var filterWidget = gopUtil.getWidgetContainer('filterSelect');
        
        if(paramWidget.isActive()){
            filterWidget.getField('NRANOORCAGOP').isVisible = false;
            filterWidget.getField('IDPERIODOORCA').isVisible = false;
            filterWidget.getField('NRPERIODOORCA').isVisible = false;
            
        }else if(operacWidget.isActive()){
            filterWidget.getField('NRANOORCAGOP').isVisible = true;
            filterWidget.getField('IDPERIODOORCA').isVisible = true;
            filterWidget.getField('NRPERIODOORCA').isVisible = true;
        }
        
        ScreenService.openPopup(filterWidget);
    };
    
    this.voltarFiltro = function(){
        var mainWidget = gopUtil.getWidgetContainer("widgetVerificacaoParam");
        ScreenService.closePopup();
        mainWidget.reload();
    };
    
    this.backAction = function(controleWidget){
        controleWidget.widgets.forEach(function(widget){
            if(!Util.isEmptyOrBlank(widget.getAction('BACK_PENDENCIAS'))){
                 widget.getAction('BACK_PENDENCIAS').isVisible = false; 
            }
        });
    };

    this.changeStripe = function(bool){
        var stripeWidget = gopUtil.getWidgetContainer('widgetStripe');
        var filterWidget = gopUtil.getWidgetContainer('filterSelect');
        
        if(filterWidget.currentRow.IDPERIODOORCA != 'ANUAL'){
            stripeWidget.getField('PERIODOORCA').isVisible = bool;
        }else{
            stripeWidget.getField('PERIODOORCA').isVisible = false;
        }
        stripeWidget.getField('UNIDADEPARAM').isVisible = !bool;
        stripeWidget.getField('ANO').isVisible = bool;
        stripeWidget.getField('UNIDADEOPERAC').isVisible = bool;
    };
    
    this.setStripe = function(filterWidget, field){
        var stripeWidget = gopUtil.getWidgetContainer('widgetStripe');
        var unidades = "";
        
        if(Util.isEmptyOrBlank(filterWidget.currentRow.CDNMFILIAL)){
            stripeWidget.currentRow[field] = "Todas";
        }else{
            if(filterWidget.currentRow.CDNMFILIAL.length > 1){
                stripeWidget.getField(field).label = "Unidades: ";
                filterWidget.currentRow.CDNMFILIAL.forEach(function(filial){
                    if(Util.isEmptyOrBlank(unidades)){
                        unidades = filial.split(" - ")[1];
                    }else{
                        unidades = ", " + filial.split(" - ")[1];
                    }
                });
                stripeWidget.currentRow[field] =  unidades;
            }else{
                stripeWidget.getField(field).label = "Unidade: ";
                stripeWidget.currentRow[field] = filterWidget.currentRow.CDNMFILIAL;
            }
        }
    };
    
    this.setStripeOperac = function(filterWidget, bool){
        var stripeWidget = gopUtil.getWidgetContainer('widgetStripe');

        this.setStripe(filterWidget, 'UNIDADEOPERAC');
        if(filterWidget.currentRow.IDPERIODOORCA != 'ANUAL'){
            stripeWidget.currentRow.PERIODOORCA = filterWidget.getField('NRPERIODOORCA').dataSource.data[filterWidget.currentRow.NRPERIODOORCA - 1].name;
            stripeWidget.getField('PERIODOORCA').isVisible = true;
        }
        else{
            stripeWidget.getField('PERIODOORCA').isVisible = false;
        }
        stripeWidget.currentRow.ANO = filterWidget.currentRow.NRANOORCAGOP;
        stripeWidget.getField('ANO').isVisible = true;
    };

    this.setFiltro = function(){
        var filterWidget = gopUtil.getWidgetContainer("filterSelect");
        
        if(filterWidget.isValid()){
            if(filterWidget.getField('NRANOORCAGOP').isVisible){
                this.setFiltroOperac(filterWidget);
            }else{
                this.setFiltroParam(filterWidget);
            }
        }
    };

    this.setFiltroParam = function(filterWidget){
        var paramWidget = gopUtil.getWidgetContainer('widgetVerificacaoParam');
        var cdfilial = "";
        
        if (filterWidget.isValid()){
            if(!Util.isEmptyOrBlank(filterWidget.currentRow.CDNMFILIAL)){
                filterWidget.currentRow.CDNMFILIAL.forEach(function(filial){
                    cdfilial += filial.split(" - ")[0] + "_";
                });
            }
            
            this.setStripe(filterWidget, 'UNIDADEPARAM');
            
            let dataSourceFilter = [
                { "name": "P_CDFILIAL", "operator": "=", "value": cdfilial },
                { "name": "P_CDOPERADOR", "operator": "=", "value": getLocalVar('CDOPERADOR') },
                { "operator": "ORDER_BY", "name": [{ "name": "CDFILIAL", "direction": "ASC" }]}
            ];
            
            gop13000_verificacao_de_pendenciasService.dataParametro(dataSourceFilter).then(function(response){
                paramWidget.dataSource.data = response.dataset.gop13000_verificacao_parametro;
                paramWidget.reload();
                paramWidget.activate();
                ScreenService.closePopup();
            });
        }
    };

    this.setFiltroOperac = function(filterWidget) {
        var operacWidget = gopUtil.getWidgetContainer('widgetVerificacaoOperac');
        var ano = filterWidget.currentRow.NRANOORCAGOP;
        var idperiodo = filterWidget.currentRow.IDPERIODOORCA;
        var nrperiodo = filterWidget.currentRow.NRPERIODOORCA;
        var cdfilial = "";
        
        if (filterWidget.isValid()){
            if(!Util.isEmptyOrBlank(filterWidget.currentRow.CDNMFILIAL)){
                filterWidget.currentRow.CDNMFILIAL.forEach(function(filial){
                    cdfilial += filial.split(" - ")[0] + "_";
                });
            }
            
            this.setStripeOperac(filterWidget, true);
            
            let dataSourceFilter = [
                { name: "P_NRPERIODOORCA", operator: "=", value: nrperiodo },
                { name: "P_CDFILIAL",      operator: "=", value: cdfilial },
                { name: "P_IDPERIODOORCA", operator: "=", value: idperiodo },
                { name: "P_CDOPERADOR",    operator: "=", value: getLocalVar("CDOPERADOR") },
                { name: "P_NRANOORCAGOP",  operator: "=", value: ano } 
            ];
            
            gop13000_verificacao_de_pendenciasService.dataOperacional(dataSourceFilter).then(function(response){
                operacWidget.dataSource.data = response.dataset.gop13000_verificacao_operacional;
                operacWidget.reload();
                operacWidget.getParent().openTab('widgetVerificacaoOperac');
                ScreenService.closePopup();
            });
        }
    };

    this.setStripe = function(filterWidget, field){
        var stripeWidget = gopUtil.getWidgetContainer('widgetStripe');
        var unidades = "";
        
        if(Util.isEmptyOrBlank(filterWidget.currentRow.CDNMFILIAL)){
            stripeWidget.currentRow[field] = "Todas";
        }else{
            if(filterWidget.currentRow.CDNMFILIAL.length > 1){
                stripeWidget.getField(field).label = "Unidades: ";
                filterWidget.currentRow.CDNMFILIAL.forEach(function(filial){
                    if(Util.isEmptyOrBlank(unidades)){
                        unidades = filial.split(" - ")[1];
                    }else{
                        unidades += ", " + filial.split(" - ")[1];
                    }
                });
                stripeWidget.currentRow[field] =  unidades;
            }else{
                stripeWidget.getField(field).label = "Unidade: ";
                stripeWidget.currentRow[field] = filterWidget.currentRow.CDNMFILIAL;
            }
        }
    };

    this.setStripeOperac = function(filterWidget, bool){
        var stripeWidget = gopUtil.getWidgetContainer('widgetStripe');

        this.setStripe(filterWidget, 'UNIDADEOPERAC');
        stripeWidget.currentRow.PERIODOORCA = filterWidget.getField('NRPERIODOORCA').dataSource.data[filterWidget.currentRow.NRPERIODOORCA - 1].name;
        stripeWidget.currentRow.ANO = filterWidget.currentRow.NRANOORCAGOP;
        stripeWidget.getField('PERIODOORCA').isVisible = true;
        stripeWidget.getField('ANO').isVisible = true;
        
    };
    
    /**
     * Função utilizada para a adequação da widget stripe
     * ao filtro da widget ativa
     * @param {object(widget)} widget widget ativa
     * Chamada ao mudar de página na tela.
     * (Parametrização e Operacionais)
    **/ 
    
    this.changeStripe = function(bool){
        var stripeWidget = gopUtil.getWidgetContainer('widgetStripe');
        
        stripeWidget.getField('UNIDADEPARAM').isVisible = !bool;
        stripeWidget.getField('PERIODOORCA').isVisible = bool;
        stripeWidget.getField('ANO').isVisible = bool;
        stripeWidget.getField('UNIDADEOPERAC').isVisible = bool;
    };
    
    /*-----------------------------------------------------------------------*/

    /*Funções relativas às telas de tratamento das pendências*/

    this.openWindowParametro = function(widget){
        var fieldName = $(event.target).closest('.zh-standard-column').attr('data-zh-field-name');
        if(fieldName == "STATUS"){
            var status = widget.currentRow.STATUS;
            var cdnmfilial = widget.currentRow.CDNMFILIAL;
            
            if(status == "Não"){
                if(cdnmfilial.indexOf('Parâmetros') != -1){
                    ScreenService.openWindow('gop_parametro#gop00100_orcamento_operacional');
                }else{
                    ScreenService.openWindow('gop_parametro#gop00200_controle_de_unidades_por_gerente').then(function(){
                        var controleWidget = gopUtil.getWidgetContainer('grupo_gerente_widget');
                        controleWidget.getAction('BACK_PENDENCIAS').isVisible = true;
                        
                    });
                }
            }
        }
    };
    
    this.openWindowOperac = function(operacWidget){
        if(operacWidget.currentRow.DESCRICAO.indexOf('a ser concluíd') != -1){
            ScreenService.openWindow('gop#ger03000_conclusao_prep_orca').then(function(){
                var conclusaoWidget = gopUtil.getWidgetContainer('widgetMainConclusao');
                conclusaoWidget.getAction('BACK_PENDENCIAS').isVisible = true;
                conclusaoWidget.activate();
                   if(!Util.isEmptyOrBlank(conclusaoWidget.currentRow)){
                        conclusaoWidget.currentRow.IDSTATUS = 'PENDENTE';
                         conclusaoWidget.currentRow.CDFILIAL = [];
                        conclusaoWidget.currentRow.CDFILIAL[0] = operacWidget.currentRow.CDFILIAL;
                          conclusaoWidget.currentRow.NMFILIAL = [];
                        conclusaoWidget.currentRow.NMFILIAL[0] = operacWidget.currentRow.NMFILIAL;   
                        conclusaoWidget.currentRow.NRANOORCAGOP = operacWidget.currentRow.NRANOORCAGOP;
                        
                    }
                    gopUtil.startFilter(conclusaoWidget,conclusaoWidget.currentRow);
                   
                     ScreenService.closePopup(true);
                
            });
        }else{
            ScreenService.openWindow('gop#gop11000_aprovacao_orca_operac').then(function(){
                var aprovaWidget = gopUtil.getWidgetContainer('widgetMainAprovacao');
                aprovaWidget.getAction('BACK_PENDENCIAS').isVisible = true;
                aprovaWidget.activate();
                    if(!Util.isEmptyOrBlank(aprovaWidget.currentRow)){
                        aprovaWidget.currentRow.IDSTATUS = 'PENDENTE';
                        aprovaWidget.currentRow.CDFILIAL = [];
                        aprovaWidget.currentRow.CDFILIAL[0] = operacWidget.currentRow.CDFILIAL;
                        aprovaWidget.currentRow.NMFILIAL = [];
                        aprovaWidget.currentRow.NMFILIAL[0] = operacWidget.currentRow.NMFILIAL;
                        aprovaWidget.currentRow.NRANOORCAGOP = operacWidget.currentRow.NRANOORCAGOP;
                    }
                 gopUtil.startFilter(aprovaWidget,aprovaWidget.currentRow);
                ScreenService.closePopup(true);
            });
        }
    };
    /**
     * Função utilizada no redirecionamento para a tela de
     * 'Verificação de Pendências', utilizado pelas telas de
     * 'Controle de Gerentes por Unidade'
     * 'Conclusão de Orçamento'
     * 'Aprovação do Orçamento'
     * @param {object(widget)} widget widget ativa
     * Chamada nas situações em que as telas forem acessadas através da
     * 'Verificação de Pendencias', a fim de facilitar o retorno 
     * após o tratamento de uma pendência.
    **/ 
    
    this.backAction = function(controleWidget){
        controleWidget.widgets.forEach(function(widget){
            if(!Util.isEmptyOrBlank(widget.getAction('BACK_PENDENCIAS'))){
                 widget.getAction('BACK_PENDENCIAS').isVisible = false; 
            }
        });
    };
    
    this.openWindowPendencias = function(tela){
        ScreenService.openWindow('gop_relatorio#gop13000_verificacao_de_pendencias').then(function(){
            var filterWidget = gopUtil.getWidgetContainer("filterSelect");
            if(tela == 'OPERAC'){
                this.setFiltroParam(filterWidget);
                this.setFiltroOperac(filterWidget);
            }else{
                this.setFiltroParam(filterWidget);
            }
        }.bind(this));
    };
    
    /*-----------------------------------------------------------------------*/


    /*Uteis*/
    
    /**Função que estabelece uma relação de dependência
     * entre um campo com template "checkbox" e um com template
     * "select". Sempre que o checkbox for marcado o select fica
     * bloqueado (readOnly = true), quando o esse for desmarcado
     * aquele fica desbloqueado (readOnly = false). 
    **/

	this.checkboxLimpaCampo = function(field, nome) {
		if (field.value() !== "T") {
			this.limpaCampo(field, nome);
		}
	};

	this.limpaCampo = function(field, nome) {
		var campo = field.widget.getField(nome);
		campo.setValue('');
	};
}
Configuration(function(ContextRegister) {
    ContextRegister.register('gop13000_verificacao_de_pendencias_Controller', gop13000_verificacao_de_pendencias_Controller);
});

// FILE: ./assets/modules/parametro/assets/js/controllers/gop00100_parametros_orcamento_Controller.js
function gop00100_parametros_orcamento_Controller(ScreenService, General, templateManager, gopUtil, gop00100_parametros_orcamento_Service, MessageService) {

    this.onEnter = function(){
        General.justInteger('#NRDIAANTELABORCA');
        General.justInteger('#NRDIAINIORCA');
    };
    
    this.afterLoadDataSource = function(args){
        var widget = args.owner;
        this.setParametroRow(widget);
    };

    this.setParametroRow = function(widget){
        
        if(widget.dataSource.data[0]){
            widget.currentRow = widget.dataSource.data[0];
        }else {
            widget.applyDefaultValue();
            widget.currentRow.__is_new = true;
            widget.currentRow.CDPARORCAOPERAC = "00001";
        }
    };
    
    this.edita = function(widget){
  		gopUtil.enabledDisableWidgets(widget, false);
        widget.edit();
        widget.template = "widget/form.html";
        this.utilizaOrca(widget);
    };
    
    this.backEdit = function(args){
		var widget = args.owner.widget;
		
		gopUtil.enabledDisableWidgets(widget, true);
		 
		General.backOrCancel(args);
		widget.view();
		widget.reload();
		
	};

    this.save = function(widget){
        if(widget.isValid() && this.validateDate(widget)){
            widget.currentRow.NRDIAINIORCA = parseFloat(widget.currentRow.NRDIAINIORCA);
            General.saveCrud(widget).then(function(){
                gopUtil.enabledDisableWidgets(widget, true);
                widget.template = "widget/form-view.html";
                widget.list();
                if(widget.currentRow.NRDIAINIORCA > 28){
                    MessageService.showMessageByCode('PARAMETRO_MESES_DIAS', {"PARAMETRO": widget.currentRow.NRDIAINIORCA});
                }
                ScreenService.successNotification("Parametrização salva com sucesso.");
            }.bind(this), function(erro){
                ScreenService.showMessage(erro);
            });
        }
    };
    
    this.openPopupFiltro = function(){
         var cadastramentoAutomatico = templateManager.container.getWidget('cadastramentoAutomatico');
         cadastramentoAutomatico.applyDefaultValue();
         ScreenService.openPopup(cadastramentoAutomatico);
    };
    
    this.salvaUnidade = function(widget){
        
         if(this.percentual(widget) && widget.isValid()){
             if(widget.currentRow.NRDIAINIORCAGOP != "" && widget.currentRow.NRDIAINIORCAGOP != null){
                 if(this.validateDateGop(widget)){
                     
                    widget.currentRow.VRPEORCBLOQFIL = gopUtil.parseMaskStringToFloat(widget.currentRow.VRPEORCBLOQFIL);
                    widget.currentRow.VRPEORCALERTFIL = gopUtil.parseMaskStringToFloat(widget.currentRow.VRPEORCALERTFIL);
                    
                    General.saveCrud(widget).then(function(){
                    ScreenService.successNotification("Operação realizada com sucesso");
                    widget.reload();
                    }, function(erro){
                        ScreenService.showMessage(erro);
                    });
                }
             }else{
                General.saveCrud(widget).then(function(){
                ScreenService.successNotification("Operação realizada com sucesso");
                  widget.reload();
                }, function(erro){
                    ScreenService.showMessage(erro);
                });
             }
         }
         
    };
    
    this.percentual = function(widget){
        
        var retorno;
        var alertaLimite;
        var alertaBloqueio;
        var bloqueio = widget.currentRow.VRPEORCBLOQFIL;
        var alerta = widget.currentRow.VRPEORCALERTFIL;
        
        if(alerta != "" && alerta != null){
            
            if(alerta < 1 || alerta > 100 ){
                
                ScreenService.showMessage("O percentual de Alerta de Limite deve estar entre 1 e 100");
                widget.currentRow.VRPERCENTPU = "";
                alertaLimite = false;
                
            }else{
                alertaLimite = true;
            }

        }else{
            alertaLimite = true;
        }
        
        if(bloqueio != "" && bloqueio != null ){
            
            if(bloqueio < 1 || bloqueio > 100 ){
                
                ScreenService.showMessage("O percentual de Tolerância para Bloqueio deve estar entre 1 e 100");
                widget.currentRow.VRPERCENTPU = "";
                alertaBloqueio = false;
                
            }else{
                alertaBloqueio = true;
            }
        }else{
            alertaBloqueio = true;
        }
        
        
        if(!alertaLimite || !alertaBloqueio){
            
            retorno = false;
            
        }else{
            
            retorno = true;
        }
        
        return retorno;
    };
    
    this.cadastramentoAutomatico = function(widget){
        var unidade = gopUtil.getWidgetContainer('gop00100_orcamento_operacional_unidade');

        if(widget.isValid()){

            if(!Util.isEmptyOrBlank(widget.currentRow.VRPEORCALERTFIL))
                widget.currentRow.VRPEORCALERTFIL = gopUtil.parseMaskStringToFloat(widget.currentRow.VRPEORCALERTFIL);
            if(!Util.isEmptyOrBlank(widget.currentRow.VRPEORCBLOQFIL))
                widget.currentRow.VRPEORCBLOQFIL = gopUtil.parseMaskStringToFloat(widget.currentRow.VRPEORCBLOQFIL);
        
            if(this.validateDate){
                gop00100_parametros_orcamento_Service.atualizaFilial([widget.currentRow]).then(function(){
                    unidade.reload();
                    ScreenService.showMessage("Operação realizada com sucesso!");
                    ScreenService.closePopup();
                    widget.list();
              
                }.bind(this),function(ex){
                    MessageService.showMessageByCode("MSG_ERRO", {"PARAMETRO": ex.data.error});
                });
            }
        }
    };

    this.validateDateGop = function(widget){
        if(widget.currentRow.NRDIAINIORCAGOP > 31 || widget.currentRow.NRDIAINIORCAGOP < 1 ){
            ScreenService.showMessage("Dia Inicial do Orçamento Inválido");
            return false;
        }else{
            return true;
        }
    };
    
    this.validateDate = function(widget){
        if(widget.currentRow.NRDIAINIORCAGOP != "" && widget.currentRow.NRDIAINIORCAGOP != null){
            if(widget.currentRow.NRDIAINIORCA > 31 || widget.currentRow.NRDIAINIORCA < 1 ){
                ScreenService.showMessage("Dia Inicial do Orçamento Inválido");
                return false;
            }else{
                return true;
            }
        }else{
            return true;
        }
    };
    
    this.utilizaOrca = function(widget){
        if(widget.currentRow.IDUTILORCAREALOC == "S"){
            widget.getField('IDCONSORCREALOC').readOnly = false;
            widget.getField('IDREALSUPORCA').readOnly = false;
            widget.getField('IDREALSUPORCA').validations = {required: true};
            widget.getField('IDREALSUPORCA').hideEmptyValue = true;
            widget.getField('IDREALSUPORCA').value(widget.getField('IDREALSUPORCA').dataSource.getFirstRow().value);
            widget.getField('IDREALINFORCA').readOnly = false;
            widget.getField('IDREALINFORCA').validations = {required: true};
            widget.getField('IDREALINFORCA').hideEmptyValue = true;    
            widget.getField('IDREALINFORCA').value(widget.getField('IDREALINFORCA').dataSource.getFirstRow().value);
        } else if(widget.currentRow.IDUTILORCAREALOC == "N"){
            widget.getField('IDCONSORCREALOC').readOnly = true;
            widget.getField('IDCONSORCREALOC').value("N") ;
            // var field = widget.getField('IDREALSUPORCA');
            widget.getField('IDREALSUPORCA').readOnly = true;
            widget.getField('IDREALSUPORCA').hideEmptyValue = false;
            widget.getField('IDREALSUPORCA').value("");
            widget.getField('IDREALSUPORCA').validations = {};
            widget.getField('IDREALINFORCA').readOnly = true;
            widget.getField('IDREALINFORCA').hideEmptyValue = false;
            widget.getField('IDREALINFORCA').value("");
            widget.getField('IDREALINFORCA').validations = {};
        }
        templateManager.updateTemplate();
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('gop00100_parametros_orcamento_Controller', gop00100_parametros_orcamento_Controller);
});

// FILE: ./assets/modules/parametro/assets/js/controllers/gop00200_controle_de_unidades_por_gerenteController.js
function gop00200_controle_de_unidades_por_gerenteController(ScreenService, General, templateManager, gopUtil, Gop00200_controle_de_unidades_por_gerenteService, ZHPromise, MessageService) {
    var self = this;
    
    this.newRow = function(widget){
        General.widgetNewRow(widget);
        gopUtil.getWidgetContainer('operador_por_grupo').enabled = false;
        
    };
    
    this.adicionaUnidade = function(widget){
        widget.getField("NMFILIAL").dataSourceFilter = [
            {name: "P_CDOPERADOR", operator: "=", value: widget.getParent().currentRow.CDOPERADOR}
        ];
        widget.getField("IDACESSOCALC").value("N");
        widget.getField("IDACESSOAPROV").value("N");
        gopUtil.adiciona(widget);
    };
    
    this.desmarcaFieldsCheckbox = function(widget){
        widget.getField("IDACESSOCALC").checked = false;
        widget.getField("IDACESSOAPROV").checked = false ;
        widget.reload();
        
    };
    
    this.editRow = function(widget){
        gopUtil.edita(widget);
        // this.enableGrupoTabs(false);
    };
    
    this.cancelEdit = function(args, widget){
        if(widget.currentRow.__is_new === true){
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
            widget.list();
        }
        else{
            General.cancelModEdit(args);
            widget.reload();
            widget.view();
        }
        this.enableGrupoTabs(true);
    };
    
    this.enableGrupoTabs = function(flag){
        gopUtil.getWidgetContainer('operador_por_grupo').enabled = flag;
        gopUtil.getWidgetContainer('filial_por_grupo').enabled = flag;
    };
    
    this.applyFilterMoveRow = function(widget){
       
        this.enableGrupoTabs(widget);
        
        var filtroOperador =  [
                {
                    "name": "P_CDGRUPGERENTEOP",
                    "operator": "=",
                    "value": widget.currentRow.CDGRUPGERENTEOP
                }
        ];
        gopUtil.getWidgetContainer("operador_por_grupo").dataSourceFilter = filtroOperador;
    };
    
    this.applyFilterMoveRowUnidade = function(widget){
        
        this.enableGrupoTabs(widget);
        
        var filialWidget = gopUtil.getWidgetContainer("filial_por_grupo");
        
        var filtroFilial =  [
                { "name" : "P_CDOPERADOR", "value": widget.currentRow.CDOPERADOR, "operator": "="},
                { "name" : "P_CDFILIAL", "value": '', "operator": "="}
        ];
        
        var filtroFieldFilial = [
            { "name" : "P_CDOPERADOR", "value": widget.currentRow.CDOPERADOR, "operator": "="}  
        ];
        
        filialWidget.dataSourceFilter = filtroFilial;
        filialWidget.getField('NMFILIAL').dataSourceFilter = filtroFieldFilial;

    };
    
    this.saveGerente = function(widget){
        var filialWidget = gopUtil.getWidgetContainer('filial_por_grupo');
        if(widget.isValid()){
            widget.currentRow.CDGRUPGERENTEOP = widget.getParent().currentRow.CDGRUPGERENTEOP;
            var filtroOperador =  [
                    {
                        "name": "P_CDGRUPGERENTEOP",
                        "operator": "=",
                        "value": widget.currentRow.CDGRUPGERENTEOP
                    }
            ];
            gopUtil.getWidgetContainer("operador_por_grupo").dataSourceFilter = filtroOperador;
            
            Gop00200_controle_de_unidades_por_gerenteService.saveOperadorGrupo(widget.currentRow).then(function(){
                this.enableGrupoTabs(true);
                filialWidget.dataSourceFilter[0].value = widget.currentRow.CDOPERADOR;
                widget.view();
                widget.reload();
                ScreenService.successNotification("Operação realizada com sucesso");
                
            }.bind(this),function(retorno){
                ScreenService.showMessage(retorno.data.error);
            });
        }
        
    };
    
    this.saveGrupo = function(widget){
        if(widget.isValid()){
            this.saveCrud(widget).then(function(response){
                if (widget.currentRow.__is_new == true)
                    widget.currentRow.CDGRUPGERENTEOP = response.dataset.grupo_gerente[response.dataset.grupo_gerente.length - 1].CDGRUPGERENTEOP;
                    ScreenService.successNotification("Operação realizada com sucesso!");
                    this.enableGrupoTabs(true);
                }.bind(this),function(ex){
                    ScreenService.showMessage(ex.data.error);
                });
        }
    };
    
    this.saveFilialGrupo = function(widget){
        
        if(widget.isValid()){
            widget.currentRow.CDGRUPGERENTEOP = widget.getParent().currentRow.CDGRUPGERENTEOP;
            widget.currentRow.CDOPERADOR = widget.getParent().currentRow.CDOPERADOR;
            
            gopUtil.saveCrudBackToView(widget).then(function(){
                ScreenService.successNotification("Operação realizada com sucesso!");
                widget.currentRow = widget.dataSource.data.filter(row=>row.CDFILIAL == widget.currentRow.CDFILIAL)[0];
                self.enableGrupoTabs(true);
            }.bind(this),function(error){
                ScreenService.showMessage(error);
            }.bind(this));
        }
    };
    
    this.deleteGrupo = function(widget){
        
        var mensagem = ScreenService.i18n('Deseja excluir o registro?');
        ScreenService.confirmMessage(mensagem, 'question', function(success) { 
        Gop00200_controle_de_unidades_por_gerenteService.excluirGrupoGerente(widget.currentRow).then(function(response){
            if(response == false){
                ScreenService.closeSwipe();
                self.enableGrupoTabs(true);
                widget.list();
                widget.reload();
                MessageService.notificationMessageByCode('SUC_DELETE', null, Constants.successNotification);
            }else{
                ScreenService.showMessage(response.dataset.errorList[0]);
            }
        }.bind(this),function(ex){
              ScreenService.showMessage(ex.data.error);
            });
        });
    };

	this.saveCrud = function (widget) {
		var deferred = ZHPromise.defer();
		if (widget.isValid()) {

			var paramsNewCode = widget.paramsNewCode;
			if (paramsNewCode && paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = paramsNewCode.__table_pk;
				widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
				widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
			}

			var fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
			if (fieldsToValidateDuplication && fieldsToValidateDuplication.length){
				General.setDuplicationValidateParamsToCurrentRow(widget);
			}

			widget.saveRow().then(function (success) {
				widget.activate();
				if (widget.currentRow.__is_new) {
					widget.reload().then(function (callback) {
    					widget.view();
						deferred.resolve(callback);
					});
				} else {
					widget.computeDescriptionOrder();
					widget.view();
					deferred.resolve(success);
				}
			}.bind(this), function (error) {
				var errorMessage = 'TIMEOUT';
				if (error.data) {
					errorMessage = error.data.error;
				}
				MessageService.showMessageByCode(errorMessage);
			});
		} else {
			// @todo create messages class
			deferred.reject("Preencha todos os campos corretamente!");
		}

		return deferred.promise;
	};
	
	this.deleteOperadorGrupo = function(widget){
        
        var mensagem = ScreenService.i18n('Deseja excluir o registro?');
        ScreenService.confirmMessage(mensagem, 'question', function(success) { 
        Gop00200_controle_de_unidades_por_gerenteService.excluirOperadorGrupo(widget.currentRow).then(function(response){

            ScreenService.closeSwipe();
            widget.reload();
            MessageService.notificationMessageByCode('SUC_DELETE', null, Constants.successNotification);
          
        }.bind(this),function(ex){
              ScreenService.showMessage(ex.data.error);
            });
        });
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('gop00200_controle_de_unidades_por_gerenteController', gop00200_controle_de_unidades_por_gerenteController);
});

// FILE: ./assets/modules/parametro/assets/js/controllers/gop02000_calendario_prep_orcaController.js
function gop02000_calendario_prep_orcaController(ScreenService, General, templateManager, gopUtil, restEngine, gop02000_calendario_prep_orca_Service, ZhKeyboardService, eventEngine) {

    this.openPopupFiltroOnce = function() {
        var filterWidget = gopUtil.getWidgetContainer('filterSelect');
        
        restEngine.post('gop#/crud/paramorcaoperac', {
            "requestType": "FilterData",
            "filter": ""
        }).then(function(callback) {
            var parametros = callback.dataset.paramorcaoperac[0];

            filterWidget.currentRow.NRDIAANTELABORCA = parametros.NRDIAANTELABORCA;
            filterWidget.currentRow.NRDIAINIORCA = parametros.NRDIAINIORCA;
            ScreenService.openPopup(filterWidget).then(function(){
                this.setPeriodo();                      
            }.bind(this));
            filterWidget.activate();

        }.bind(this), function(erro) {
            ScreenService.showMessage(erro);
        });
    };

    /* Seta o período de acordo com os parametros*/
    this.setPeriodo = function() {
        var filterWidget = gopUtil.getWidgetContainer('filterSelect');
        var idperiodo = filterWidget.dataSource.data[0].IDPERIODOORCA;
        
        filterWidget.currentRow.IDPERIODOORCA = idperiodo;

        if (!Util.isEmptyOrBlank(idperiodo)) {
            if (idperiodo != "ANUAL") {
                filterWidget.getField('NRPERIODOORCA').dataSource.data = gopUtil.getPeriodo(idperiodo);
                filterWidget.getField("NRPERIODOORCA").readOnly = false;
                filterWidget.getField("NRPERIODOORCA").validations = {
                    "required": { "message": "Campo obrigatório" }
                };
            } else {
                filterWidget.getField("NRPERIODOORCA").readOnly = true;
                filterWidget.getField('NRPERIODOORCA').dataSource.data = [
                    { "name": "Anual", "value": "1", "mesInicial": "01"}
                ];
                filterWidget.currentRow.NRPERIODOORCA = 1;
                filterWidget.getField('NRPERIODOORCA').isVisible = false;
            }
        } else {
            filterWidget.getField("NRPERIODOORCA").readOnly = true;
            filterWidget.getField("NRPERIODOORCA").validations = {
                "required": false
            };
        }
    };
    
    //Abre a widget de filtro ('filterSelect')
    this.openPopupFiltro = function() {
        var filterWidget = gopUtil.getWidgetContainer("filterSelect");
        filterWidget.currentRow.IDPERIODOORCA = filterWidget.dataSource.data[0].IDPERIODOORCA;
        ScreenService.openPopup(filterWidget);
        filterWidget.activate();
    };
    
    this.afterInitContainer = function() {
        let filterSelect = gopUtil.getWidgetContainer('filterSelect');
        var ref = this;

        filterSelect.openFilterShortcut = eventEngine.bindEvent(ZhKeyboardService, 'mod+lPressed', function(eventData) {
            return ref.openPopupFiltro();
        });
    };
    
    this.beforeCloseContainer = function(args) {
        let filterSelect = gopUtil.getWidgetContainer('filterSelect');
        if (filterSelect.openFilterShortcut >= 0)
            eventEngine.unbindEvent(ZhKeyboardService, 'mod+lPressed', filterSelect.openFilterShortcut);
    };

    this.setFiltro = function(filterWidget) {
        var mainWidget = gopUtil.getWidgetContainer('widgetCalendario');
        var nrperiodo = filterWidget.currentRow.NRPERIODOORCA;
        
        if(filterWidget.isValid()) {
            if (filterWidget.currentRow.IDPERIODOORCA != "ANUAL")
                mainWidget.currentRow.DESCNRPERIODOORCA = filterWidget.getField('NRPERIODOORCA').dataSource.data[nrperiodo - 1].name;
            else
                mainWidget.getField('DESCNRPERIODOORCA').isVisible = false;
                
            mainWidget.dataSourceFilter = [
                { name: "P_NRANOORCAGOP", operator: "=", value: filterWidget.currentRow.NRANOORCAGOP},
                { name: "P_IDPERIODOORCA", operator: "=", value: filterWidget.currentRow.IDPERIODOORCA},
                { name: "P_NRPERIODOORCA", operator: "=", value: nrperiodo}
            ];
            
            this.setStripe(filterWidget);
            
            ScreenService.closePopup();
            mainWidget.reload();
            mainWidget.activate();
        }
    };

    this.setStripe = function(filterWidget) {
        var stripeWidget = gopUtil.getWidgetContainer("stripeWidget");
        var nrperiodoorca = filterWidget.getField('NRPERIODOORCA');
        var stripeMsg;
        
        if(nrperiodoorca.dataSource.data.length > 0)
            stripeMsg = "Período: " + nrperiodoorca.dataSource.data[filterWidget.currentRow.NRPERIODOORCA - 1].name;
        else
            stripeMsg = "Período: Anual";

        stripeWidget.getField("ANO").label = "Ano: " + filterWidget.currentRow.NRANOORCAGOP;
        stripeWidget.getField("PERIODO").label = stripeMsg;
    };

    this.cancelEdit = function(args, widget) {
        
        if (widget.currentRow.__is_new === true) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
            widget.list();
        } else {
            General.cancelModEdit(args);
            widget.reload();
            widget.view();
        }
    };
    
    this.diaAlerta = function(widget) {
        if (widget.currentRow.NRDIAALERTA > widget.currentRow.NRDIAAREALIZA) {
            ScreenService.showMessage("O campo Dias Alerta não pode ser maior que o campo Dias de Duração. ");
            return false;
        }
        return true;
    };

    this.mes = function(widget, filterWidget) {
        var idperiodo = filterWidget.dataSource.data[0].IDPERIODOORCA;
        var mes = gopUtil.getPeriodo(idperiodo)[filterWidget.currentRow.NRPERIODOORCA - 1].mesInicial;
        
        return mes;
    };

    this.duracao = function(widget) {
        var filterWidget = gopUtil.getWidgetContainer("filterSelect");
        
        if (widget.currentRow.NRDIAAREALIZA > 0) {
            var diaInicio = widget.currentRow.DTINICIOPRE;
            var diaInicioMoment = moment(diaInicio, 'DD/MM/YYYY');
            var iniPeriodo = filterWidget.currentRow.NRDIAINIORCA + "/" + this.mes(widget, filterWidget) + "/" + widget.currentRow.NRANOORCAGOP;
            var iniPeriodoMoment = moment(iniPeriodo, 'DD/MM/YYYY');
            var duracao = widget.currentRow.NRDIAAREALIZA;

            if ((diaInicioMoment.add(duracao, "days")).isAfter(iniPeriodoMoment)) {
                ScreenService.showMessage("A duração do pré orçamento não deve ultrapassar o início do orçamento.");
                return false;
            } else {
                return true;
            }
        } else {
            ScreenService.showMessage("O campo Dias de Duração deve ser maior que 0.");
            return false;
        }
    };
    
    /*Retorna a dataInicial e a duração da etapa anterior*/
    this.getRegistroAnterior = function(mainWidget){
        var nrcalendario = [];
        var registro;
        var checkNr;
        
        mainWidget.dataSource.data.forEach(function(row) {
            if(!Util.isEmptyOrBlank(row.NRCALENPREORCAGOP))
                nrcalendario.push(row.NRCALENPREORCAGOP);
        });
        
        nrcalendario.forEach(function(nr) {
            if (Util.isEmptyOrBlank(checkNr)) {
                checkNr = nr;
            } else {
                if (checkNr < nr) {
                    checkNr = nr;
                }
            }
        });
        registro = mainWidget.dataSource.data.filter(row=>row.NRCALENPREORCAGOP == checkNr)[0];
        return {dataInicioAnterior: registro.DTINICIOPRE, diasRealizacao: registro.NRDIAAREALIZA};
    };
    
    this.setDataInicialDefault = function(mainWidget, filterWidget){
        var registroAnterior = this.getRegistroAnterior(mainWidget);
        var ano = mainWidget.dataSourceFilter.filter(param => param.name == "P_NRANOORCAGOP")[0].value;
        var dataFinalMoment;

        if (mainWidget.dataSource.data.length > 1) {
            var dataFinalAnteriorMoment = moment(registroAnterior.dataInicioAnterior, "DD/MM/YYYY");
            dataFinalMoment = dataFinalAnteriorMoment.add((registroAnterior.diasRealizacao + 1), 'day');
            mainWidget.currentRow.DTINICIOPRE = dataFinalMoment.format('DD/MM/YYYY');
            mainWidget.getField('DTINICIOPRE').defaultValue = dataFinalMoment;
        } else {
            var diaOrca = filterWidget.currentRow.NRDIAINIORCA + "/" + this.mes(mainWidget, filterWidget) + "/" + ano;
            var lastMonthDay = moment(diaOrca.replace(diaOrca.substr(0, 2), '01'), 'DD/MM/YYYY').endOf('month').format('DD'); 
            var diaOrcaMoment = diaOrca.substr(0, 2) > lastMonthDay ? moment(lastMonthDay + diaOrca.substr(2), "DD/MM/YYYY") : moment(diaOrca, "DD/MM/YYYY");
            dataFinalMoment = diaOrcaMoment.subtract(filterWidget.currentRow.NRDIAANTELABORCA, 'day');

            if (dataFinalMoment.isAfter(diaOrcaMoment)) {
                ScreenService.showMessage("Dia Inicial deve ser antes do Período do Orçamento");
                return false;
            } else {
                mainWidget.currentRow.DTINICIOPRE = dataFinalMoment.format("DD/MM/YYYY");
                mainWidget.getField('DTINICIOPRE').defaultValue = dataFinalMoment;
                return true;
            }
        }
    };

    this.verificaAdiciona = function(mainWidget) {
        var filtro = [
            {"name": "IDPERIODOORCA", "operator": "=", "value": mainWidget.dataSourceFilter.filter(a=>a.name == "P_IDPERIODOORCA")[0].value},
            {"name": "NRPERIODOORCA", "operator": "=", "value": mainWidget.dataSourceFilter.filter(a=>a.name == "P_NRPERIODOORCA")[0].value},
            {"name": "NRANOORCAGOP", "operator": "=", "value": mainWidget.dataSourceFilter.filter(a=>a.name == "P_NRANOORCAGOP")[0].value}
        ];
        
        gop02000_calendario_prep_orca_Service.existeConcpreorcagop(filtro).then(function(response){
            var retorno = response.dataset.gop02000_existe_concpreorcagop[0];
            if(!Util.isEmptyOrBlank(retorno)){
                if(retorno.IDSTATUS == "PENDENTE"){
                    var mensagem = ScreenService.i18n("O orçamento do período selecionado já foi concluído, porém foi reprovado ou cancelado. Deseja adicionar novas etapas?");
                    ScreenService.confirmMessage(mensagem, 'question', function(success) {
                        this.adiciona(mainWidget);
                    }.bind(this));

                }else if(retorno.IDSTATUS == "CONCLUIDO"){
                    ScreenService.showMessage("Impossível adicionar. Existe orçamento concluído no período selecionado.");
                }
            }else{
                this.adiciona(mainWidget);
            }
                

        }.bind(this), function(erro) {
            ScreenService.showMessage(erro);
        });
        
    };
    
    this.adiciona = function(mainWidget){
        var filterWidget = gopUtil.getWidgetContainer("filterSelect");
        
        if (!Util.isEmptyOrBlank(filterWidget.currentRow.NRANOORCAGOP)) {
            General.widgetNewRow(mainWidget);
            mainWidget.currentRow.IDPERIODOORCA = filterWidget.currentRow.IDPERIODOORCA;
            mainWidget.currentRow.NRPERIODOORCA = filterWidget.currentRow.NRPERIODOORCA;
            mainWidget.currentRow.NRANOORCAGOP = filterWidget.currentRow.NRANOORCAGOP;

            if (mainWidget.currentRow.IDPERIODOORCA != "ANUAL")
                mainWidget.currentRow.DESCNRPERIODOORCA = filterWidget.getField('NRPERIODOORCA').dataSource.data[(mainWidget.currentRow.NRPERIODOORCA) - 1].name;
            
            this.setDataInicialDefault(mainWidget, filterWidget);
        } else {
            ScreenService.showMessage("É necessário filtrar o Ano e o Período do Orçamento para adicionar.");
            this.openPopupFiltro();
        }
    };

    this.save = function(widget) {
        if (widget.isValid() && this.diaAlerta(widget) && this.verificaDataInicial(widget) && this.duracao(widget)) {
            if (widget.currentRow.IDPERIODOORCA == "ANUAL") {
                widget.currentRow.NRPERIODOORCA = "1";
            }
            General.saveCrud(widget).then(function() {
                ScreenService.closeSwipe();
                widget.reload();
            }, function(erro) {
                ScreenService.showMessage(erro);
            });
        }
    };
    
    this.getDate = function(mainWidget){
        var filterWidget = gopUtil.getWidgetContainer("filterSelect");
        
        var registroAnterior = this.getRegistroAnterior(mainWidget);
        var dataAnterior = registroAnterior.dataInicioAnterior;
        var dataFinalAnteriorMoment = moment(dataAnterior, "DD/MM/YYYY");
        var dataFinal = dataFinalAnteriorMoment.add(registroAnterior.diasRealizacao, 'day');
        
        var diaInicio = mainWidget.currentRow.DTINICIOPRE;
        var diaInicioMoment = moment(diaInicio, 'DD/MM/YYYY');
        var diasAntesElaboracao = filterWidget.currentRow.NRDIAANTELABORCA;
        var dataFinalMoment = diaInicioMoment.subtract(diasAntesElaboracao, 'day');
        

        var dia = filterWidget.currentRow.NRDIAINIORCA;
        var mes = this.mes(mainWidget, filterWidget);
        var ano = mainWidget.dataSourceFilter.filter(param => param.name == "P_NRANOORCAGOP")[0].value;
        if (dia < 10)
            dia = "0" + dia;
        var diaOrcaMoment = moment(dia + "/" + mes + "/" + ano, "DD/MM/YYYY");
            
        return { "dataFinal": dataFinal, "dataFinalMoment": dataFinalMoment, "diaOrcaMoment": diaOrcaMoment};
    };
    
    this.verificaDataInicial = function(mainWidget) {
        var date = this.getDate(mainWidget);

        if (Util.isEmptyOrBlank(mainWidget.currentRow.DTINICIOPRE))
            return true;

        if (mainWidget.dataSource.data.length > 1 && date.dataFinalMoment.isBefore(date.dataFinal)) {
            ScreenService.showMessage("Data Inicial Prevista deve ser igual ou maior que a Data Final Prevista da atividade anterior");
            return false;
            
        } else {
            if (date.dataFinalMoment.isBefore(date.diaOrcaMoment)) {
                return true;
            } else {
                ScreenService.showMessage("Dia Inicial deve ser antes do Período do Orçamento");
                return false;
            }
        }
    };

    this.rowDesc = function() {
        var filterWidget = gopUtil.getWidgetContainer("filterSelect");
        var mainWidget = gopUtil.getWidgetContainer('widgetCalendario');
        var nrperiodo =  mainWidget.dataSourceFilter.filter(param => param.name == "P_NRPERIODOORCA")[0].value;
        
        if(filterWidget.currentRow.IDPERIODOORCA == "ANUAL")
            mainWidget.getField('DESCNRPERIODOORCA').isVisible = false;

        if(mainWidget.currentRow.__is_new != true)
            mainWidget.currentRow.DESCNRPERIODOORCA = filterWidget.getField('NRPERIODOORCA').dataSource.data[(nrperiodo) - 1].name;
    };

    this.voltar = function() {
        var mainWidget = gopUtil.getWidgetContainer('widgetCalendario');
        ScreenService.closePopup();
        mainWidget.activate();
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('gop02000_calendario_prep_orcaController', gop02000_calendario_prep_orcaController);
});

// FILE: ./assets/modules/parametro/assets/js/controllers/gop21000_percentual_por_item_operacional_Controller.js
function gop21000_percentual_por_item_operacional_Controller(ScreenService, General, templateManager, gopUtil, gop21000_percentual_por_item_operacional_Service, ZhUtil, maskEngine, MessageService){
    /*global Util, getLocalVar*/  
    var self = this;
    
    this.orcamentoUnidadeMoveRow = function(widget){
        var orcamentoUnidadeWidget = gopUtil.getWidgetContainer("ORCAMENTO_UNIDADE_ITEM") ;
        orcamentoUnidadeWidget.dataSourceFilter = [
            {"name": "P_CDFILIAL", "value": widget.currentRow.CDFILIAL, "operator": "="},
            {"name": "NRORG", "value": getLocalVar('NRORG'), "operator": "="}
        ];
    };
    
    this.desabilitaWidget = function(widget, bool){
        widget.widgets.forEach(function(wid){
           wid.enabled  = bool; 
        });
    };
    
    this.adicionar = function(widget){
        widget.widgets.forEach(widget => {widget.enabled = false;});
        widget.newRow();
    };
    
    this.edita = function(widget){
        this.desabilitaWidget(widget, false);
        General.edit(widget);
    };
    
    this.cancela = function(args){
        this.desabilitaWidget(args.owner.widget, true);
        General.backOrCancel(args);
        args.owner.widget.reload();
    };
    
    this.exclui = function(widget, confirmMsg){
        gopUtil.exclui(widget, "", confirmMsg, ["list", "view"]);
    };
    
    this.orcamentoDetalhamentoMoveRow = function(widget, idGeralUnid){
        
        //gopUtil.underBlockWidget();
        var nrSeqItem = widget.currentRow.NRSEQITEMORCA;
        var tipo = "";
        var filtroDesp = [
            { "name": "IDATIVO", "operator": "=", "value": "S"},
            { "operator": "ORDER_BY",
              "name": [ 
                  { "name": "CDDESPPREV", "direction": "ASC" }
                ]
            }
        ];
        var nrPercit = idGeralUnid == 1 ? widget.currentRow.NRPERCITEMGOP : widget.currentRow.NRPERCITFILGOP;
        var filtro = [
            { "name": "P_NRPERCITEMGOP", "value": nrPercit, "operator": "=" }
        ];
        var detalhamentoUnidadeWidget = gopUtil.getWidgetContainer('ORCAMENTO_DETALHAMENTO_UNIDADE');
        var detalhamentoWidget = gopUtil.getWidgetContainer('ORCAMENTO_DETALHAMENTO');
        
        if(nrSeqItem == 3 || nrSeqItem == 5 || nrSeqItem == 6 || nrSeqItem == 7){
            
            if(nrSeqItem == 3){
                tipo = "MAOOBRA";
                
            }else if(nrSeqItem == 5){
                tipo = "MATRIZ";
                filtroDesp.push({"name": "IDDESPESAMATRIZ", "value": "S", "operator": "="});
            }
            else if(nrSeqItem == 6){
                tipo = "DIVERSA";
                filtroDesp.push({"name": "IDDESPESAMATRIZ", "value": "N", "operator": "="});
                filtroDesp.push({"name": "IDDESPLANENTR", "value": "N", "operator": "="});
            }
            else{
                tipo = "LANCENTRADA";
                filtroDesp.push({"name": "IDDESPLANENTR", "value": "S", "operator": "="});
                if(widget.currentRow.IDORCDESPENTROP == "O"){
                    nrSeqItem = 71;
                }
            }
            
            if(widget.currentRow.IDORCDESPENTROP == "O" && tipo == "LANCENTRADA"){
                    gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_PRODUTO_UNIDADE").isVisible = false;
                    gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_PRODUTO").isVisible = false;
                    gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_MAOOBRA_UNIDADE").isVisible = false;
                    gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_MAOOBRA").isVisible = false;
                    
            }else{
                gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_PRODUTO_UNIDADE").isVisible = true;
                gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_PRODUTO").isVisible = true;
            }
                
            filtro = [
                { "name": "P_NRPERCITEMGOP", "operator": "=", "value": nrPercit },
                { "name": "P_IDDESPPREV", "operator": "=", "value": tipo }
            ];
            
        }else{
            gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_PRODUTO").isVisible = false;
            gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_PRODUTO_UNIDADE").isVisible = false;
            gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_MAOOBRA").isVisible = false;
            gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_MAOOBRA_UNIDADE").isVisible = false;
        }

        this.exibeColunasItem(nrSeqItem, filtro, filtroDesp, idGeralUnid);
                
        detalhamentoWidget.unSelectAll();
        detalhamentoUnidadeWidget.unSelectAll();
        
    };
    
    this.orcamentoDetalhamentoItemMoveRow = function(args, idGeralUnid){
        var widget = args.owner;
        //this.addCountSwipe();
        if(!widget.currentRow.__is_new){
            widget.widgets.forEach(wid=> {
                if(wid.isVisible)
                    wid.enabled = true;
            });
        }
        if(widget.widgets[0].isVisible || widget.widgets[1].isVisible){
            var nrSeqItem = widget.getParent().currentRow.NRSEQITEMORCA;
                
            if(nrSeqItem == 3 || nrSeqItem == 5 || nrSeqItem == 6 || nrSeqItem == 7){
                this.orcamentoDespesaMoveRow(widget, idGeralUnid);
            }
        }else if(widget.currentRow.__is_new == false){
                args.e.stopPropagation();
        }
        
        

    };
    
    this.orcamentoDespesaMoveRow = function(widget, idGeralUnid){
        ///gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_PRODUTO").dataSourceFilter
        
        var nrPercit = "";
        var filtroDesp;
        var orcamentoDetalheMo = "";
        var orcamentoDespesaWidgetGeral = "ORCAMENTO_DESPESA_PRODUTO";
        var orcamentoDespesaWidgetUnidade = "ORCAMENTO_DESPESA_PRODUTO_UNIDADE";

        if(idGeralUnid == 1){
            nrPercit = widget.currentRow.NRPERCITEMDETALGOP;
            filtroDesp = [
                { "name": "P_NRPERCITEMDETALGOP", "operator": "=", "value": nrPercit },
                { "name": "P_CDDESPPREV", "operator": "=", "value": widget.currentRow.CDDESPPREV }
            ];
            if(widget.currentRow.IDDESPPREV == 'MAOOBRA' || widget.getParent().currentRow.NRSEQITEMORCA == 3){
                orcamentoDetalheMo = gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_MAOOBRA");
                orcamentoDetalheMo.isVisible = true;
                gopUtil.getWidgetContainer(orcamentoDespesaWidgetGeral).isVisible = false;
                widget.getField('DSORCAMENTOMO').dataSourceFilter = [
                    { "name": "TIPO", "operator": "=", "value": "T" },// diario e mensal
                    { "name": "P_CDFILIAL", "operator": "=", "value": null }
                ];
            }else{
                orcamentoDetalheMo = gopUtil.getWidgetContainer(orcamentoDespesaWidgetGeral);
                orcamentoDetalheMo.isVisible = true;
                gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_MAOOBRA").isVisible = false;
                orcamentoDetalheMo.dataSourceFilter = filtroDesp;
                //orcamentoDetalheMo.getField('NMPRODUTO').filterProperties.dataSourceFilter = filtroDesp;
                orcamentoDetalheMo.reload();
            }
        }else if(idGeralUnid == 2){
            nrPercit = widget.currentRow.NRPERCITDETALFILGOP;
            filtroDesp = [
                { "name": "P_NRPERCITDETALFILGOP", "operator": "=", "value": nrPercit },
                { "name": "P_CDDESPPREV", "operator": "=", "value": widget.currentRow.CDDESPPREV },
                { "name": "P_CDFILIAL", "operator": "=", "value": widget.getParent().currentRow.CDFILIAL }
            ];
            if(widget.currentRow.IDDESPPREV == 'MAOOBRA' || widget.getParent().currentRow.NRSEQITEMORCA == 3){
                orcamentoDetalheMo = gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_MAOOBRA_UNIDADE");
                orcamentoDetalheMo.isVisible = true;
                gopUtil.getWidgetContainer(orcamentoDespesaWidgetUnidade).isVisible = false;
                widget.getField('DSORCAMENTOMO').dataSourceFilter = [
                    { "name": "TIPO", "operator": "=", "value": "T" },// diario e mensal
                    { "name": "P_CDFILIAL", "operator": "=", "value": widget.getParent().currentRow.CDFILIAL }
                ];
            }else{
                orcamentoDetalheMo = gopUtil.getWidgetContainer(orcamentoDespesaWidgetUnidade);
                orcamentoDetalheMo.isVisible = true;
                gopUtil.getWidgetContainer("ORCAMENTO_DESPESA_MAOOBRA_UNIDADE").isVisible = false;
                orcamentoDetalheMo.dataSourceFilter = filtroDesp;
                //orcamentoDetalheMo.getField('NMPRODUTO').filterProperties.dataSourceFilter = filtroDesp;
                orcamentoDetalheMo.reload();
            }
        }
    };

    this.exibeColunasItem = function(nrItem, filtro, filtroDesp, idGeralUnid){
        var objColunas =  idGeralUnid == 1 ? gopUtil.buscaColunas(nrItem) : gopUtil.buscaColunasUnidade(nrItem);
        var itemDetalhe = '';
        if(idGeralUnid == 1){
            itemDetalhe = gopUtil.getWidgetContainer("ORCAMENTO_DETALHAMENTO");
        }else{
            itemDetalhe = gopUtil.getWidgetContainer("ORCAMENTO_DETALHAMENTO_UNIDADE");
            filtro.push({"name": "P_CDFILIAL", "operator": "=", "value":  itemDetalhe.getParent().currentRow.CDFILIAL});
        }
        
        if(objColunas){
            itemDetalhe.isVisible = true;
            if(nrItem == 5 || nrItem == 6 || nrItem == 7){
                itemDetalhe.getField("NMDESPPREV").dataSourceFilter = filtroDesp;
                itemDetalhe.getField("NMDESPPREV").filterProperties.dataSourceFilter = filtroDesp;
                itemDetalhe.getField("NMDESPPREV").reload();
            }
        }else{
            itemDetalhe.isVisible = false;
            return;
        }
        
        var fields = itemDetalhe.fields;
        
        fields.forEach(function(field){
           field.isVisible = false; 
        });
        
        fields.forEach(function(field){
            field.isVisible = objColunas.Columns.indexOf(field.name) >= 0;
            field.skipFilter = !field.isVisible;
            
            if(Util.isEmptyOrBlank(itemDetalhe.__filterPopup)){
                if(field.label != "Código" && field.name != "VRPERCENTUAL"){
                    field.filterProperties.isVisible = field.isVisible;
                    field.filterProperties.readOnlyOnEdit = !field.isVisible;
                }
            }else{
                this.setFilterProperties(itemDetalhe, field);
            }   
           
        }.bind(this));
        
        itemDetalhe.getField("VRPERCENTUAL").isVisible = true;
        itemDetalhe.dataSource.name = objColunas.DataSource; 
        itemDetalhe.dataSourceFilter = filtro;
        itemDetalhe.reload();
    };
    
    this.getChangedRows = function(widget) {
        var dataWidget = widget.dataSource.data;
        var editedRows = dataWidget.filter(function(row) {
            return Object.keys(row).filter(function(key) {
                return key.indexOf('changed_') !== -1;
            }).length;
        });
        
        return editedRows;
    };
    
    this.save = function(widget){
        var field = widget.getField("VRPERCITEMGOP");
        var fieldToCheckPercentualName = field.name;
        var invalidRows = 0;
        var mode = widget.mode;
        
        widget.currentRow[field.name] = Number(widget.currentRow[field.name].replace(',', '.')).toFixed(3);
        if(mode != 'inline_edit'){
            if(isNaN(parseFloat(widget.currentRow.VRPERCITEMGOP))){
                invalidRows = {
                    NAME: widget.currentRow.DSITEMORCAMENTO,
                    VALUE: widget.currentRow.VRPERCITEMGOP
                };
            }
        }else{
            var depreciacao = widget.dataSource.data.filter(reg => reg.NRSEQITEMORCA == 11).map(reg => {
                return {
                    NAME: widget.currentRow.DSITEMORCAMENTO,
                    VALUE: widget.currentRow.VRPERCITEMGOP
                };
            });
            if(Util.isEmptyOrBlank(depreciacao)){
                if(isNaN(depreciacao.VALUE)){
                    invalidRows = depreciacao;
                }
            }
        }
        if(widget.isValid()){
            if((Util.isEmptyOrBlank(widget.getField(fieldToCheckPercentualName).value()) ||  widget.getField(fieldToCheckPercentualName).value()< 1000000) && !invalidRows.length){    
                    General.setFloatAttrOnCurrentRow(widget.currentRow, fieldToCheckPercentualName);
                    var fieldToCheckValue = widget.currentRow[fieldToCheckPercentualName];
                    widget.currentRow[fieldToCheckPercentualName] = Util.isEmptyOrBlank(fieldToCheckValue) ? 0 : fieldToCheckValue;
                    General.saveCrud(widget).then(function(){
                        this.desabilitaWidget(widget, true);
                        if(mode == "inline_edit"){
                            widget.list();
                        }else{
                            widget.view();
                        }
                         widget.reload();
                         ScreenService.successNotification("Registro salvo com sucesso.");
                    }.bind(this), function(erro){
                              
                    });
            }else {
                if(invalidRows.length && widget.mode=='inline_edit'){
                    var mensagens = [];
                    invalidRows.forEach(function(reg){
                        mensagens.push(reg.NAME + ': "'+reg.VALUE+'"');
                    });
                    General.showErrorMessages(widget, mensagens, ScreenService.i18n('Percentuais Inválidos'));
                } 
                    
            }
        }
        this.editDepreciacao(widget, true);
        
    };
   
    this.saveItem = function(widget, idGeralUnid){
        var invalidRows = widget.dataSource.data.filter(row=>isNaN(General.valueToFloat(row.VRPERCENTUAL))).map(row=>{
            var name = widget.fields.filter(field => ((field.name.indexOf("NM") != -1)||(field.name.indexOf("DS") != -1)) && field.isVisible)[0].name;
            var code = widget.fields.filter(field => field.name.indexOf("CD") != -1 && field.isVisible)[0].name;
            return {
                DETAILCODE:row[code],
                NAME:row[name],
                VALUE:row.VRPERCENTUAL
            };
        });
        var field = widget.getField("VRPERCENTUAL");
        var fieldToCheckPercentualName = field.name;
        var nrItem = widget.getParent().currentRow.NRSEQITEMORCA;
        widget.currentRow[field.name] = isNaN(General.valueToFloat(widget.currentRow[field.name])) ?  General.valueToFloat(widget.currentRow[field.name]) : widget.currentRow[field.name] ;
        widget.currentRow.VRPERCENTUAL = General.valueToFloat(widget.currentRow.VRPERCENTUAL);
        
        if(widget.isValid()){// && gopUtil.validaPercentual(widget.currentRow.VRPERCENTUAL) && this.validaPercentualTotal(widget)){
            if(Util.isEmptyOrBlank(widget.getField(fieldToCheckPercentualName).value())|| widget.getField(fieldToCheckPercentualName).value()< 1000000 || !invalidRows.length){       
                General.setFloatAttrOnCurrentRow(widget.currentRow, fieldToCheckPercentualName);
                var fieldToCheckValue = widget.currentRow[fieldToCheckPercentualName];
                widget.currentRow[fieldToCheckPercentualName] = Util.isEmptyOrBlank(fieldToCheckValue) ? 0 : fieldToCheckValue;
                var soma = widget.dataSource.data.reduce((sum, row) => {
                    return row.__is_new || row.VRPERCENTUAL == widget.dataSource.data.VRPERCENTUAL ? sum : sum + General.valueToFloat(row.VRPERCENTUAL);
                }, 0);
                
                if(widget.currentRow.__is_new){
                    soma += General.valueToFloat(widget.currentRow.VRPERCENTUAL);
                }

                if(soma <= 100){
                    General.setFloatAttrOnCurrentRow(widget.currentRow, fieldToCheckPercentualName);
                    if(nrItem == 5 || nrItem == 6 || nrItem == 7){
                        var tpDespesa = "";
                        if(nrItem == 5){
                            tpDespesa = "MATRIZ";
                        }else if(nrItem == 6){
                            tpDespesa = "DIVERSA";
                        }else{
                            tpDespesa = "LANCENTRADA";
                        }
                        
                        widget.currentRow.IDDESPPREV = tpDespesa;
                    }
                    if(idGeralUnid == 1){
                        this.saveItemGeral(widget);
                    }else if(idGeralUnid == 2){
                        this.saveItemUnidade(widget);
                    }
                }else{
                    MessageService.showMessageByCode('PARAMETRO_SOMA_MAIOR_LIMITE', {"PARAMETRO": soma.toFixed(3).replace(".", ",")});
                }
            } else{
                if(invalidRows.length && widget.mode=='inline_edit'){
                    var mensagens = [];
                    invalidRows.forEach(function(reg){
                        reg.DETAILCODE = applyMaskProd(reg.DETAILCODE);
                        mensagens.push(reg.DETAILCODE + " - " + reg.NAME + ': "'+reg.VALUE+'"');
                    });
                    General.showErrorMessages(widget, mensagens, ScreenService.i18n('Percentuais Inválidos'));
                }
            }
        } 
    };
    
    this.saveItemGeral = function(widget){
        widget.currentRow.NRPERCITEMGOP = widget.getParent().currentRow.NRPERCITEMGOP;
        gop21000_percentual_por_item_operacional_Service.saveItemOrcamento(widget.currentRow).then(function(response){
            self.desabilitaWidget(widget, true);
            widget.currentRow.__is_new = false;
            widget.currentRow.NRPERCITEMDETALGOP = parseInt(response.dataset.data.NRPERCITEMDETALGOP, 10);
            gopUtil.underBlockWidget(widget.currentRow);
            widget.view();
            if(widget.template != '/widget/master_detail/grid.html'){
                widget.dataSource.data.push(widget.currentRow);    
            }
            ScreenService.successNotification("Registro salvo com sucesso.");
            widget.reload();
        }, function(retorno){
              ScreenService.showMessage(retorno.data.error);
        });
    };
    
    function applyMaskProd(value){
        let mask = {
            "type": "fix",
            "params": {
                "mask": getLocalVar('CDPICTPROD')
            }
        };
        return Util.applyMask(value, mask, maskEngine, ZhUtil);
    }
    
    this.saveItemUnidade = function(widget){
        widget.currentRow.NRPERCITFILGOP = widget.getParent().currentRow.NRPERCITFILGOP;
        widget.currentRow.CDFILIAL = widget.getParent().currentRow.CDFILIAL;
        gop21000_percentual_por_item_operacional_Service.saveItemOrcamentoUnidade(widget.currentRow).then(function(response){
            self.desabilitaWidget(widget, true);
            widget.currentRow.__is_new = false;
            widget.currentRow.NRPERCITDETALFILGOP = parseInt(response.dataset.data.NRPERCITDETALFILGOP, 10);
            gopUtil.underBlockWidget(widget);
            widget.view();
            if(widget.template != '/widget/master_detail/grid.html'){
                widget.dataSource.data.push(widget.currentRow);    
            }
            ScreenService.successNotification("Registro salvo com sucesso.");
            widget.reload();
        }, function(retorno){
              ScreenService.showMessage(retorno.data.error);
        });
    };
    
    this.updateItemGeral = function(widget, idGeralUnid){
        var editedRows = this.getChangedRows(widget);
        editedRows.forEach(function(row){
            row.VRPERCENTUAL = row.VRPERCENTUAL === "" ? 0 : row.VRPERCENTUAL;
        });
        if(widget.isValid()){
            var soma = 0;
            var percentuais = widget.dataSource.data.map(a=>General.valueToFloat(a.VRPERCENTUAL));
            
            percentuais.forEach(function(percent){
                soma += percent;
            });
            
            soma = General.valueToFloat(soma);
            
            if(soma <= 100){
            
                editedRows.forEach(function(row){
                    if(!Util.isEmptyOrBlank(row.VRPERCENTUAL)){
                        row.VRPERCENTUAL = General.valueToFloat(row.VRPERCENTUAL);
                    }
                    row.NRPERCITEMGOP = widget.getParent().currentRow.NRPERCITEMGOP;
                });
            
                gop21000_percentual_por_item_operacional_Service.updateItemGeral(editedRows).then(function(){
                    self.desabilitaWidget(widget, true); 
                    widget.list();
                    widget.getAction('DEL').isVisible = true;
                    widget.reload();
                    ScreenService.successNotification("Registro salvo com sucesso.");
                }, function(retorno){
                      ScreenService.showMessage(retorno.data.error);
                });
            }else{
                MessageService.showMessageByCode('SOMA_PERCENTUAL', {"PARAMETRO": soma});
            }
            widget.getParent().enabled = true;
        }
    };
    
    this.updateItemUnidade = function(widget, idGeralUnid){
        var editedRows = this.getChangedRows(widget);
        var itemWidget = gopUtil.getWidgetContainer("ORCAMENTO_UNIDADE_ITEM");
        editedRows.forEach(function(row){
            row.VRPERCENTUAL = row.VRPERCENTUAL === "" ? 0 : row.VRPERCENTUAL;
        });
        
        if(widget.isValid()){            
            var soma = 0;
            var percentuais = widget.dataSource.data.map(a=>General.valueToFloat(a.VRPERCENTUAL));
            
            percentuais.forEach(function(percent){
                soma += percent;
            });
            
            soma = General.valueToFloat(soma);
            
            if(soma <= 100){
                editedRows.forEach(function(row){
                    if(!Util.isEmptyOrBlank(row.VRPERCENTUAL)){
                        row.VRPERCENTUAL = General.valueToFloat(row.VRPERCENTUAL);
                    }
                    row.NRPERCITFILGOP = widget.getParent().currentRow.NRPERCITFILGOP;
                    row.CDFILIAL = widget.getParent().currentRow.CDFILIAL;
                });
                gop21000_percentual_por_item_operacional_Service.updateItemUnidade(editedRows).then(function(){
                    self.desabilitaWidget(widget, true); 
                    widget.list();
                    widget.getAction('DEL').isVisible = true;
                    widget.reload();
                    ScreenService.successNotification("Registro salvo com sucesso.");
                }, function(retorno){
                      ScreenService.showMessage(retorno.data.error);
                });
            }else{
                MessageService.showMessageByCode('SOMA_PERCENTUAL', {"PARAMETRO": soma});
            }
            itemWidget.enabled = true;
        }
    };
    
    this.saveMoDetalhe = function(widget, idGeralUnid){
        var nrItem = "";
        
        if(idGeralUnid == 1){
            nrItem = widget.getParent().currentRow.NRPERCITEMDETALGOP;
              widget.currentRow.NRPERCITEMDETALGOP = nrItem;
        }else{
            nrItem = widget.getParent().currentRow.NRPERCITDETALFILGOP;
            widget.currentRow.NRPERCITDETALFILGOP = nrItem;
            widget.currentRow.CDFILIAL = widget.getParent().getParent().currentRow.CDFILIAL;
        }
          
        widget.currentRow.CDDESPADMFAT = widget.getParent().currentRow.CDDESPADMFAT; 
          
        if(widget.isValid()){
            General.setFloatAttrOnCurrentRow(widget.currentRow, "VRPERCENTUAL");
            if(idGeralUnid == 1){
                  
                gop21000_percentual_por_item_operacional_Service.saveMo(widget.currentRow).then(function(){
                    this.desabilitaWidget(widget, true); 
                    if(widget.mode == "inline_edit"){
                        widget.list();
                    }else{
                        ScreenService.closeSwipe();
                    }
                    widget.reload();
                    ScreenService.successNotification("Operação realizada com sucesso");
                      
                }.bind(this), function(retorno){
                      ScreenService.showMessage(retorno.data.error);
                });
                
            }else if(idGeralUnid == 2){
                
                    gop21000_percentual_por_item_operacional_Service.saveMoUnidade(widget.currentRow).then(function(){
                    this.desabilitaWidget(widget, true); 
                    if(widget.mode == "inline_edit"){
                        widget.list();
                    }else{
                        ScreenService.closeSwipe();
                    }
                    widget.reload();
                    ScreenService.successNotification("Operação realizada com sucesso");
                      
                }.bind(this), function(retorno){
                      ScreenService.showMessage(retorno.data.error);
                });
            }
        }
    };
    
    this.saveInlineProdutoGeral = function(widget){
        var invalidRows = widget.dataSource.data.filter(row=>isNaN(General.valueToFloat(row.VRPERCENTUAL))).map(row=>{
            var name = widget.fields.filter(f => ((f.name.indexOf("NM") != -1)||(f.name.indexOf("DS") != -1)) && f.isVisible)[0].name;
            var code = widget.fields.filter(f => f.name.indexOf("CD") != -1 && f.isVisible)[0].name;
            return {
                DETAILCODE:row[code],
                NAME:row[name],
                VALUE:row.VRPERCENTUAL
            };
        });
        var field = widget.getField("VRPERCENTUAL");
        var fieldToCheckPercentualName = field.name;
        widget.currentRow[field.name] = General.valueToFloat(widget.currentRow[field.name]);
        widget.currentRow.VRPERCENTUAL = General.valueToFloat(widget.currentRow.VRPERCENTUAL);
        if(widget.isValid()){// && gopUtil.validaPercentual(widget.currentRow.VRPERCENTUAL) && this.validaPercentualTotal(widget)){
            if(Util.isEmptyOrBlank(widget.getField(fieldToCheckPercentualName).value())|| widget.getField(fieldToCheckPercentualName).value()< 1000000 && !invalidRows.length){       
                var editedRows = this.getChangedRows(widget);
                var newRows = [];
                var oldRows = [];
                var soma = 0;
                var percentuais = widget.dataSource.data.map(a=>General.valueToFloat(a.VRPERCENTUAL));
                widget.currentRow.VRPERCENTUAL = General.valueToFloat(widget.currentRow.VRPERCENTUAL);
                percentuais.forEach(function(percent){
                    soma += percent;
                });
                if(soma <= 100){
                    editedRows.forEach(function(row){
                        if(row.NRPERCDETALPRODGOP){
                            oldRows.push(row);
                        }else{
                            newRows.push(row);
                        }
                    });
                    if(!Util.isEmptyOrBlank(newRows)){
                        var nrItem = widget.getParent().currentRow.NRPERCITEMDETALGOP;
                        newRows.forEach(function(row){
                            row.NRPERCITEMDETALGOP = nrItem;
                            row.VRPERCENTUAL = General.valueToFloat(row.VRPERCENTUAL);
                            row.__is_new = true;
                        });
                        this.saveProdutoDetalheGeral(newRows, widget);
                    }
                    if(!Util.isEmptyOrBlank(oldRows)){
                        oldRows.forEach(function(row){
                            if(!Util.isEmptyOrBlank(row.VRPERCENTUAL)){
                                row.VRPERCENTUAL = General.valueToFloat(row.VRPERCENTUAL);
                            }
                            row.NRPERCITEMDETALGOP = widget.getParent().currentRow.NRPERCITEMDETALGOP;
                        });
                        
                        this.updateProdutoDetalheGeral(oldRows, widget);
                    }
                }else{
                    MessageService.showMessageByCode('SOMA_PERCENTUAL', {"PARAMETRO": soma});
                }
            }else{
                if(invalidRows.length && widget.mode=='inline_edit'){
                    var mensagens = [];
                    invalidRows.forEach(function(reg){
                        reg.DETAILCODE = applyMaskProd(reg.DETAILCODE);
                        mensagens.push(reg.DETAILCODE + " - " + reg.NAME + ': "'+reg.VALUE+'"');
                    });
                    General.showErrorMessages(widget, mensagens, ScreenService.i18n('Percentuais Inválidos'));
                }
            }
        }
        widget.getParent().enabled = true;
    };
    
    this.saveProdutoDetalheGeral = function(editedRows, widget){
        
        gop21000_percentual_por_item_operacional_Service.saveProd(editedRows).then(function(){
            widget.list();
            widget.reload();
            ScreenService.successNotification("Registro salvo com sucesso.");
        }.bind(this), function(retorno){
              ScreenService.showMessage(retorno.data.error);
        });
        
    };

    this.updateProdutoDetalheGeral = function(oldRows, widget){

        gop21000_percentual_por_item_operacional_Service.updateProd(oldRows).then(function(){
            widget.list();
            widget.reload();
            ScreenService.successNotification("Registro salvo com sucesso.");
        }.bind(this), function(retorno){
              ScreenService.showMessage(retorno.data.error);
        });
        
    };
    
    this.saveProdutoDetalheUnidade = function(editedRows, widget){
        
        gop21000_percentual_por_item_operacional_Service.saveProdUnidade(editedRows).then(function(){
            if(widget.getParent().getParent().currentRow.NRSEQITEMORCA == 3){
                this.setFiltroDespesa(widget);
            }else{
                this.setFiltroProduto(widget);
                this.hideDeleteButton(widget, false);
            }
            widget.list();
            widget.reload();
            ScreenService.successNotification("Registro salvo com sucesso.");
        }.bind(this), function(retorno){
              ScreenService.showMessage(retorno.data.error);
        }); 
        
    };
    
    this.saveInlineProdutoUnidade = function(widget){
        var invalidRows = widget.dataSource.data.filter(reg=>isNaN(General.valueToFloat(reg.VRPERCENTUAL))).map(reg=>{
            var name = widget.fields.filter(f => ((f.name.indexOf("NM") != -1)||(f.name.indexOf("DS") != -1)) && f.isVisible)[0].name;
            var code = widget.fields.filter(f => f.name.indexOf("CD") != -1 && f.isVisible)[0].name;
            return {
                DETAILCODE:reg[code],
                NAME:reg[name],
                VALUE:reg.VRPERCENTUAL
            };
        });
        var field = widget.getField("VRPERCENTUAL");
        var fieldToCheckPercentualName = field.name;
        widget.currentRow[field.name] = General.valueToFloat(widget.currentRow[field.name]);
        widget.currentRow.VRPERCENTUAL = General.valueToFloat(widget.currentRow.VRPERCENTUAL);
        if(widget.isValid()){// && gopUtil.validaPercentual(widget.currentRow.VRPERCENTUAL) && this.validaPercentualTotal(widget)){
            if(Util.isEmptyOrBlank(widget.getField(fieldToCheckPercentualName).value())|| widget.getField(fieldToCheckPercentualName).value() < 1000000 && !invalidRows.length){               
                    var editedRows = this.getChangedRows(widget);
                    var newRows = [];
                    var oldRows = [];
                    var soma = 0;
                    var percentuais = widget.dataSource.data.map(a=>General.valueToFloat(a.VRPERCENTUAL));
                    
                    percentuais.forEach(function(percent){
                        soma += percent;
                    });
            
                    if(soma <= 100){
                        
                        editedRows.forEach(function(row){
                            if(row.NRPERCDETPRODFILGOP){
                                oldRows.push(row);
                            }else{
                                newRows.push(row);
                            }
                        });
                        
                        let widgetOrcaDetUni = templateManager.container.getWidget('ORCAMENTO_DETALHAMENTO_UNIDADE');
                        
                        if(!Util.isEmptyOrBlank(newRows)){
                            var nrItem =  widget.getParent().currentRow.NRPERCITDETALFILGOP;
                            newRows.forEach(function(row){
                                row.NRPERCITDETALFILGOP = nrItem;
                                row.VRPERCENTUAL = General.valueToFloat(row.VRPERCENTUAL);
                                row.CDFILIAL = widget.getParent().getParent().currentRow.CDFILIAL;
                                row.CDFILIALMO = widgetOrcaDetUni.currentRow.CDFILIALMO;
                                row.__is_new = true;
                            });
                            this.saveProdutoDetalheUnidade(newRows, widget);
                        }
                        
                        if(!Util.isEmptyOrBlank(oldRows)){
                            oldRows.forEach(function(row){
                                if(!Util.isEmptyOrBlank(row.VRPERCENTUAL)){
                                    row.VRPERCENTUAL = General.valueToFloat(row.VRPERCENTUAL);
                                }
                                row.NRPERCITDETALFILGOP = widget.getParent().currentRow.NRPERCITDETALFILGOP;
                                row.CDFILIAL = widget.getParent().currentRow.CDFILIAL;
                                row.CDFILIALMO = widgetOrcaDetUni.currentRow.CDFILIALMO;
                            });
                            
                            this.updateProdutoDetalheUnidade(oldRows, widget);
                        }
                    }else{
                        MessageService.showMessageByCode('SOMA_PERCENTUAL', {"PARAMETRO": soma});
                    }
                }else{
                    if(invalidRows.length && widget.mode=='inline_edit'){
                        var mensagens = [];
                        invalidRows.forEach(function(reg){
                            reg.DETAILCODE = applyMaskProd(reg.DETAILCODE);
                            mensagens.push(reg.DETAILCODE + " - " + reg.NAME + ': "'+reg.VALUE+'"');
                        });
                        General.showErrorMessages(widget, mensagens, ScreenService.i18n('Percentuais Inválidos'));
                    }
            }
        }
        widget.getParent().enabled = true;
    };
    
    this.updateProdutoDetalheUnidade = function(editedRows, widget){
        gop21000_percentual_por_item_operacional_Service.updateProdUnidade(editedRows).then(function(){
                this.desabilitaWidget(widget, true);
                this.hideDeleteButton(widget, false);
                widget.list();
                widget.reload();
                ScreenService.successNotification("Registro salvo com sucesso.");
            }.bind(this), function(retorno){
                  ScreenService.showMessage(retorno.data.error);
            }); 
    };
    
    this.verificaInitUnidade = function(widget){
        
        if(Util.isEmptyObject(widget.dataSource.data) && !Util.isEmptyObject(widget.getParent().currentRow)){
            var unidade = widget.getParent().currentRow.CDFILIAL;
            
            gop21000_percentual_por_item_operacional_Service.verificaInitUnidade(unidade).then(function(){
                widget.reload();
            },function(error){
                ScreenService.showMessage("Erro ao inicializar a unidade. Tente novamente!" );
            });
        }
        
    };

    this.validaPercentualTotal = function(widget){
        var total = 0;
        widget.dataSource.data.forEach(function(row){
            total += Number(row.VRPERCENTUAL);
        });
        
        if(total > 100){
            var diferenca = (total - 100);
            MessageService.showMessageByCode('LIMITE_ULTRAPASSADO', {"PARAMETRO": diferenca});
            return false;
        }else{
            return true;
        }
    };
    
    this.validaPercentualTotalItem = function(widget){
        var total = 0;
        widget.dataSource.data.forEach(function(row){
            total += Number(row.VRPERCITEMGOP);
        });
        
        if(total > 100){
            var diferenca = (total - 100);
            MessageService.showMessageByCode('LIMITE_ULTRAPASSADO', {"PARAMETRO": diferenca});
            return false;
        }else{
            return true;
        }
    };
    
    this.voltar = function(widget){

       //this.subCountSwipe();
        ScreenService.closeSwipe();
        widget.unSelectAll();
        widget.list();
        widget.reload();
    };
    
    this.voltarDepreciacao = function(widget){
        
        ScreenService.closeSwipe();
        widget.list();
    };
    
    this.openPopupAutoOperation = function(){
        var autoOperation = gopUtil.getWidgetContainer("autoOperation");
        if(autoOperation.getField('NMFILIAL').value){
            autoOperation.getField('CDFILIAL_DESTINO').readOnly = false;
        }else{
            autoOperation.getField('CDFILIAL_DESTINO').readOnly = true;
        }
        ScreenService.openPopup(autoOperation);
    };
    
    this.cadAutoPercentUnidade = function(widget){
        ScreenService.confirmMessage("Ao realizar o cadastramento automático, os dados da unidade de destino serão apagados. Deseja prosseguir?").then(function (){
            if(widget.isValid()){
                gop21000_percentual_por_item_operacional_Service.cadAutoPercentUnidade(widget.currentRow.CDFILIAL,widget.currentRow.CDFILIAL_DESTINO).then(function(){
                    ScreenService.successNotification("Cadastramento efetuado com sucesso!" );
                    widget.currentRow.CDFILIAL = '';
                    widget.currentRow.NMFILIAL = '';
                    widget.currentRow.CDFILIAL_DESTINO = [];
                    ScreenService.closePopup();
                },function(error){
                    ScreenService.showMessage("Erro ao executar a operação. Tente novamente!");
                });
            }
        }, function (){
            
        });
        
    };
    

    /*
	 * Função que seta o field VRPERCENTUAL ou o que for mandado como 'campo' como inLineEditable sempre que 
	 * for a última widget, ou seja, quando não houverem mais "swipes" a serem 
	 * abertos além do da própria widget, para que a pessoa não tenha que abrir
	 * o swipe, clicar em editar, tudo isso para editar apenas um campo.
	 * OBS.: para o funcionamento correto, é necessário chamar a função que salva os 
	 * valores da widget atual, e adaptar a função para quando o widget.mode == "inline_edit"
	 * e chamar essa função no evento "WidgetOnSaveInlineEdit".
	 *
	 */
   
    this.setInLineEdit = function(widget, setFilter = false, campo = 'VRPERCENTUAL'){
        var flag = true;

        widget.widgets.forEach(function(childWidget){
            if(childWidget.isVisible){
                flag = false;
            }
        });
        
        if((Util.isEmptyOrBlank(widget.widgets) || flag) && widget.currentRow.NRSEQITEMORCA!=11){
            widget.template = "/widget/grid.html";
            widget.getField(campo).inlineEditable = true;
            // widget.showCheckbox = true;
            if(!setFilter) {
                widget.reload();
            }
        }else{
            widget.template = "/widget/master_detail/grid.html";
            widget.getField(campo).inlineEditable = false;
            // widget.showCheckbox = false;
        }
    };
   /*
    * Função que seta a propriedade 'inlineEditable' como true apenas
    * para a row com o NRPERCITEMGOP == 11. Faz isso retirando o 'inlineEdit'
    * das outras rows.
    */
    
    this.setInlineEditable = function(element, row){
        var widget = [];
        var orcamentoGeralWidget = gopUtil.getWidgetContainer('ORCAMENTO_GERAL');
        var orcamentoUnidadeWidget = gopUtil.getWidgetContainer('ORCAMENTO_UNIDADE_ITEM');
        
        if(orcamentoGeralWidget.isActive()){
            widget = orcamentoGeralWidget;
        }else{
            widget = orcamentoUnidadeWidget;
        }
        
        element.on("click", function(evt){
            if(widget.mode == 'inline_edit'){
                evt.stopPropagation(); 
            }
        });
        
        if( element.field &&  row.NRSEQITEMORCA != 11){
            if(element.context.children[1]){
                element.context.children[1].remove();
            }
        }
        
    };
    
    this.filterCadAuto = function(widget){
        var data = [
            { "name": "P_CDOPERADOR", "operator": "=", "value": getLocalVar('CDOPERADOR') },
            { "name": "P_IDCNTRRESC", "operator": "=", "value": "N" }
        ];
        
        if(!Util.isEmptyOrBlank(widget.currentRow.NMFILIAL)){
            widget.getField('CDFILIAL_DESTINO').readOnly = false;
        }else{
            widget.getField('CDFILIAL_DESTINO').readOnly = true;
        }
        
        if(!Util.isEmptyOrBlank(widget.currentRow.CDFILIAL)){
            data.push({ "name": "P_EXCDFILIAL", "operator": "=", "value": widget.currentRow.CDFILIAL });
        }else{
            data.push({ "name": "P_EXCDFILIAL", "operator": "=", "value": 'T'});
        }
        
        widget.getField('CDFILIAL_DESTINO').dataSourceFilter = data;
        widget.currentRow.CDFILIAL_DESTINO = "";
        widget.getField('CDFILIAL_DESTINO').reload();
    };
    
    this.setFilterProperties = function(widget, field){
        
        if(field.label != "Código" && field.name != "VRPERCENTUAL"){
            widget.__filterPopup.getField(field.name).isVisible = field.isVisible;
            widget.__filterPopup.getField(field.name).readOnlyOnEdit = !field.isVisible;
        }
    };

    this.setFiltroProduto = function(widget){
        var parent = widget.getParent().currentRow;
        
        this.setInLineEdit(widget, true);
        if(!Util.isEmptyOrBlank(widget.getParent().getParent().currentRow.CDFILIAL)){
            widget.dataSourceFilter = [
                {"name": "P_NRPERCITDETALFILGOP", "value": parent.NRPERCITDETALFILGOP , "operator": "="},
                {"name": "P_CDDESPPREV", "value": parent.CDDESPPREV, "operator": "="},
                {"name": "P_CDFILIAL", "value": widget.getParent().getParent().currentRow.CDFILIAL, "operator": "="}
            ];
        }else {
            widget.dataSourceFilter = [
                {"name": "P_NRPERCITEMDETALGOP", "value": parent.NRPERCITEMDETALGOP , "operator": "="},
                {"name": "P_CDDESPPREV", "value": parent.CDDESPPREV, "operator": "="},
            ];
        }
        widget.reload();
    };
    
    this.hideDeleteButton = function(widget, visibility){
        
        if(widget.mode == 'inline_edit' && widget.getAction('DEL')){
            widget.getAction('DEL').isVisible = !visibility;
        }
    };
    
    this.cancelInlineEdit = function(widget){
		widget.reload();
		widget.list();
		widget.editing = false;
        widget.getAction('DEL').isVisible = true;
        var itemWidget = gopUtil.getWidgetContainer(widget.getParent().name);
        itemWidget.enabled = true;
    };
    
    this.setZeroIfNull = function(args){
        var field = args.owner.field;
        if(Util.isEmptyOrBlank(field.value())){
            field.value(0);
        }
    };
    this.justInteger = function (selector, retryCount) {
		if (!retryCount || retryCount > 5)
			retryCount = 5;
		if (!selector) {
			throw {
				"message": "Selector is not defined!"
			};
		}
		setTimeout(function () {
			var elem = $(selector);
			if (elem[0]) {
				var length = selector.split(',').length;
				for (var i = 0; i < length; i++) {
					if (elem.hasOwnProperty(i)) {
						elem[i].onkeypress = avoidNonInteger;
						elem[i].onpaste = preventEvent;
					}
				}
			} else {
				if (!retryCount) {
					console.error("Selector not found!");
				} else {
					retryCount--;
					this.justInteger(selector, retryCount);
				}
			}
		}.bind(this), 500);
	};

	function avoidNonInteger(e) {
		var integerNumbers = [44, 46, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
		var numberCode = e.which;
		if (integerNumbers.indexOf(numberCode) < 0) {
			preventEvent(e);
		}
	}
	
	function preventEvent(e) {
		e.preventDefault();
	}
	
	/**
	 * Aplica 'avoidNonInteger' ao field clicado no inlineEdit.
	 * Evita a inserção de não inteiros.
	 * 
	 * @param {Args} args Objeto args.
	 */
	this.applyJustIntegerOnPercentual = function(args){
	   var widget = args.owner.field.widget;
	   if(widget.mode!='edit' && args.owner.context.children[1]){
	       args.owner[0].getElementsByTagName('div')[0].onkeypress = avoidNonInteger;
	   }
	};
	
	this.inlineDetalhamento = function(args){
        var widget = args.owner.field.widget;
        if(widget.mode == 'inline_edit'){
            widget.getParent().enabled = false;
            this.hideDeleteButton(widget, true);
        }
	};
	
	this.editDepreciacao = function(widget, flag){
	    if(event && event.target.className.indexOf('ng-pristine') > -1)
	        return;
	    if(widget.name == 'ORCAMENTO_GERAL' || widget.name == "ORCAMENTO_UNIDADE_ITEM")
	        gopUtil.getWidgetContainer("ORCAMENTO_UNIDADE").enabled = flag;
	    else
	        gopUtil.getWidgetContainer("ORCAMENTO_GERAL").enabled = flag;
    };
	
	this.cancelInLine = function(args){
	    var widget = args.owner;
	    widget.list();
	    widget.reload();
	    if(widget.name.indexOf("ORCAMENTO_DESPESA_PRODUTO") == -1){
	        gopUtil.getWidgetContainer("ORCAMENTO_UNIDADE").enabled = true;
	    }else{
	        if(widget.name.indexOf("UNIDADE") != -1)
	            gopUtil.getWidgetContainer("ORCAMENTO_DETALHAMENTO_UNIDADE").enabled = true;
	        else
	            gopUtil.getWidgetContainer("ORCAMENTO_DETALHAMENTO").enabled = true;
	    }
	    var itemWidget = gopUtil.getWidgetContainer(widget.getParent().name);
        itemWidget.enabled = true;
	};
	
	this.limitaValor = function(args, tamanho){
	    var valor = args.row.VRPERCITEMGOP_inline;
        valor = valor.toString();
        valor = valor.length;
	    if(valor == tamanho){
	        args.e.preventDefault();
	    }
	};
    this.cancelaAdd = function(args){
        var widget = args.owner.widget;
        widget.isVisible = false;
        this.desabilitaWidget(widget, true);
        General.backOrCancel(args);
        args.owner.widget.reload();
    };
    
    this.setFiltroDespesa = function(widget){
        var parent = widget.getParent().currentRow;
        
        // this.setInLineEdit(widget);
        if(!Util.isEmptyOrBlank(widget.getParent().getParent().currentRow.CDFILIAL)){
            widget.dataSourceFilter = [
                {"name": "P_NRPERCITDETALFILGOP", "value": parent.NRPERCITDETALFILGOP , "operator": "="},
                {"name": "P_CDORCAMENTOMO", "value": parent.CDORCAMENTOMO, "operator": "="},
                /*{"name": "P_CDFILIAL", "value": widget.getParent().getParent().currentRow.CDFILIAL, "operator": "="},*/
                {"name": "P_CDFILIALMO", "value": parent.CDFILIALMO, "operator": "="}
            ];
        }else {
            widget.dataSourceFilter = [
                {"name": "P_NRPERCITEMDETALGOP", "value": parent.NRPERCITEMDETALGOP , "operator": "="},
                {"name": "P_CDORCAMENTOMO", "value": parent.CDORCAMENTOMO, "operator": "="},
            ];
        }
        widget.reload();
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('gop21000_percentual_por_item_operacional_Controller', gop21000_percentual_por_item_operacional_Controller);
});

// FILE: ./assets/modules/parametro/assets/js/controllers/gop22000_percentual_de_correcaoController.js
function gop22000_percentual_de_correcaoController(General, ScreenService, templateManager, gopUtil, gop22000_percentual_de_correcao_Service) {

    this.setDataSource = function(widget){
        
        var currentYear = General.getCurrentDate('YYYY');
        var maior = parseInt(currentYear) + 50;
        var menor = parseInt(currentYear) - 10;
        var array = [];
        
        while(menor < maior){
            array.push({"name" : menor, "value": menor});
            menor++;
        }
        
        widget.getField('MESES').dataSource.data = gopUtil.getDataMensal();
        widget.getField('ANOS').dataSource.data = array;
    };    

    this.setCurrentDate = function(){
        var geralWidget             = gopUtil.getWidgetContainer('gop22000_percentual_de_correcao_geral_field');
        var filialWidget            = gopUtil.getWidgetContainer('gop22000_percentual_de_correcao_filial_field');
        
        geralWidget.currentRow.ANO  = General.getCurrentDate('YYYY');
        filialWidget.currentRow.ANO = General.getCurrentDate('YYYY');
    };
    

    this.openPopupCadAutomatico = function(widget){
        var cadastramentoAutomatico = templateManager.container.getWidget('gop00800_cadastramentoAutomatico');
        cadastramentoAutomatico.newRow();
        ScreenService.openPopup(cadastramentoAutomatico);
    };
    

    this.setAno = function(widget, flag){
        if(widget.widgets[0].mode != "inline_edit"){
            var filialWidget = gopUtil.getWidgetContainer('gop22000_percentual_de_correcao_filial');
            var geralWidget  = gopUtil.getWidgetContainer('gop22000_percentual_de_correcao_geral');
            var ano          = widget.currentRow.ANO;
            var filial       = widget.currentRow.CDFILIAL; 
            var nmFilial     = widget.currentRow.NMFILIAL;
            if(widget.isValid() && !Util.isEmptyOrBlank(ano) && ano > 1899 && ano < 3000){
                let ano = widget.fields[0].value();
                widget.currentRow.FILTERED_CDANO = ano;
                widget.currentRow.FILTERED_CDFILIAL = filial;
                widget.currentRow.FILTERED_NMFILIAL = nmFilial;
                
                if(filial && nmFilial){
                    setLocalVar('CDFILIAL',filial);
                    setLocalVar('NMFILIAL',nmFilial);
                }
                
                if(geralWidget.isActive()){
                    // geralWidget.getParent().currentRow.ANO       = General.getCurrentDate('YYYY');
                    // geralWidget.getParent().currentRow.CDFILIAL  = getLocalVar('CDFILIAL');
                    geralWidget.floatingControl.visibleModes = false;
                    geralWidget.dataSourceFilter[2].value = ano; 
                    geralWidget.reload();
                    // templateManager.updateTemplate();
                } 
                
                if(filialWidget.isActive()){
                    // filialWidget.getParent().currentRow.ANO      = General.getCurrentDate('YYYY');
                    filialWidget.floatingControl.visibleModes = false;
                    filialWidget.getParent().currentRow.CDFILIAL = getLocalVar('CDFILIAL');
                    filialWidget.getParent().currentRow.NMFILIAL = getLocalVar('NMFILIAL');
                    filialWidget.dataSourceFilter[2].value = ano; 
                    filialWidget.reload();
                    // templateManager.updateTemplate();
                }
                // $('body').scope().$apply();
                // templateManager.updateTemplate();
                
                if(flag =='G'){
                    this.setStripGeral(widget);
                    ScreenService.notificationMessage('Filtro realizado com sucesso', 'success');
                }else{
                    this.setStripUnidade(widget);
                    ScreenService.notificationMessage('Filtro realizado com sucesso', 'success');
                }
            }
        } else {
            ScreenService.showMessage("Não é possível filtrar pois a tabela está em modo de edição.");
        }    
    };
    
    
    this.prepareWidgetFilial = function(widget){
        var filialWidget = gopUtil.getWidgetContainer('gop22000_percentual_de_correcao_filial');
        
        if(!filialWidget.getParent().currentRow.NMFILIAL){
           filialWidget.getParent().currentRow.CDFILIAL = getLocalVar('CDFILIAL');
           filialWidget.getParent().currentRow.NMFILIAL = getLocalVar('NMFILIAL');
        }
        
        this.setStripUnidade(widget);
        widget.reload();
    };
    
    this.setBlankAno = function(widget, flag){
        var geralWidget = gopUtil.getWidgetContainer('gop22000_percentual_de_correcao_geral');
        var filialWidget = gopUtil.getWidgetContainer('gop22000_percentual_de_correcao_filial');
        
        if(Util.isEmptyOrBlank(widget.currentRow.ANO)){
            this.checkFiltro(widget, flag);  
        }
        
        geralWidget.list();
        filialWidget.list();
    };
    
    this.isValidPercent = function(widget){
        var flag       = true;
        var mensagem   = [];
        var headerMsg  = "";
        var editedRows = [];
        var meses      = "";
        var tipoAtual  = "";
        var stMes      = "";
        
        widget.dataSource.data.map(function(obj) {
            if(headerMsg ==""){
    	       headerMsg = ScreenService.i18n("O valor informado deve estar entre  -100 e 100. Gentileza verificar: ");
               mensagem.push(headerMsg+"\n");
            }
            
            if(meses){
                stMes = meses.length > 10? ScreenService.i18n("Meses") + ":  " : ScreenService.i18n("Mês")+ ":  ";
                mensagem.push(ScreenService.i18n("Item") + ": " + ScreenService.i18n(tipoAtual));
                mensagem.push(stMes+ meses + " \n" );
                meses = "";
            }
            
            return Object.keys(obj).map(function(chave) {
                if(chave =='IDTIPOORCAOP'){
                    var map = widget.getField(chave).dataSource.data;
                    // console.log(obj[chave]);
                    editedRows = map.filter(function(row) { 
                        if(obj[chave] == row.value){
                            return row.name;
                        }
                    });
                    tipoAtual = editedRows[0].name;
                }
                
                if( chave.indexOf('VRMES') != -1 && chave.indexOf('_inline') ==-1 && chave.indexOf('changed_')  == -1){
                    var valorMes = General.valueToFloat(obj[chave]);
                    
                    if(valorMes <-100 || valorMes > 100 ){
                        widget.fields.forEach(function(field){
                            if(field.name.indexOf(chave) != -1){
                               if (meses != ""){
                                    meses = meses +', ';
                               }
                        	   meses = meses +" " + field.label;
                            }
                        	flag = false;
                        });
                    }
                }
            });
        });
        
        if(meses && tipoAtual == 'Depreciação' ){
            stMes = meses.length > 10? ScreenService.i18n("Meses") + ":  " : ScreenService.i18n("Mês")+ ":  ";
            mensagem.push(ScreenService.i18n("Item") + ": " + ScreenService.i18n(tipoAtual));
            mensagem.push(stMes+ meses + " \n" );
            meses = "";
        }
        
        if(!flag){
            General.showErrorMessages(widget,mensagem, ScreenService.i18n('Favor verificar: '));
        }
        return flag;    
    };
    

    this.getNewChangedRows = function(widget){
        var dataWidget = widget.dataSource.data;
        var field = "NRPERCCORRECAOGOP";

        var editedRows = dataWidget.filter(function(row) {
                return Util.isEmptyOrBlank(row[field]) && Object.keys(row).filter(function(key) {
                    return key.indexOf('changed_') !== -1;
                }).length ;
            });
                
        return editedRows;
    };
    
    this.getOldChangedRows = function(widget){
        var dataWidget = widget.dataSource.data;
        var field = "NRPERCCORRECAOGOP";
        
        
        var editedRows = dataWidget.filter(function(row) {
                return !Util.isEmptyOrBlank(row[field]) && Object.keys(row).filter(function(key){
                    return key.indexOf('changed_') !== -1;
                }).length ;
            });
                
        return editedRows;
    };
    
    this.savePercentCorrecao = function(widget){
        
        if(this.isValidPercent(widget)){
            var dataNew = this.getNewChangedRows(widget);
            var dataOld = this.getOldChangedRows(widget);
            
            if(!Util.isEmptyOrBlank(dataNew)){
                dataNew.forEach(function(row){
                    row.VRMES01 = row.VRMES01 == undefined ? '0' : row.VRMES01;
                    row.VRMES02 = row.VRMES02 == undefined ? '0' : row.VRMES02;
                    row.VRMES03 = row.VRMES03 == undefined ? '0' : row.VRMES03;
                    row.VRMES04 = row.VRMES04 == undefined ? '0' : row.VRMES04;
                    row.VRMES05 = row.VRMES05 == undefined ? '0' : row.VRMES05;
                    row.VRMES06 = row.VRMES06 == undefined ? '0' : row.VRMES06;
                    row.VRMES07 = row.VRMES07 == undefined ? '0' : row.VRMES07;
                    row.VRMES08 = row.VRMES08 == undefined ? '0' : row.VRMES08;
                    row.VRMES09 = row.VRMES09 == undefined ? '0' : row.VRMES09;
                    row.VRMES10 = row.VRMES10 == undefined ? '0' : row.VRMES10;
                    row.VRMES11 = row.VRMES11 == undefined ? '0' : row.VRMES11;
                    row.VRMES12 = row.VRMES12 == undefined ? '0' : row.VRMES12;
                });
                gop22000_percentual_de_correcao_Service.saveFilial(dataNew).then(function(response){
                    widget.reload();
                    widget.list();
                    this.enabledTab();
                    ScreenService.notificationMessage('Registro salvo', 'success');
                }.bind(this));
            }
            
            if(!Util.isEmptyOrBlank(dataOld)){
                dataOld.forEach(function(row){
                    row.VRMES01 = row.VRMES01 == undefined ? '0' : row.VRMES01;
                    row.VRMES02 = row.VRMES02 == undefined ? '0' : row.VRMES02;
                    row.VRMES03 = row.VRMES03 == undefined ? '0' : row.VRMES03;
                    row.VRMES04 = row.VRMES04 == undefined ? '0' : row.VRMES04;
                    row.VRMES05 = row.VRMES05 == undefined ? '0' : row.VRMES05;
                    row.VRMES06 = row.VRMES06 == undefined ? '0' : row.VRMES06;
                    row.VRMES07 = row.VRMES07 == undefined ? '0' : row.VRMES07;
                    row.VRMES08 = row.VRMES08 == undefined ? '0' : row.VRMES08;
                    row.VRMES09 = row.VRMES09 == undefined ? '0' : row.VRMES09;
                    row.VRMES10 = row.VRMES10 == undefined ? '0' : row.VRMES10;
                    row.VRMES11 = row.VRMES11 == undefined ? '0' : row.VRMES11;
                    row.VRMES12 = row.VRMES12 == undefined ? '0' : row.VRMES12;
                });
                gop22000_percentual_de_correcao_Service.updateFilial(dataOld).then(function(response){
                    widget.reload();
                    widget.list();
                    this.enabledTab();
                    ScreenService.notificationMessage('Registro salvo', 'success');
                }.bind(this));
            }
        }
    };
    

    this.cadastramentoAutomaticoFilial = function(){
        var cadAutoWidget = templateManager.container.getWidget('gop00800_cadastramentoAutomatico');
        
        var filialWidget  = gopUtil.getWidgetContainer("gop22000_percentual_de_correcao_filial");
        var geralWidget   = gopUtil.getWidgetContainer("gop22000_percentual_de_correcao_geral");
        
        var percent       = cadAutoWidget.currentRow.PERCENTUALCORRECAO;
            percent       = General.valueToFloat(percent);
            cadAutoWidget.currentRow.PERCENTUALCORRECAO = percent;

            
        if(cadAutoWidget.isValid()){
            
            if (percent >=-100 && percent  <= 100 ){
        
                filialWidget.fields.forEach(function(fieldFino){
                    
                    var name = fieldFino.name;
                    if(name.indexOf('VRMES') !== -1){
                        
                        cadAutoWidget.currentRow[name] = null;
                    }
                });
            
                var meses = cadAutoWidget.currentRow.MESES;
                
                meses.forEach(function(mes){
                     var field = filialWidget.fields.filter(function(field){
                    	return field.name.indexOf(mes) !== -1;
                    });
                    
                    cadAutoWidget.currentRow[field[0].name] = cadAutoWidget.currentRow.PERCENTUALCORRECAO;
                });
                
                cadAutoWidget.currentRow.NRORG = getLocalVar('NRORG');
                gop22000_percentual_de_correcao_Service.updateFilialCadAuto([cadAutoWidget.currentRow]).then(function(response){
                    
                    filialWidget.list();
                    geralWidget.list();
                    ScreenService.closePopup();
                    if(filialWidget.dataSource.data[0] != undefined)
                        filialWidget.reload();
                    // geralWidget.reload();
                    
                });
                ScreenService.notificationMessage('Cadastro Automático realizado com sucesso', 'success');
            }else{
                
                ScreenService.showMessage("Percentual de correção padrão deve estar entre -100 e 100");   
                
            }
        }
    };
    
    this.setStripGeral = function(widget){
	 
		var stripeWid   = gopUtil.getWidgetContainer("widgetStripe");
		var textoStripe = "Ano: ";
        textoStripe += widget.currentRow.FILTERED_CDANO || '';
		stripeWid.getField("Label_Stripe").label = textoStripe;
	    stripeWid.getField("Unidade_Stripe").label = "";
	};
	
    this.setStripUnidade = function(widget){
		var stripeWid = gopUtil.getWidgetContainer("widgetStripe");
		var textoStripe = "Ano: ";
		var unidade = "Unidade: ";
// 		textoStripe += (widget.currentRow.ANO == undefined && widget.currentRow.NMFILIAL == undefined) || (widget.currentRow.NMFILIAL == "")  ? "  Unidade: " : widget.currentRow.FILTERED_CDANO + " Unidade: " + widget.currentRow.FILTERED_CDFILIAL + " - " + widget.currentRow.FILTERED_NMFILIAL;  
		textoStripe += (widget.currentRow.FILTERED_CDANO || '' );  
// 		textoStripe = textoStripe.replace("  ", "\n");
        unidade += (widget.currentRow.FILTERED_CDFILIAL || '') + " - " + (widget.currentRow.FILTERED_NMFILIAL || '');
		stripeWid.getField("Label_Stripe").label = textoStripe;
        stripeWid.getField("Unidade_Stripe").label = unidade;	 
	};
	
	this.verificaFilial = function(widget){
	    
	   // this.setAno(widget,'L');
	    gop22000_percentual_de_correcao_Service.verificaFilial([[widget.getParent().currentRow.CDFILIAL]]).then(function(response){
            var filialWidget = gopUtil.getWidgetContainer('gop22000_percentual_de_correcao_filial');
            if (response == 'S'){
                filialWidget.fields.forEach(function(field){
                    if(field.name != 'IDTIPOORCAOP'){
                        field.inlineEditable = false;
                    }
                });
            }else {
               filialWidget.fields.forEach(function(field){
                    if(field.name != 'IDTIPOORCAOP'){
                        field.inlineEditable = "number";
                    }
                });
            } 
	    });
	};
	
	this.disabledTab = function (){
	   // templateManager.container.getActiveWidget().fields.forEach(function(field){
    //     	if(field.name == 'ANO' || field.name == 'NMFILIAL'){
    //     		field.readOnly = true;
    //         }
    //     })
	    if(templateManager.container.getActiveWidget().name == "gop22000_percentual_de_correcao_geral"){
	        templateManager.container.getWidget("gop22000_percentual_de_correcao_filial_field").enabled = false;
	        templateManager.container.getWidget("gop22000_percentual_de_correcao_geral_field").fields.forEach(function(field){
            	if(field.name == 'ANO' || field.name == 'NMFILIAL' || field.name == 'BUTTON'){
            		field.readOnly = true;	
            	}
            });
	    } else if (templateManager.container.getActiveWidget().name == "gop22000_percentual_de_correcao_filial"){
	        templateManager.container.getWidget("gop22000_percentual_de_correcao_geral_field").enabled = false;
	        templateManager.container.getWidget("gop22000_percentual_de_correcao_filial_field").fields.forEach(function(field){
            	if(field.name == 'ANO' || field.name == 'NMFILIAL' || field.name == 'BUTTON'){
            		field.readOnly = true;	
            	}
	        });	
	    }        
	};
	
	this.enabledTab = function (){
	   // templateManager.container.getActiveWidget().fields.forEach(function(field){
    //     	if(field.name == 'ANO' || field.name == 'NMFILIAL' || field.name == 'BUTTON'){
    //     		field.readOnly = false;
    //         }
    //     })
	    templateManager.container.getActiveWidget().reload();
	    templateManager.container.getActiveWidget().list();
	    if(templateManager.container.getActiveWidget().name == "gop22000_percentual_de_correcao_geral"){
	        templateManager.container.getWidget("gop22000_percentual_de_correcao_filial_field").enabled = true;
	        templateManager.container.getWidget("gop22000_percentual_de_correcao_geral_field").fields.forEach(function(field){
            	if(field.name == 'ANO' || field.name == 'NMFILIAL' || field.name == 'BUTTON'){
            		field.readOnly = false;	
            	}
            });
	    } else if (templateManager.container.getActiveWidget().name == "gop22000_percentual_de_correcao_filial"){
	        templateManager.container.getWidget("gop22000_percentual_de_correcao_geral_field").enabled = true;
	        templateManager.container.getWidget("gop22000_percentual_de_correcao_filial_field").fields.forEach(function(field){
            	if(field.name == 'ANO' || field.name == 'NMFILIAL' || field.name == 'BUTTON'){
            		field.readOnly = false;	
            	}
	        });	
	    }        
	};
}

Configuration(function(ContextRegister) {
    ContextRegister.register('gop22000_percentual_de_correcaoController', gop22000_percentual_de_correcaoController);
});

// FILE: ./assets/templates/controllers/ger80200_templateController.js
function ger80200_templateController($scope, ScreenService, restEngine, metaDataFactory, templateManager){

	/* DESIRED BEHAVIOR:
	*  - There is a limit of two visible actions on the widget
	*	 - Normal fields should show label and value
	*  - Normal fields should have their width customized by the developer
	*  - Direct inputs should have the same width
	*  - Direct inputs should occupy only one row
	*  - Direct inputs are not allowed on smartphones due to it's small screen nature
	*  - When collapsed the widget should show only the first two fields that have the flag 'showWhenCollapsed'
	*  - When there are no fields with 'showWhenCollapsed' the deafult behavior is to show the first two fields 
	*/

	var self = this;

	this.__defineGetter__('widget', function() {
		return $scope.widget;
	});

	self.widget.directInput = self.widget.directInput && !Util.isPhone();
	$scope.fieldWidth = Math.max(1, parseInt(12 / self.widget.fields.length));

	function groupFieldsByRowSize(fields, property, maxSize) {
		if (fields) {
			var rows = [{
				fields: []
			}];
			var rowSize = 0;
			var colSize;
			for (var i = 0; i < fields.length; i++) {
				colSize = parseInt(fields[i][property], 10);
				if (colSize > maxSize) {
					colSize = maxSize;
					fields[i][property] = colSize;
				}

				rowSize += colSize;
				if (rowSize > maxSize || fields[i].breakline) {
					rows.push({
						fields: []
					});
					rowSize = colSize;
				}
				rows[rows.length - 1].fields.push(fields[i]);
			}
			return rows;
		}
	}

	function buildStripeFields(visibleFields) {
		var alwaysVisibleFields = visibleFields.filter(function(field) {
			return field.showWhenCollapsed;
		});

		var visibleRows = groupFieldsByRowSize(visibleFields, "class", 12);
		var alwaysVisibleRows = groupFieldsByRowSize(alwaysVisibleFields, "class", 12).slice(0,2);

		if(alwaysVisibleFields.length === 0) {
			alwaysVisibleRows = visibleRows.slice(0,2);
		}


		if (self.widget.collapsible) {

			$scope.collapsed = true;
			$scope.stripeRows = alwaysVisibleRows;
			$scope.shouldCollapse = visibleRows.length > 2;

			$scope.collapseRows = function() {
				$scope.collapsed = !$scope.collapsed;
				$scope.stripeRows = ($scope.collapsed) ? alwaysVisibleRows : visibleRows;
			};

		} else {
			$scope.stripeRows = visibleRows;
		}
	}

	$scope.$watchCollection(function() {
		return self.widget.fields.filter(function(field) {
			return field.isVisible && field.aclIsVisible;
		});
	}, function(fields) {
		buildStripeFields(fields);
	});
	
	this.changeButton = function(buttonName){
		var widgets = ["ger80200_geral", "ger80200_faturamento",
			"ger80200_consumo", "ger80200_estoque",
			"ger80200_despesa", "ger80200_deducao_receita"
		];
		
		widgets.forEach(function(widgetName){
		    $("#" + widgetName).css('background-color', '#d9534f');
		    $("#" + widgetName).css('color', 'white');
		});
		
		$("#" + buttonName).css('color', '#d9534f');
		$("#" + buttonName).css('background-color', '#f4d7d7');
	};
	
	this.setVisibleWidget = function(widgetName){
		var widget = self.widget.container.getWidget("ger80200_tabbedWidget");
		var filialWidget = templateManager.container.getWidget("listaFiliais");
		var currentWidget = self.widget.container.getWidget(widgetName);
        var cdfilial = Util.isEmptyOrBlank(filialWidget.currentRow.CDFILIAL) ? "P" : filialWidget.currentRow.CDFILIAL;
        
		widget.widgets.forEach(function(childWidget){
    		childWidget.isVisible = false;
		});
		widget.widgets.filter(wid => wid.menu == widgetName).forEach(function(widname){
			self.widget.container.getWidget(widname).isVisible = true;
		});
        currentWidget.dataSourceFilter = [
            { "name": "CDFILIAL", "operator": "=", "value": cdfilial}
        ];
        
        currentWidget.activate();
        currentWidget.reload().then(function(){
        	currentWidget.currentRow = currentWidget.dataSource.data[0];
        	this.setFiltro(currentWidget);
        }.bind(this));
	};
	
	this.openWidgetButton = function(widgetName) {
		var widget = self.widget.container.getActiveWidget();
		var tabbedWidget = templateManager.container.getWidget("ger80200_tabbedWidget");
		var geralWidget = templateManager.container.getWidget("ger80200_geral");
		
		if(!Util.isEmptyOrBlank(geralWidget.dataSource.data[0])){
			if(geralWidget.dataSource.data[0].IDUTPADRROPFIL == "S" && geralWidget.dataSource.data[0].CDFILIAL != "P"){
				widgetName = "ger80200_geral";
			}
			if((widget.mode != 'edit') && (geralWidget.dataSource.data[0].IDUTPADRROPFIL != "S") && !Util.isEmptyOrBlank(geralWidget.dataSource.data)){
				tabbedWidget.currentMenu = widgetName;
				this.changeButton(widgetName);
				if(!Util.isEmptyOrBlank(tabbedWidget.menusToOpen) && tabbedWidget.menusToOpen.includes(widgetName)){
					this.loadWidgets(widgetName);
				}else{
					this.setVisibleWidget(widgetName);
				}
			}
		}
	};
	
	this.loadWidgets = function(widgetName){
        try{
            restEngine.requestMetaData('gop_parametro#json/containers/ger80200_widgets_resultado_operacional.json').then(function(response){
                if(response.data){
                    var tabbedWidget = templateManager.container.getWidget("ger80200_tabbedWidget");
                    var menus = tabbedWidget.menusToOpen;
                    var widgets = response.data.widgets;
                	widgets.filter(widget => widget.menu == widgetName).forEach(function(widget){
                		this.factoryWidgets(widget, tabbedWidget);
                	}.bind(this));
                    tabbedWidget.menusToOpen.splice(menus.indexOf(widgetName), 1);
                    this.setVisibleWidget(widgetName);
                }
            }.bind(this));
        }
        catch(err){
            return 0;
        }
        
	};
	
	/**
    * Inicializa as widgets da widget e suas 
    * filhas recursivamente.
    * 
    * @param {object} widget widget de evoluÃ§Ã£o.
    * 
    */ 
    this.factoryWidgets = function(widget, tabbedWidget){
    	widget.parent = tabbedWidget;
		var factoredWidget = metaDataFactory.widgetFactory(widget, self.widget.container);
        
        this.factoryWidgetActions(factoredWidget);
        this.factoryWidgetFields(factoredWidget);

        tabbedWidget.widgets.push(factoredWidget);

    };
	
	/**
    * Inicializa as actions da widget e suas 
    * filhas recursivamente.
    * 
    * @param {object} widget widget de evoluÃ§Ã£o.
    * 
    */ 
    this.factoryWidgetFields = function(widget){
        if(widget.widgets){
            widget.widgets.forEach(function(inWidget){
                this.factoryWidgetFields(inWidget);
            }.bind(this));
        }
        widget.fields.forEach(function(field){
            metaDataFactory.fieldFactory(field, widget);
        });
    };
    
     /**
    * Inicializa os fields da widget e suas 
    * filhas recursivamente.
    * 
    * @param {object} widget widget de evolução.
    * 
    */ 
    this.factoryWidgetActions = function(widget){
        if(widget.widgets){
            widget.widgets.forEach(function(inWidget){
                this.factoryWidgetActions(inWidget);
            }.bind(this));
        }
        widget.actions.forEach(function(action){
            metaDataFactory.actionFactory(action, widget);
        });
    };
    
    this.setFiltro = function(widget){
    	var filialWidget = templateManager.container.getWidget("listaFiliais");
	   var cdfilial = Util.isEmptyOrBlank(filialWidget.currentRow.CDFILIAL) ? "P" : filialWidget.currentRow.CDFILIAL;
    	
    	switch (widget.name) {
    		case "ger80200_faturamento":
    			this.setFiltroFaturamento(widget, cdfilial);
    			break;
    		case "ger80200_consumo":
	    		this.setFiltroConsumo(widget, cdfilial);
    			break;
			case "ger80200_estoque":
				this.setFiltroEstoque(widget, cdfilial);
				break;
			case "ger80200_despesa":
				this.setFiltroDespesa(widget, cdfilial);
				break;
			case "ger80200_deducao_receita":
				this.setFiltroDeducaoReceita(widget, cdfilial);
				break;
    		default:
    			// code
    	}
    };
    
    this.setDataSourceFilter = function(widgetName, cdfilial){
    	var widget = templateManager.container.getWidget(widgetName);
        widget.dataSourceFilter.filter(p=>p.name == "P_CDFILIAL")[0].value = cdfilial;
        widget.reload();
    };
    
    //Faturamento
    
    this.defineComposicaoFaturamento = function(consumoFaturamento){ //CDFILIAL, IDFATUROP
        var faturamentoMedicaoWidget = templateManager.container.getWidget("ger80200_faturamento_medicao");
        var faturamentoBrutoWidget = templateManager.container.getWidget("ger80200_faturamento_bruto");
        var faturamentoImpostoLista = templateManager.container.getWidget("ger80200_faturamento_impostos_lista");

        faturamentoBrutoWidget.isVisible = (consumoFaturamento.indexOf("FS") != -1 || consumoFaturamento == "AB");
        faturamentoMedicaoWidget.isVisible = (consumoFaturamento.indexOf("FE") != -1 || consumoFaturamento == "AB");
        faturamentoImpostoLista.getField("IDTPIMPOSTOROP").isVisible = (consumoFaturamento == "AB");
    };
    
    this.consideraPercentualImposto = function(idperfixcalcimp){
        var impostoGridWidget = templateManager.container.getWidget("ger80200_faturamento_impostos_lista");
        impostoGridWidget.getField("VRPERFIXCALCIMP").isVisible = (idperfixcalcimp == "S");
    };
    
    this.consideraServico = function(idservfatrop){
        var servicoWidget = templateManager.container.getWidget("ger80200_medicao_servico");
        servicoWidget.isVisible = (idservfatrop != "S");
    };

    this.setFiltroFaturamento = function(widget, cdfilial){
       var faturamentoMedicaoGeralWidget = templateManager.container.getWidget('ger80200_faturamento_medicao_geral');
	    
       faturamentoMedicaoGeralWidget.currentRow = widget.dataSource.data[0];
    	
        var servicoWidget = templateManager.container.getWidget("ger80200_medicao_servico");
        servicoWidget.getField("NMSERVICO").dataSourceFilter.filter(p=>p.name == "P_CDFILIAL")[0].value = cdfilial;
        servicoWidget.dataSourceFilter.filter(p=>p.name == "P_CDFILIAL")[0].value = cdfilial;
        servicoWidget.getField("CDFILIAL").isVisible = (cdfilial == "P");
        servicoWidget.getField("NMFILIAL").isVisible = (cdfilial == "P");
        
        var detalhamentoWidget = templateManager.container.getWidget('ger80200_medicao_servico_detalhe');
        detalhamentoWidget.getField("NMSERVICO").dataSourceFilter.filter(p=>p.name == "P_CDFILIAL")[0].value = cdfilial;
        detalhamentoWidget.dataSourceFilter.filter(p=>p.name == "P_CDFILIAL")[0].value = cdfilial;
        
        detalhamentoWidget.getField("NMSERVICO").readOnly = (cdfilial == "P");
        detalhamentoWidget.getField("CDFILIAL" ).isVisible = (cdfilial == "P");
        detalhamentoWidget.getField("NMFILIAL" ).isVisible = (cdfilial == "P");
        
        var tipoOperWidget = templateManager.container.getWidget("ger80200_faturamento_tipo_operacao");
        tipoOperWidget.dataSourceFilter.filter(p=>p.name == "P_CDFILIAL")[0].value = cdfilial;
        
        var impostoGridWidget   = templateManager.container.getWidget("ger80200_faturamento_impostos_lista");
        impostoGridWidget.dataSourceFilter.filter(p=>p.name == "P_CDFILIAL")[0].value = cdfilial;
        impostoGridWidget.dataSourceFilter.filter(p=>p.name == "P_IDTPIMPOSTOROP")[0].value = (widget.currentRow.IDFATUROP == "AB") ? "FS_FE" : widget.currentRow.IDFATUROP;
        
        var tipoCtPagarWidget = templateManager.container.getWidget("ger80200_faturamento_tipoctpag");
        tipoCtPagarWidget.dataSourceFilter.filter(p=>p.name == "P_CDFILIAL")[0].value = cdfilial;
        
        this.defineComposicaoFaturamento(widget.currentRow.IDFATUROP);
        this.consideraPercentualImposto(widget.currentRow.IDPERFIXCALCIMP);
        this.consideraServico(widget.currentRow.IDSERVFATROP);
        
    };
    
    //Consumo

	this.setFields = function(widget, cdfilial){
        /*var bool = cdfilial == "P";
        
        widget.getField('CDFILIAL').isVisible = bool;
        widget.getField("NMFILIAL").isVisible = bool;
        widget.getField("NMSERVICO").fields[0].isVisible = bool;
        widget.getField('NMFILIAL').filterProperties.isVisible = bool;
        
        widget.getField('NMSERVICO').dataSourceFilter.filter(p=>p.name == "P_CDFILIAL")[0].value = cdfilial;
        widget.getField('NMSERVICO').reload();*/
	    
	};
	
	this.habilitaServico = function(row){
        var geralWidget = templateManager.container.getWidget('ger80200_consumo');
        var servicoWidget = templateManager.container.getWidget('ger80200_consumo_servico');

        if(Util.isEmptyOrBlank(geralWidget.currentRow)){
            servicoWidget.isVisible = (geralWidget.dataSource.data[0].IDSERVRETROP == "N");
        }else{
            servicoWidget.isVisible = (geralWidget[row].IDSERVRETROP == "N");
        }
        servicoWidget.reload();
    };
    
    this.setFiltroConsumo = function(widget, cdfilial){
        var widgets = ["ger80200_consumo_detalhamento_servico", "ger80200_consumo_servico", 
            "ger80200_consumo_lancEnt", "ger80200_consumo_lancSaida"
        ];
        
        widgets.forEach(function(widgetName){
            this.setDataSourceFilter(widgetName, cdfilial);
        }.bind(this));
            
        ["ger80200_consumo_detalhamento_servico", "ger80200_consumo_servico"].forEach(function(widgetName){
        	this.setFields(widget, cdfilial);
        }.bind(this));
        this.habilitaServico("currentRow");

	};
    
    //Estoque
    
    this.setFiltroEstoque = function(widget, cdfilial){
        var widgets = ["ger80200_estoque_impostos", "ger80200_estoque_entrada_tipo_operacao",
            "ger80200_estoque_entrada_nivel_produto", "ger80200_estoque_saida"
        ];
            
        widgets.forEach(function(widgetName){
            this.setDataSourceFilter(widgetName, cdfilial);
        }.bind(this));
	};
	
    //Despesa
    
    this.verificaExibicao = function(){
        var geralWidget = templateManager.container.getWidget("ger80200_despesa");
        
        this.exibeContasPagar(geralWidget);
        this.exibeLancamentoAjuste(geralWidget);
        this.exibeLancamentoBancario(geralWidget);
        this.itensLancamentoEntrada(geralWidget);
    };
    
    this.exibeWidgetByField = function(geralWidget, tabbedWidgetName, widgets){
        var tabbedWidget = templateManager.container.getWidget(tabbedWidgetName);
        var widgetsVisiveis;
        
        widgets.forEach(function(wid){
            var widget = templateManager.container.getWidget(wid.name);
            widget.isVisible = (geralWidget.currentRow[wid.field] == "S");
        });
        
        widgetsVisiveis = !Util.isEmptyOrBlank(tabbedWidget.widgets.filter(w=>w.isVisible == true)[0]);
        tabbedWidget.defaultTab = widgetsVisiveis ? tabbedWidget.widgets.filter(w=>w.isVisible == true)[0].name : "";
        tabbedWidget.isVisible = widgetsVisiveis;
    };
    
    this.exibeContasPagar = function(geralWidget){
        var widgetsContaPagar = [
            {name: "ger80200_despesa_diversas", field: "IDAPUCTPGROP"},
            {name: "ger80200_despesa_mao_obra", field: "IDAPUMOCTPGROP"},
            {name: "ger80200_despesa_investimentos", field: "IDAPURAINVCTPAGROP"}
        ];
        this.exibeWidgetByField(geralWidget, "ger80200_despesa_contas_a_pagar", widgetsContaPagar);
    };
    
    this.exibeLancamentoAjuste = function(geralWidget){
        var widgetsLancamentoAjuste = [
            {name: "ger80200_lancamento_de_ajuste_contas_a_pagar", field: "IDCONSLANCAJUSPAG"},
            {name: "ger80200_lancamento_de_ajuste_contas_a_receber", field: "IDCONSLANCAJUSREC"}
        ];
        this.exibeWidgetByField(geralWidget, "ger80200_lancamento_de_ajuste", widgetsLancamentoAjuste);
    };
    
    this.exibeLancamentoBancario = function(geralWidget){
        var widgetsLancamentoBancario = [
            {name: "ger80200_lancamento_bancario_credito", field: "IDCONSCREDBCO"},
            {name: "ger80200_lancamento_bancario_debito", field: "IDCONSDEBBCO"}
        ];
        this.exibeWidgetByField(geralWidget, "ger80200_lancamento_bancario", widgetsLancamentoBancario);
    };
    
    this.exibeItensLancamentoEntrada = function(geralWidget){
        var entradasWidget = templateManager.container.getWidget("ger80200_despesa_lancamentos_entrada");
        
        entradasWidget.dataSource.data = [];
        
        if(geralWidget.currentRow.IDDESPENTROP == "S"){
            entradasWidget.dataSource.data.push({ "TIPODESPESA":"Despesa", "IDTIPODESPESA": "DE"});
        }
        if(geralWidget.currentRow.IDAPURDESPENTRMO == "S"){
            entradasWidget.dataSource.data.push({ "TIPODESPESA":"Mão de Obra", "IDTIPODESPESA": "MO"});
        }
        entradasWidget.isVisible = !Util.isEmptyOrBlank(entradasWidget.dataSource.data);
    };
    
    this.enableFieldRateio = function(geralWidget){
        var despesa = geralWidget.getField("IDDESPENTROP");
        var maoObra = geralWidget.getField("IDAPURDESPENTRMO");
        var rateioDespesa = geralWidget.getField("IDCONSPERRATFIL");
        var rateioMaoObra = geralWidget.getField("IDCONSRATENTRMO");
        
        rateioDespesa.readOnly = (despesa.value() != 'S');
        rateioMaoObra.readOnly = (maoObra.value() != 'S');
        rateioDespesa.value(despesa.value() == "N" ? rateioDespesa.value("N") : rateioDespesa.value());
        rateioMaoObra.value(despesa.value() == "N" ? rateioDespesa.value("N") : rateioDespesa.value());
        
    };
    
    this.itensLancamentoEntrada = function(geralWidget){
        this.exibeItensLancamentoEntrada(geralWidget);
        this.enableFieldRateio(geralWidget);
    };
    
    this.setFiltroDespesa = function(widget, cdfilial){
        var widgets = ["ger80200_despesa_tp_operacao", "ger80200_despesa_nivel_produto", "ger80200_despesa_excecoes",
            "ger80200_despesa_tp_operacao_mo", "ger80200_despesa_nivel_produto_mo", "ger80200_despesa_excecoes_mo",
            
            "ger80200_despesa_diversas", "ger80200_despesa_mao_obra", "ger80200_despesa_investimentos",
            
            "ger80200_lancamento_de_ajuste_contas_a_pagar", "ger80200_lancamento_de_ajuste_contas_a_receber",
            "ger80200_lancamento_bancario_credito", "ger80200_lancamento_bancario_debito"
        ];
            
        widgets.forEach(function(widgetName){
            var widget = templateManager.container.getWidget(widgetName);
            widget.dataSourceFilter.filter(p=>p.name == "P_CDFILIAL")[0].value = cdfilial;
        }.bind(this));
            
        ["ger80200_despesa_excecoes", "ger80200_despesa_excecoes_mo"].forEach(function(widgetName){
            var widget = templateManager.container.getWidget(widgetName);
            widget.getField('NMPRODUTO').dataSourceFilter.filter(p=>p.name == "P_CDFILIAL")[0].value = cdfilial;
        });
        
        this.verificaExibicao();
    };
    
    //Deducao e Receitas
    
    this.habilitaNumeroNF = function(widget){
        widget = widget ? widget : templateManager.container.getWidget("ger80200_deducao_receita");
        
        if(widget.currentRow.IDEXIBEBENEFDEDU == 'N'){
            widget.getField("IDEXIBENUMNFDEDU").readOnly = true;
            widget.currentRow.IDEXIBENUMNFDEDU = "N";
        }else{
            widget.getField("IDEXIBENUMNFDEDU").readOnly = false;
        }
    };
    
    this.setFiltroDeducaoReceita = function(widget, cdfilial){
        var widgets = ["ger80200_ded_rec_desconto_venda", "ger80200_ded_rec_devolucoes_estorno"];
        
        widgets.forEach(function(widgetName){
            this.setDataSourceFilter(widgetName, cdfilial);
        }.bind(this));
        
        this.habilitaNumeroNF();
    };
}

// FILE: ./assets/templates/controllers/zRegisterGopComponents.js
var zRegisterGopComponents = function(decodeTemplateFieldService, eventAggregator) {
    eventAggregator.subscribe('beforeProjectStartup', function register() {
    
        window.templateUrls = window.templateUrls || [];
        window.templateUrls.push({
            "package": "gopTemplates",
            "baseUrl": "assets/templates/widget/",
            "zeedhi": false
        });
    });
    
    decodeTemplateFieldService.registerViewTemplate('gop-component#widget/gop-projecao-grid.html', 'gop#widget/gop-projecao-grid.html');
    eventAggregator.subscribe('beforeProjectStartup', function register() {
        decodeTemplateFieldService.registerViewTemplate('gop-component#widget/gop-dro-stripe-widget.html', 'gop#widget/gop-dro-stripe-widget.html');
    });
};

Configuration(function (ContextRegister, eventEngine) {
	ContextRegister.register('zRegisterGopComponents', zRegisterGopComponents);
});
